"""
The basic brain class. All brains derive from these classes.

(c) 2005, PyrobRobotics.org. Licenced under the GNU GPL.
"""

__author__ = "Douglas Blank <dblank@brynmawr.edu>"
__version__ = "$Revision: 1.34 $"

import threading, time, operator, copy
import pyrobot.gui.console

def avg(myList):
    """Returns the arithemetic average of a sequences of numbers."""
    if len(myList) < 1:
        raise ValueError, "avg() arg is an empty sequence"
    sum = reduce(operator.add, myList)
    return sum / float(len(myList))

def middle(myList):
    """Returns the middle (len/2) value of a sequences of numbers."""
    # If len == even, then next one
    if len(myList) < 1:
        raise ValueError, "middle() arg is an empty sequence"
    return myList[len(myList)/2]

def median(myList):
    """Returns the middle (len/s) value of a sorted copy of a sequence."""
    # If len == even, then next one
    if len(myList) < 1:
        raise ValueError, "median() arg is an empty sequence"
    tmpList = copy.copy(myList)
    tmpList.sort()
    return tmpList[len(tmpList)/2]

import os
import time

class Brain :
    """
    This is the main thread for running a robot controler (ie, a brain).
    """
    def __init__(self, name = 'brain', engine = 0, **kwargs):
        self.debug = 0
        self.devData = {}
        self.devData["stepCount"] = 0
        self.lastRun = time.time() # seconds
        self.id = name
        self.engine = engine
        if engine is not 0:
            self.robot = engine.robot
	self.thread = 0
        self.needToStop = 1
        self.needToQuit = 0
        self.needToStep = 0
        self.__alive = 0
        self.pauseTime = 0.1 # time to sleep() in main loop. 0.1 means brain step() runs at max 10/sec
        if self.robot != 0:
            self.robot.localize()
        self.profilePerformance = 0
        self.profileCount = 0
        self.setup(**kwargs)
        # start the thread:
        self.start()
        
    # wrappers here to talk to default robot:
    def set(self, path, value):
        return self.robot.set(path, value)
    def isAlive (self,):
        return self.__alive    
    def get(self, *args):
        return self.robot.get(*args)
    def move(self, *args):
        return self.robot.move(*args)
    def translate(self, *args):
        return self.robot.translate(*args)
    def rotate(self, *args):
        return self.robot.rotate(*args)
    def stop(self):
        return self.robot.stop()
    def startDevice(self, *args, **keywords):
        return self.robot.startDevice(*args, **keywords)
    def removeDevice(self, *args, **keywords):
        return self.robot.removeDevice(*args, **keywords)
    def update(self):
        return self.robot.update()
    def motors(self, *args):
        return self.robot.motors(*args)
    def getDevice(self, *args):
        return self.robot.getDevice(*args)
    def hasA(self, *args):
        return self.robot.hasA(*args)
    def requires(self, *args):
        return self.robot.requires(*args)

    def __getattr__(self, attr):
        """ Overides default get attribute to return devData if exists """
        # avoid calling self. as that will call this recursively!
        if attr in self.__dict__["devData"]:
            return self.__dict__["devData"][attr]
        elif attr in self.__dict__:
            return self.__dict__[attr]
        else:
            raise AttributeError, ("'<type %s>' object has no attribute '%s'" % (self.__class__.__name__, attr))

    def getAll(self, *args):
        return self.robot.getAll(*args)

    def _draw(self, options, renderer):
        pass

    def getRobot(self):
        return self.robot

    def getEngine(self):
        return self.engine

    def quit(self):
        self.needToStop = 0
        self.needToQuit = 1
        if self.engine and self.engine.gui:
            self.engine.gui.done = 1
    def start (self) : pass #self.run()
    def run(self):
        self.couldBeMoving = 0
        if True :
            count = 0
            if True :
                count += 1
                if count > 20:
                    return
            
            if self.needToQuit:
                return
            elif self.needToStep > 0:
                self.needToStep -= 1 #protectedvariable
                self.needToStop = 1 #will be picked up next pass
            elif self.needToStop:

                if self.couldBeMoving:
                    self.couldBeMoving = 0
                    self.robot.move(0, 0)
            self.robot.update()
            self.step()
            self.devData["stepCount"] += 1
            self.couldBeMoving = 1
            if self.profilePerformance == 2:
                self.profileCount += 1
                self.profileTotalTime += time.time() - self.lastRun
                if self.profileCount % 100 == 0:
                    print "Profile: brain running at %.3f steps/second" % (float(self.profileCount) / self.profileTotalTime)
                    self.profileTotalTime = 0.0
                    self.profileCount = 0
            if self.profilePerformance == 1:
                self.profilePerformance = 2
            self.lastRun = time.time() # seconds
            
    def pleaseQuit(self):
        self.needToQuit = 1
        
    def pleaseStep(self):
        count = 0
        if True :
            count += 1
            if count > 20:
                return
        self.needToStep += 1 #protected variable
        self.pleaseRun()
        
    def pleaseStop(self):
        self.needToStop = 1
        self.__alive = False
        
    def pleaseRun(self, callback = 0):
        if not self.isAlive():
            print "Brain thread is not alive but request to run was made."
            self.__alive = True
        self.needToStop = 0
        if callback != 0:
            callback()
		
    def step(self):
        print "need to override pyro.brain.Brain.step()."
        
    def callback (self, src, event, msg): 
        """Can be overriden by brain script, gets keystrokes and gui events"""
        pass

    def setup(self, **kwargs):
        """
        User init method
        """
        pass

    def activated (self):
        """called each time we activate the brain"""
        self.__alive = True
        pass

    def deactivated (self):
        """called each time we deactivate the brain"""
        self.__alive = False
        pass

    def makeWindow(self):
        pass

    def redraw(self):
        pass

    def destroy(self):
        """
        Method to override of you create objects.
        """
        pass
    
    def execute (self, msg) : return eval(msg)


