""" Base GUI Class for Python Robotics. This is used for the -g tty GUI. """

import os
import sys
import signal
import string
from pyrobot.system.version import version as version
from pyrobot.system import help, usage, about, file_exists

class TextWatcher:
   def __init__(self):
      self.list = []
   def watch(self, exp):
      self.list.append(exp)
   def unwatch(self, exp):
      self.list.remove(exp)
   def update(self, locals = None):
      res = ""
      if len(self.list) == 0: return res
      res = res + "\n" + "=== Pyrobot Expression Watcher: " + ("=" * 65-30)#[:65]
      if locals == None:
         locals = globals()
      for exp in self.list:
         try:
            value = str(eval(exp, locals))
         except Exception, e :
            value = "<Undefined got "+str(e)+">"
         res = res + "\n" + "   "+ str(exp) + "=>" + str( value )
      res = res + "\n" + "=" * 65
      return res

class BrainStem:
   """
   A stub used in the Pyrobot command evaluator to define "self"
   so that self.robot will work. Without this
   self is not defined. Only works when you have a robot, but
   no brain yet.
   """
   def __init__(self, robot = 0):
      self.robot = robot
   # wrappers here to talk to default robot:
   def move(self, *args):
      """Short-cut to call the robot's move method."""
      return self.robot.move(*args)
   def translate(self, *args):
      """Short-cut to call the robot's translate method."""
      return self.robot.translate(*args)
   def rotate(self, *args):
      """Short-cut to call the robot's rotate method."""
      return self.robot.rotate(*args)
   def stop(self):
      """Short-cut to call the robot's stop method."""
      return self.robot.stop()
   def startDevice(self, *args, **keywords):
      """Short-cut to call the robot's startDevice method."""
      return self.robot.startDevice(*args, **keywords)
   def removeDevice(self, *args, **keywords):
      """Short-cut to call the robot's removeDevice method."""
      return self.robot.removeDevice(*args, **keywords)
   def update(self):
      """Short-cut to call the robot's update method."""
      return self.robot.update()
   def motors(self, *args):
      """Short-cut to call the robot's motors method."""
      return self.robot.motors(*args)
   def getDevice(self, *args):
      """Short-cut to call the robot's getDevice method."""
      return self.robot.getDevice(*args)
   def hasA(self, *args):
      """Short-cut to call the robot's hasA method."""
      return self.robot.hasA(*args)
   def requires(self, *args):
      """Short-cut to call the robot's requires method."""
      return self.robot.requires(*args)
   
class gui:
   """
   This is the base class for a gui.
   """
   
   def __init__(self, name = 'abstract gui', options = {}, engine = 0):
      """
      Child classes should do initialization pertaining to the creation
      of the GUI in the constructor.
      """
      self.watcher = None
      self.triedToStop = 0
      self.alreadyCleanedUp = 0
      self.engine = engine
      self.engine.gui = self
      self.prevsighandler = signal.signal(signal.SIGINT, self.INThandler)
      self.history = []
      self.history_pointer = 0
      self.MAXHISTORY = 1000
      self.environment = {}
      self.environment["gui"] = self
      self._populateEnv()
      self.lastDir = {}
      if file_exists("/.pyrobothist"):
         fp = open("/.pyrobothist", "r")
         self.history = map( string.strip, fp.readlines())
         fp.close()
         self.history_pointer = len(self.history)

   def updateDeviceList(self, clear = 0, select = -1):
      pass
   
   def listCommandHistory(self, search = None):
      cnt = 1
      res = ""
      if search:
         res = "Command history containing '%s':" % search
      else:
         print "Command history:"
      for line in self.history:
         if search:
            if search in line:
               res = res + "%4d: %s\n" % (cnt, line)
         else:
            res = res + "%4d: %s\n" % (cnt, line)
         cnt += 1
      return res

   def addCommandHistory(self, command):
      if len(self.history) > 0:
         if command != self.history[ len(self.history) - 1]:
            self.history.append(command)
      else:
            self.history.append(command)
      self.history_pointer = len(self.history)

   def run(self, command = []):
      """
      Child classes should do the beef of what they do right here.
      """
      done = 0
      res = ""
      res = res + "\n" + "========================================================="
      res = res + "\n" +  "Pyrobot, Python Robotics, (c) 2005, D.S. Blank"
      res = res + "\n" +  "http://PyroRobotics.org"
      res = res + "\n" +  "Version " + version()
      res = res + "\n" +  "========================================================="
      while done is not 1:
         res = res + "\n" +  "pyrobot> ", 
         if len(command) > 0:
            res = res + "\n" +  command[0],
            retval = command[0].strip()
            command = command[1:]
         else:
            retval = sys.stdin.readline()
         res = res + "\n" +  ""
         if retval == '':
            done = 1
            continue
         pres = self.processCommand(retval)
         if pres <> None :
            done = 1
            res = res + "\n"
         else : res = res + "\n" + pres
      return res

   def makeWatcher(self):
      """ Text-based watcher """
      self.watcher = TextWatcher()
      
   def watch(self, exp):
      if self.watcher == None:
         self.makeWatcher()
      self.watcher.watch(exp)

   def setCurrentConfig(self, config):
      raise "setCurrentConfig not supported"
      cwd = os.getcwd()
      if self.engine.simfile:
         file = self.engine.simfile
         if os.environ["PYROBOT"] + "/plugins/" in file or \
                cwd in file:
            file = file.split(os.path.sep)[-1]
         config.put("simulator", "file", file)
      if self.engine.worldfile:
         file = self.engine.worldfile
         if os.environ["PYROBOT"] + "/plugins/" in file or \
                cwd in file:
            file = file.split(os.path.sep)[-1]
         config.put("world", "file", file)
      if self.engine.robotfile:
         file = self.engine.robotfile
         if os.environ["PYROBOT"] + "/plugins/" in file or \
                cwd in file:
            file = file.split(os.path.sep)[-1]
         config.put("robot", "file", file)
      if self.engine.brainfile:
         file = self.engine.brainfile
         if os.environ["PYROBOT"] + "/plugins/" in file or \
                cwd in file:
            file = file.split(os.path.sep)[-1]
         config.put("brain", "file", file)

   def printCommandLine(self):
      raise "printCommandLine not supported"
      commandLine = sys.argv[0] + " "
      if self.engine.simfile:
         file = self.engine.simfile
         if os.environ["PYROBOT"] + "/plugins/" in file:
            file = file.split(os.path.sep)[-1]
         commandLine += "-s %s " % file
      if self.engine.worldfile:
         file = self.engine.worldfile
         if os.environ["PYROBOT"] + "/plugins/" in file:
            file = file.split(os.path.sep)[-1]
         commandLine += "-w %s " % file
      if self.engine.robotfile:
         file = self.engine.robotfile
         if os.environ["PYROBOT"] + "/plugins/" in file:
            file = file.split(os.path.sep)[-1]
         commandLine += "-r %s " % file
      if self.engine.brainfile:
         file = self.engine.brainfile
         if os.environ["PYROBOT"] + "/plugins/" in file:
            file = file.split(os.path.sep)[-1]
         commandLine += "-b %s " % file
      print commandLine

   def _populateEnv(self):
      if self.engine.brain:
         self.environment["self"] = self.engine.brain
      else:
         self.environment["self"] = BrainStem(self.engine.robot)
      self.environment["engine"] = self.engine
      self.environment["robot"] = self.engine.robot
      self.environment["brain"] = self.engine.brain

   def processCommand(self, retval):
      retval = retval.replace("\n", "")
      retval = retval.replace("\r", "")
      retval = retval.strip()
      res = ""
      if retval == "":
         if self.watcher <> None : 
            self._populateEnv ()
            res = res + "\n" + self.watcher.update(self.environment)
            return res
      self.addCommandHistory(retval)
      # Macro-style substitutions here:
      if len(retval)>= 1 and retval[0] == ".":
         if len(retval) >= 2:
            if retval[1].isalpha():
               retval = "self" + retval
         else:
            retval = "self" + retval
      # Now process the command, case-like:
      if retval == "run":
         res = res + "\n" + self.inform("Running in engine")
         self.engine.pleaseRun() # pass in callback, or not
         # self.engine.pleaseRun(self.redraw) # pass in callback
      elif retval == "runtillquit":
         self.done = 0
         self.engine.pleaseRun()
         res = res + "\n" + self.inform("Running in engine")
         return res
      elif retval == "step":
         self.stepEngine()
      elif retval == "info":
         res = res + "\n" +  "-------------------------------------------------------------"
         res = res + "\n" +  "Brain file:\t%s" % self.engine.brainfile
         res = res + "\n" +  "Brain:\t\t%s" % self.engine.brain
         res = res + "\n" +  "Robot:\t\t%s" % self.engine.robot
         res = res + "\n" +  "World:\t\t%s" % self.engine.worldfile
         res = res + "\n" +  "-------------------------------------------------------------"
         return res
      elif retval == "help":
         res = res + "\n" + self.help()
      elif retval == "usage" or retval == "manual" :
         res = res + "\n" + self.usage()
      elif retval == "update":
         if self.engine.robot != 0:
            self.engine.robot.update()
            res = res + "\n" + self.inform("Done!")
         else:
            self.inform("Define a robot first!")
      elif len(retval) >= 1 and retval[0] == "$":
         return str(os.system(retval[1:]))
      elif len(retval) >= 1 and retval[0] == "!":
         if retval == "!":
            res = res + "\n" + self.listCommandHistory()
         elif retval == "!!":
            res = res + "\n" + self.processCommand(self.history[-2]) # -1 is !!
         elif "-" in retval:
            start, stop = retval[1:].split("-")
            start, stop = start.strip(), stop.strip()
            if start == "": # neg number
               res = res + "\n" + self.processCommand(self.history[-int(stop) - 1])
            else:  
               for i in range(int(start), int(stop) + 1):
                  res = res + "\n" + self.processCommand(self.history[i - 1])
         else:
            val = retval[1:]
            if val.strip().isdigit():
               res = res + "\n" + self.processCommand(self.history[int(val) - 1])
            else:
               self.listCommandHistory(val)               
      elif retval == "about":
         res = res + "\n" + self.about()
      elif retval == "reload" or  retval == "reset" or retval == "restart" :
         res = res + "\n" + str(self.engine.reset())
      elif retval == "load robot":
         res = res + "\n" + "load robot command not supported" #str(self.loadRobot())
      elif retval == "load brain":
         res = res + "\n" + str(self.loadBrain())
      elif retval == "load simulator" or retval == "load server":
         res = res + "\n" + "load simulator / server command not supported" #self.loadSim()
      elif retval == "stop" or retval == "quit" or retval == "exit" or retval == "bye":
         res = res + "\n" + str(self.engine.pleaseStop())
         res = res + "\n" + self.inform("Stopped!")
      elif retval == "edit":
         if self.engine.brainfile != '':
            if os.getenv("EDITOR"): 
               editor = os.getenv("EDITOR")
            else:
               editor = "notepad"
            os.system("%s %s" % (editor, self.engine.brainfile))
            res = res + "\n" + self.inform("Reloading...")
            self.engine.reset()
         else:
            res = res + "\n" + self.inform("Need to load a brain first!")
      elif retval[:8] == "unwatch ":
         if self.watcher <> None: self.watcher.unwatch(retval[7:].strip())
      elif retval[:5] == "view ":
         res = res + "\n" + self.objectBrowser(retval[5:])
      elif retval[:6] == "watch ":
         self._populateEnv()
         self.watch(retval[5:].strip())
      elif retval[:7] == "browse ":
         res = res + "\n" + self.objectBrowser(retval[7:].strip())
      else:
         # elif len(retval) > 0 and retval[0] == "!":
         exp1 = """_retval = """ + string.strip(retval)
         _retval = "error"
         exp2 = string.strip(retval)
         # perhaps could do these once, but could change:
         self._populateEnv()
         res = res + "\n" + ">>> "
         res = res + retval
         try:
            _retval = eval(exp2, self.environment)
         except:
            try:
               exec exp1 in self.environment
            except:
               try:
                  exec exp2 in self.environment
               except:
                  res = res + self.formatExceptionInfo()
               else:
                  res = res + "\n" + "Ok"
            else:
              res = res + "\n" + "Ok"
         else:
            if _retval != None:
               res = res + "\n" + str(_retval)
      self.updateDeviceList()
      if self.watcher <> None : res = res + "\n" + self.watcher.update()
      return res
   def __formatTraceStack (self) :
      import sys, traceback
      try:
         etype, value, tb = sys.exc_info()
         traceback.print_exception(etype, value, tb, limit, sys.stderr)
      finally:
         etype = value = tb = None
      
   def formatExceptionInfo(self, maxTBlevel=10):
      import sys, traceback
      res = ""
      cla, exc, trbk = sys.exc_info()
      #res = res + "\n" + "ERROR:", cla, exc
      res = res + "\n" + str(traceback.format_exc())
      if type(exc) == type(""):
         excName = exc   # one our fake, string exceptions
      elif type(cla) == type(""):
         excName = cla
      else:
         if "__dict__" in dir(cla) and  cla.__dict__.get("__name__") != None:
            excName = cla.__name__  # a real exception object
         else:
            excName = cla   # one our fake, string exceptions
      if "__dict__" in dir(exc):
         excArgs = ("",)
         if exc.__dict__.has_key("args") : 
            excArgs = exc.__dict__["args"]
      else:
         excArgs = ("",)
      excTb = traceback.format_tb(trbk, maxTBlevel)
      # FIX: This is only the errors back four lines!
      # how do you get them before that?
      #for err in excTb:
      #   print err
      res = res + "\n" + "%s: %s %s" % (excName, excArgs[0], "in command line")
      return res

   def redraw(self):
      # FIX: this is way awkward:
      f = GenericStream()
      r = StreamRenderer(f)
      self.draw({}, r) # get data from robot, other things
      f.close()
      s = StreamTranslator(f, TTYRenderer())
      s.process()
      f.close()

   def _draw(self,options,renderer):
      """
      If the gui need draw something itself it should go here.
      """
      #render world
      #renderer.xformPush()
      renderer.color((1, 1, 1))
      renderer.rectangle((-10, -10, 0), (10, -10, 0), (10, 10, 0))
      #renderer.xformPop()
      #print "Redraw gui..."

   def makeMenu(self,name,commands):
      """ Could bind a key right here ^1, ^2, ^3..."""
      pass

   def fileloaddialog(self, type, skel, olddir = ''):
      """ Read a line from user """
      print "\n%s Filename: " % type,
      retval =  sys.stdin.readline()
      retval = retval.replace("\n", "")
      retval = retval.replace("\r", "")
      if file_exists(retval):
         return retval
      elif file_exists(olddir +"/" + retval):
         return olddir +"/" + retval
      else:
         # hope this is in the path!
         return retval

   def cleanup(self):
      if not self.alreadyCleanedUp:
         self.alreadyCleanedUp = 1
         self.done = 1
         try:
            sys.stdout = self.sysstdout
            sys.stderr = self.sysstderr
         except:
            pass
         if self.engine != 0:
            self.engine.shutdown()
         try:
            fp = open("/.pyrobothist", "w")
            line_count = min( len(self.history), self.MAXHISTORY)
            for i in range( line_count ):
               fp.write( self.history[ len(self.history) - line_count + i] +"\n" )
            fp.close()
         except:
            pass
         try:
             sys.stdout.flush()
         except IOError:
             pass
         sys.exit(1)
      else:
         os._exit(1)

   def stepEngine(self):
      self.engine.pleaseStep()
      return "\n" + self.inform("Step done!")

   def runEngine(self):
      self.engine.pleaseRun()
      return "\n" + self.inform("Running...")

   def stopEngine(self): # stop!
      self.engine.pleaseStop()
      return "\n" + self.inform("Stopped!")

   def stopTranslate(self):
      self.engine.robot._moveDir('ST')

   def stopRotate(self):
      self.engine.robot._moveDir('SR')

   def stepForward(self):
      self.engine.robot._moveDir('F')

   def stepBack(self):
      self.engine.robot._moveDir('B')

   def stepLeft(self):
      self.engine.robot._moveDir('L')

   def stepRight(self):
      self.engine.robot._moveDir('R')

   def resetEngine(self):
      self.engine.reset() 
      
   def loadBrain(self):
      f = self.fileloaddialog("brains","*.py", self.lastDir.get("brain", ''))
      if f != '' and f != 0:
         self.lastDir["brain"] = string.join(f.split('/')[:-1],'/')
         self.freeBrain()
         self.engine.loadBrain(f)
         self._populateEnv()

   def loadDevice(self):
      f = self.fileloaddialog("devices","*.py",self.lastDir.get("devices",''))
      if f != '' and f != 0:
         self.lastDir["devices"] = string.join(f.split('/')[:-1],'/')
         if self.engine != 0 and self.engine.robot != 0:
            self.engine.robot.startDevices(f)

   def freeBrain(self):
      self.engine.pleaseStop()
      self.engine.destroyBrain()
      self.engine.freeBrain()
      self.engine.brainfile = ''

   def loadSim(self, worldfile = ''):
      pyropath = os.getenv('PYROBOT')
      f = self.fileloaddialog("simulators","*",self.lastDir.get("sim", ''))
      if f != '' and f != 0:
         self.lastDir["sim"] = string.join(f.split('/')[:-1],'/')
         if worldfile == '':
            simulatorName = f.split('/')[-1]
            if simulatorName[-6:] == "Server":
               configDirName = simulatorName[:-6]
               worldfile = self.fileloaddialog("configs","*.cfg",
                                               self.lastDir.get("%s-config" % simulatorName,
                                                                "%s/plugins/configs/%s/" %
                                                                (pyropath, configDirName)))
               if worldfile == "":
                  return
               self.lastDir["%s-config" % simulatorName] = string.join(worldfile.split('/')[:-1],'/')
            else:
               # ends with "Simulator"
               simDirName = simulatorName[:-9]
               if simulatorName == "PyrobotSimulator":
                  worldfile = self.fileloaddialog("worlds","*.py",
                                                  self.lastDir.get("%s-world" % simulatorName,
                                                                   "%s/plugins/worlds/%s/" %
                                                                   (pyropath, simDirName)))
               elif simulatorName == "StageSimulator":
                  worldfile = self.fileloaddialog("worlds","*.cfg",
                                                  self.lastDir.get("%s-world" % simulatorName,
                                                                   "%s/plugins/worlds/%s/" %
                                                                   (pyropath, simDirName)))
               else:
                  worldfile = self.fileloaddialog("worlds","*.world",
                                                  self.lastDir.get("%s-world" % simulatorName,
                                                                   "%s/plugins/worlds/%s/" %
                                                                   (pyropath, simDirName)))
               if worldfile == "" or worldfile == 0:
                  return
               self.lastDir["%s-world" % simulatorName] = string.join(worldfile.split('/')[:-1],'/')
         else:
            simulatorName = worldfile
            self.lastDir["%s-world" % simulatorName] = string.join(worldfile.split('/')[:-1],'/')
         self.engine.worldfile = worldfile
         self.engine.simfile = f
         pyroPID = os.getpid()
         if os.name in ['nt', 'dos', 'os2'] :
            # FIXME: this assumes program to run is a python program; how will we know?
            # could leave out "python" and windows will ask
            print "start python %s %d %s" % (f, pyroPID, worldfile)
            os.system("start python \"%s\" \"%d\" \"%s\"" % (f, pyroPID, worldfile))
         elif os.name in ['posix']:
            os.system(f + (" %d " % pyroPID) + worldfile + " &")
         else:
            raise AttributeError, "your OS (%s) is not supported" % os.name
         
   def loadRobot(self):
      f = self.fileloaddialog("robots","*.py", self.lastDir.get("robot", ''))
      if f != '' and f != 0:
         self.lastDir["robot"] = string.join(f.split('/')[:-1],'/')
         self.freeBrain()
         self.freeRobot()
         self.engine.loadRobot(f)
         #if self.engine.robot:
         #   for device in self.engine.robot.builtinDevices:
         #      self.menuButtons["Built-in Devices"].add_command(label=device,command=lambda:self.startDevice(device))

   def freeRobot(self):
      self.freeBrain()
      self.engine.robotfile = ''
      self.engine.freeRobot()

   def INThandler(self, signum, frame):
      print "STOP ----------------------------------------------------"
      self.triedToStop += 1
      if self.triedToStop > 1:
         os.system("killall -9 pyrobot")
      self.engine.pleaseStop()
      self.cleanup()

   def inform(self, message):
      return message
      
   def filesavedialog(self, type, skel, startdir = ''):
      """ Read a line from user """
      print "\nFilename: ",
      retval =  sys.stdin.readline()
      retval = retval.replace("\n", "")
      retval = retval.replace("\r", "")
      return retval

   def newBrain(self):
      """
      Tk gui overrides this method.
      """
      pass
