__version__ = "$Revision: 1.17 $"
__source__ = "$Source: /cvsroot/game-client-system/scripts/SystemHandler.py,v $"
__author__ = "Kjartan A Jonsson"
__copyright__ = "Copyright (c) 2007 aGameCompany"
__license__ = "aGameCompany"

import game
import fileIO

__gameinstance = []
__registered = {}   #holds if we have registered the callback and if it is enabled using key pair <str:bool>
  
class SystemHandlerError(Exception): pass
class SystemKeysError (SystemHandlerError) : pass
class SystemTimerError (SystemHandlerError) : pass
class SystemNetworkError (SystemHandlerError) : pass
class SystemIntersectionError (SystemHandlerError) : pass
class SystemFileError (SystemHandlerError) : pass

class GameBase :
    """Is the game base framework that the main game class must extend. Must be registered using call registerGameClass(...)"""
    __name__ = "GameBase"
    def setup (self) : 
        """First thing called after registration"""
        pass
    def destroy (self) : 
        """First thing called when game system is destroyed"""
        pass
    def keyCallback (self, bkeyDown, strKey) : 
        """The function called when the user presses or releases keyboard buttons after the key is registered through keyEnabled(...).
        When bKeyDown equals false then the user released the button. strKey is the string representation of the key in question.
        """
        pass
    def networkCallback (self, event, data) : 
        """The function called when the system gets network package from an external resource. Is enabled after a call to networkConnect(...).
        Event is an id (not used) and data is the a string that was receaved.
        """        
        pass
    def intersectionCallback (self, objectName, event, sourceStateTuple) : 
        """
        objectName is the name of the entity who detected the instersection. event is
        what actully happened and sourceTargetTuple is an array of all the objects that
        are currently intersecting this movable object. 
        For instance is may get this movableObjectName -> robot_1, status -> Intersecting, 
        sourceStateTuple -> ["robot_2","true"] for an intersecting robot_2 intersecting with
        robot_1.        
        """
        pass
    def timerCallback (self, trap, message) : 
        """The function called when a user requested a callback through funciton timerCallBack(...).
        trap is the trap id sent through the timerCallBack call and message is the associated string.
        """
        pass    
    def systemCallback (self, event, message):
        """
        This callback will be called every time the system wants to say something. Messages expected are:
        Event 10 means that we are loading something. Message is formated as a dictionary {'name':'Page_0_1', 'precent':51.32}
        Event 11 means that we are unloading something. Message is formated as a dictionary {'name':'Page_0_1', 'precent':21}
        """
        pass
    def resourceCallback (self):
        """
        This callback will be called every time the system loads or unloads something (note must be notified by the system to turn up here).
        """
        print "I am getting callled by system, from resoruce callback"
        
##########################################################################################################    
#
#    Keys
#
##########################################################################################################         
__keys = {}
def __callKeyCallback (notused, bkeyDown, strKey) :
    """
    Is the callback that is called by the system when user registeres a key by calling keyEnabled
    """
    if (len(__gameinstance) == 1) :
        __gameinstance[0].keyCallback(bkeyDown, strKey)
def __cleanUpKeys () :
    """Sets the callback string to NULL disabling the callback after removing all the keys"""
    keysLeft = []
    for k in __keys.keys() : 
        keysLeft = keyEnabled(k, False)
        del __keys[k]
    if keysLeft <> [] : raise SystemKeysError, "Key cleaning up failed due to python versus core bookeeping missmatch"
    game.gameEventCreateGameEventHandlerInputCallback ("")
def keyEnabled (key, b=True) : 
    """
    Adds a key to be catched by our callback handler where key is a string. Refer to constants module for available keys. 
    Raises SystemKeysError on failure.
    """
    __checkError()
    keyEnabled = 0
    if b: keyEnabled = 1    
    if not __registered.has_key("keys") :
        __registered["keys"] = game.gameEventCreateGameEventHandlerInputCallback ("__callKeyCallback")
        game.systemRegisterHandler (__registered["keys"], 2) 
    if __registered["keys"] <> None :
        registeredKeys = game.gameEventRegisterGameEventHandlerInputCallbackKey (__registered["keys"], key, keyEnabled)
        if registeredKeys <> None: 
            __keys[key] = b
            if registeredKeys == "" : return []
            else : return registeredKeys.split(";")
        else : raise SystemKeysError, "Key named '" + str(key) + "' is not a valid key"
    else : raise SystemKeysError, "Could not register input callback handler successfully"

##########################################################################################################    
#
#    Network
#
##########################################################################################################

import network
def __callNetworkCallback (type, event, data) :
    if (len(__gameinstance) == 1) :
        if type <> 105 : raise SystemNetworkError, "Callback notified but not with type network"
        __gameinstance[0].networkCallback(event, data)

def networkConnect (port, strIp=None) : 
    """
    Connects this application to port 'port' at ip 'strIp' having the ip as string. We will get all network callbacks and messages through the class 
    that extends the GameBase class. The method that receaves the call is networkCallback(...)
    """
    __checkError()
    if not __registered.has_key("network") :
        if game.registerSystemString (105, "__callNetworkCallback") == 0 : raise SystemNetworkError, "Could not register network callback function"
        __registered["network"] = False
    if __registered["network"] == None : raise SystemNetworkError, "Trying to connect to a computer while already connected to one"
    if strIp == None : strIp = ""
    emsg = network.connect (port, strIp)
    __registered["network"] = emsg
    if emsg <> None : 
        raise SystemNetworkError, "Could not make connection got error :'" + emsg + "'"

def networkSend (str) : 
    """
    If networkConnect was successfull then this call sends str to connected computer. Raises SystemNetworkError on failure.
    Currently disabled and not supported.
    """
    return None
    __checkError()
    connectionNoneMsg = "Trying to send string to a computer before connecting to one"
    if not __registered.has_key("network") : raise SystemNetworkError, connectionNoneMsg
    else : 
        if __registered["network"] <> None : raise SystemNetworkError, connectionNoneMsg
        else : network.send(str)

def networkSession (account, controler) : 
    """
    Set the network account and session keys
    """
    network.session(account, controler)

    
##########################################################################################################    
#
#    Intersection
#
##########################################################################################################
__subscribersIntersectionDetection = {} #collection of objects interested in intersection detection
def __callIntersectionCallback (movableObjectName, status, sourceStateTuple) :
    """
    Internal helper function to forward intersection data to callback method.
    movableObjectName is the name of the entity who detected the instersection. Status is
    what actully happened and sourceTargetTuple is an array of all the objects that
    are currently intersecting this movable object. 
    For instance is may get this movableObjectName -> robot_1, status -> Intersecting, 
    sourceStateTuple -> ["robot_2","true"] for an intersecting robot_2 intersecting with
    robot_1.
    """
    if (len(__gameinstance)) :
        __gameinstance[0].intersectionCallback(movableObjectName, status, sourceStateTuple)

def addIntersectionDetection (movableObjectName) :
    """
    Adds the movableobjectname to intersection detection. This object will be called back when ever another object
    intersects it. The other object must exist in the list as well. Raises SystemHandlerError if we have not created
    the base class or SystemIntersectionError if we could not add it to the system intersection list.
    """
    __checkError ()
    if not __registered.has_key("intersection") :
        __registered["intersection"] = game.gameEventCreateGameEventHandlerIntersectionCallback ("__callIntersectionCallback")
    if game.gameEventHandlerIntersectionCallbackSubscribe (__registered["intersection"], movableObjectName) == None : raise SystemIntersectionError, "Could not register this movable object make sure that that it exists in the system";
    __subscribersIntersectionDetection[movableObjectName] = True #just remember the name, we use dic for convinence sake

def delIntersectionDetection (movableObjectName) :
    """
    Removes the movableobjectname from the list interested in intersection detection. Raises SystemHandlerError if we have not created
    the base class or SystemIntersectionError if we could not remove from the system intersection list.
    """
    __checkError ()
    if __registered.has_key("intersection") :
        if game.gameEventHandlerIntersectionCallbackUnSubscribe (__registered["intersection"], movableObjectName) == None : raise SystemIntersectionError, "Could not remove '"+movableObjectName+"' movable object from intersectio dectection list be sure that that it exists in the system";
        del __subscribersIntersectionDetection[movableObjectName] #we use dic for convinence sake

def __cleanUpIntersectionDetection () :
    """
    Removes the objects we have registered to and destroys the event handler.
    """
    pass
#    if __registered.has_key("intersection") :
#        for object in __subscribersIntersectionDetection.keys() : 
#            if game.gameEventHandlerIntersectionCallbackUnSubscribe (__registered["intersection"], object) == None : raise SystemIntersectionError, "Could not clean up '"+object+"' movable object make sure that that it exists in the system";
#        del __subscribersIntersectionDetection #we use dic for convinence sake
#            
#        game.gameEventDestroyGameEventHandler (__registered["intersection"])
#        del __registered["intersection"]

##########################################################################################################    
#
#    Timer
#
##########################################################################################################
def __callTimerCallback (event, trap, arg) :
    if (len(__gameinstance) == 1) :
       __gameinstance[0].timerCallback(trap, arg)
def __cleanUpTimer():
    if game.registerSystemString (101, "__callTimerCallback") == 0: raise SystemTimerError, "Timer callback function could not be unregistered"
def timerCallBack (msec, trap=0, message = "") :
    """Timer request to be notified after msec with trap and message. The callback will be catched by the class that was registered using 
    SystemManager.registerGameClass(...) and the method timerCallback(...) will be called.
    """
    __checkError()
    timercallinit = __registered.has_key("timer")
    if not __registered.has_key("timer") :
       if game.registerSystemString (101, "__callTimerCallback") == 1: 
           __registered["timer"] = None #register that we returned with success
           timercallinit = True
       else : raise SystemTimerError, "Timer callback function could not be registered into game system"
    if timercallinit : 
        if msec <= 0 : raise SystemTimerError, "Timer does not except negative milliseconds as callback time"
        if message == "" : message = "None"
        if game.systemCallback (msec, trap, str(message)) == 0 :
            raise SystemTimerError, "Timer request to be called back failed with parameters " + "msec:"+str(msec)+" trap:"+str(trap)+" message:'"+str(message)+"'"

##########################################################################################################    
#
#    GUI
#
##########################################################################################################
       
def receaveMenuEvent (caller, event, param) :
    """
    This will call the overrided receaveMenuEvent function made by implementer
    """
    print str(param)
    getGameClass().receaveMenuEvent (caller, event, param)


##########################################################################################################    
#
#    SystemHandler
#
##########################################################################################################
def __callSystemCallback (type, event, msg):
    if (len(__gameinstance)) :
       __gameinstance[0].systemCallback(event, msg)

def __checkError () :
    if (len(__gameinstance) == 0) : raise SystemHandlerError, "Game class extended from GameFramework.Base was not registered with GameFramework.registerGameClass"

def _cleanUp ():
    if (len(__gameinstance) == 1) : 
        __gameinstance[0].destroy()
    __cleanUpKeys()
    __cleanUpTimer()
    __cleanUpIntersectionDetection()
    if len(__gameinstance) == 1 : 
        del __gameinstance[0]
    for k in __registered.keys() : 
        del __registered[k]
    
def cleanUpAndExit () :
    _cleanUp()
    game.systemExit() 
    
def registerGameClass (c) :
    """
    Registers a class that extends the GameBase class. If no game class is registered then it will register c and call c.setup() initalizing the call.
    """
    if game.registerSystemString (104, "__callSystemCallback") == 0 : 
        raise SystemHandlerError, "Could not register system callback function"        
    
    if (len(__gameinstance) == 0) :
        if (c.__name__ == "GameBase") : __gameinstance.append(c)
        #call the setup function that should be overriden by the programmer
        c.setup()
    else : raise SystemHandlerError, "Can not register more that one game class"

def getGameClass() : 
    if (len(__gameinstance) == 1) : return __gameinstance[0]

##########################################################################################################    
#
#    File IO
#
##########################################################################################################    

class FileIO :
    __f = None
    def __init__ (self, file, option="r") :
        """Opens a file and appends it to the open file collection on success
					if failed we return string describing error. On success we return NULL string and we added the file to the file collection
					Options are 
						w: will allow write and read on a file removing current content 
						r: will allow read only on a file 
						a: will append to current file
						rw:will write into current content allowing reading as well"""
        self.__file = file
        res = fileIO.fileOpen (self.__file, option)
        if res <> None : raise SystemFileError, "Could not open file '"+self.__file+"' with options '"+option+"' file may be open. Got:" + str(res)
        
    def write (self, content) :
        """Write to a file"""
        res = fileIO.fileWrite (self.__file, content)
        if res <> None : raise SystemFileError, "Could not write to file '"+self.__file+"'"
        
    def read (self) :
        """Read a file"""
        res = fileIO.fileRead (self.__file)
        if res == None : raise SystemFileError, "Could not read file '"+self.__file+"'."
        return res
        
    def close (self) :
        """Closes a file and removes it from the open file collection"""
        res = fileIO.fileClose (self.__file)
        if res <> None : raise SystemFileError, "Could not close file '"+self.__file+"' file may be in use or already closed got: " + str(res)
