"""
Sound library wrapper

Notes on how animation and materials (collisions) sounds work:
We create and define sounds using the sound objects. And we can assign it as Material and/or Animation. 
If this is done we will play the sound depending on what was assigned. If we assign it as Animation we will play the sound when the animation is started. 
If it was defined as Material we will play it as default sound on collision between objects. 
A Material can also be assigned more than one sound and is done though a  call where we define two Materials for each sound. 
This will be played when these two objects collide with each other.

Some notes on Animation:
When playing an animation it is best to define the sound object to loop if we have an looping animation. Also if we 
do not stop the current animation the sound will not stop. This means that before changing the animation we need to stop
the current playing animation so the sound stops correctly. This is handled currently by entity.setAnimation. It calls
stop animation if we have played one before.

"""
__version__ = "$Revision: 1.4 $"
__source__ = "$Source: /cvsroot/game-client-system/scripts/sound.py,v $"
__author__ = "Kjartan A Jonsson"
__copyright__ = "Copyright (c) 2007 aGameCompany"
__license__ = "aGameCompany"

import gameSound

from constants import *

class SoundError (Exception) : pass

class Sound :

	sceneNode = None
	__name = None
	__file = None
	
	def __init__ (self, name, file, loop) :
		"""where id is the id of our camera, if camera id is 0 then we are accessing a new instance of default camera"""
		if gameSound.createObject (name, file, loop) == False : raise SoundError, " could not create sound. '"+name+"' was probably not unique or sound files is not in the resource system."
		self.__name = name
		self.__file = file
	
	def getName (self) : return self.__name
	
	def getFile (self) : return self.__file
		
	def attachToSceneNode (self, sceneNode) :
		self.sceneNode = sceneNode
		gameSound.attachSceneNode (self.__name, sceneNode.id)
		
	def play (self) :
		gameSound.play(self.__name)

	def stop (self) : 
		gameSound.stop(self.__name)

	def setCone (self, innerAnge, outerAngle, outerGain) :
		gameSound.setCone(self.__name, innerAnge, outerAngle, outerGain)
		
	def setDistance (self, referenceRad, maxRad) :
		gameSound.setDistance (self.__name, referenceRad, maxRad)

	def setGain (self, gain, gmin, gmax) :
		gameSound.setGain (self.__name, gain, gmin, gmax)

	def setPitch (self, pitch) :
		gameSound.setPitch (self.__name, pitch)

	def setAnimationSound (self, entity, animationName) :
		"""
		Will play this sound with specified sound settings when animtion is played through the python interface
		"""
		r = gameSound.setAnimationSound (self.__name, entity.id, animationName)
		if r <> None : raise SoundError, "Animation '"+animationName+"' sound for '"+entity.id+"' could not be set because: " + str(r)
		
	def setMaterialSound (self, materialA, materialB = None) :
		"""
		Will play this sound with specified sound settings when materialA collides with materialB.
		if materialB is not set we use this sound as the default collision sound for materialA.
		"""
		if materialB <> None : r = gameSound.setMaterialSound (self.__name, materialA, materialB)
		else : r = gameSound.setMaterialSound (self.__name, materialA)
		if r <> None : raise SoundError, "Material sound could not be set because " + str(r)
		
	def destroy (self) :
		gameSound.removeObject(self.__name)
