"""
"""
__author__ = "Kjartan A Jonsson"
__version__ = "$Revision: 1.4 $"
__copyright__ = "Copyright (c) 2007 aGameCompany"
__license__ = "aGameCompany"

import xmlrpclib
import settings

__machines = {}
class Machine :
    """
    Has many machine modules and is controlled by an Controller
    """
    devices = {} #is a collection of devices that this machine uses
    control = None #this is the controller handling
    parts = {}
    name = ""
    id = ""
    __machine_base = None #is the base device, can be a rover, hover, pillar, walker, ... anything that moves the robots base position
    
    def __init__ (self) :
        self.devices = {} #is a collection of devices that this machine uses
        self.control = None #this is the controller handling
        self.parts = {}
        self.id = ""
        self.name = ""        
    
    def setControl (self, c) : 
        """
        Sets the controller of this machine. The controler must be of class Controller and 
        can have different handling dependent on implementation:
        - user controlled
        - ai controller
        - network controlled
        If c is None the machine control will be disabled. If a controller is defined we
        will desable that old contoller and set the new one.
        """
        if c == None : 
            if self.control <> None : 
                self.control.giveControl()
                self.control = c
                return
        else :
            if self.control <> None : self.control.giveControl()
            self.control = c
            self.control.takeControl(self)

    def hasDevice (self, deviceName):
        """
        Returns True or False.
        """
        if deviceName in self.parts.keys() : return True
        return False
    
    def getDevices (self) : return self.parts
    
    def getDevice (self, deviceName):
        if deviceName in self.parts.keys() : return self.parts[deviceName]
    
    def getSize (self) : return 1
    
    def addDevice (self, name, device) :
        """
        Add a machine device to the device collection that makes this machine.
        Currently support 'vehicle'
        Future support includes
        - sensors
        - range
        - pose
        - target (for gravity gun, ....)
        """
        self.parts[name] = device
        if device.machine_group == "vehicle" or device.machine_group == "walker"   or device.machine_group == "footing"  : #register all we know for device
            self.__robot = device
            self.__machine_base = self.__robot
            
    def getBase (self) : return self.__machine_base
    def _setBase (self, base) : self.__machine_base = base
            
    def callback (self, source, event, arguments):
        for partid in self.parts :
            self.parts[partid].callback(source, event, arguments)
        
def add (name, machine, nicename=None, isowner=True) :
    """
    Will append the machine into the collection
    """
    if not isowner : return None #since we are not the owner dont store it
    __machines[name] = machine
    __machines[name].id = name
    if nicename == None : __machines[name].name = __machines[name].id
    else : __machines[name].name = nicename

def get (name) :
    """
    Will return machine by name. If name does not 
    exist we will return None
    """
    if name in __machines.keys() : return __machines[name]

def getMachines () :
    """
    Returns a dictionary of all the machines with name data relationship.
    """
    return __machines

def getMachineProfile (account) :
    """
    Will get machine profile from online server with three attempts to retreve it.
    """
    __rpc_machine = xmlrpclib.ServerProxy( settings.SERV_STATE_MACHINE, transport=settings.proxy )
    try : return __rpc_machine.getMachine( account )
    except :
        print "Getting machine profiles failed during first attempt. Trying second time."
        try : return __rpc_machine.getMachine( account )
        except :
            print "Getting machine profiles failed during first attempt. Trying third time."
            try : return __rpc_machine.getMachine( account )
            except : print "Getting machine profiles failed in all three attempts. Network connection to "+settings.SERV_STATE_MACHINE+" failed."
    return None

class TestMachine  :
    
    def testSetControl (self):
        """
        """
        pass
    