"""
VehicleManager handles everything related to an vehicle. It creates, destroys (static and dynamic)
instances of the vehicle. It manages changing parts and the clients vehicle parts that he currently
has available to him.
This class does not handle:
- purchasing
- exchangin parts with other players
- sponsoring system
- vehicle parts (resource) information
"""
__author__ = "Kjartan A Jonsson"
__version__ = "$Revision: 1.10 $"
__copyright__ = "Copyright (c) 2007 aGameCompany"
__license__ = "aGameCompany"

import xmlrpclib
import PhysicsManager
import SceneManager
#import DriverManager
import SystemHandler
import settings

__vehicleDic = None
__vehicleList = []
__vehiclePartType = ['axis', 'suspension', 'chassi', 'engine', 'tire', 'brake', 'steer'] #all of our supported types
__vehiclePart = ['axis front', 'suspension front', 'axis rear', 'suspension rear', 'tire front', 'tire rear', 'chassi', 'engine', 'brake', 'steer'] #all of our supported types

class VehicleManagerError ( Exception ): pass

def __loadVehicleFromServer( playerName, vehicleName ):
    """
    Will retreave the player 'playerName' vehicles information. Used to update internal state and data.
    """
    #global __vehicleDic
    #__rpc = xmlrpclib.ServerProxy( settings.SERV_STATE_MACHINE, transport=settings.proxy  )
    #res = __rpc.getVehicle( playerName, vehicleName )
    try : res = settings.cache ("vehicle_"+playerName+"_"+vehicleName)
    except Exception : pass
    if res == None : raise VehicleManagerError, "Request was bad, got None from server. Probably because user '" + playerName + "' is not valid"
    else : __vehicleDic = res
    return __vehicleDic
        
def __loadVehicle ( playerName, vehicleName, instanceName ):
    """
    Creates a vehicle using information from dictionary 'dic'. If stickerEnabled is true then we will use sticker else we use default 
    material. Returns vehicle on success but raises VehicleManagerError on failure
    """
    __vehicleDic = __loadVehicleFromServer( playerName, vehicleName )
    if __vehicleDic == None : raise VehicleManagerError, "Could not load vehicle '"+vehicleName+"', got error from server"
    if not __vehicleDic.has_key ( vehicleName ) : raise VehicleManagerError, "Could not load vehicle '"+vehicleName+"', vehicle not owned by user or does not exist"
    
    vsetup = __vehicleDic[vehicleName] #lets simplify our queries
    v = PhysicsManager.addCar (instanceName)
    #v.setPosition (0, 0, 0)

    v.setChassis (vsetup["chassi"]["mesh"], vsetup["chassi"]["mass"])
    #set axis
    v.addAxis ("Front", vsetup["axle_front"]["mesh"], vsetup["chassi"]["axis_position_front"])
    v.addAxis ("Rear", vsetup["axle_rear"]["mesh"], vsetup["chassi"]["axis_position_rear"])    
    #set wheel & tire
    v.addWheel("FrontLeft", vsetup["tire_front"]["mesh_left"])
    v.addWheel("FrontRight",vsetup["tire_front"]["mesh_right"])
    v.addWheel("RearLeft",  vsetup["tire_rear"]["mesh_left"])
    v.addWheel("RearRight", vsetup["tire_rear"]["mesh_right"])
    #set tire
    v.setTireBehavior("Front", 
                      vsetup["tire_front"]["lateral_asymptote_slip"], 
                      vsetup["tire_front"]["lateral_asymptote_value"], 
                      vsetup["tire_front"]["lateral_extreme_slip"], 
                      vsetup["tire_front"]["lateral_extreme_value"], 
                      vsetup["tire_front"]["longitudal_asymptote_slip"], 
                      vsetup["tire_front"]["longitudal_asymptote_value"], 
                      vsetup["tire_front"]["longitudal_extreme_slip"], 
                      vsetup["tire_front"]["longitudal_extreme_value"],)
    v.setTireGrip ("Front", vsetup["tire_front"]["lateral_stiffness_factor"], vsetup["tire_front"]["longitudal_stiffness_factor"])
    v.setTireBehavior("Rear", 
                      vsetup["tire_rear"]["lateral_asymptote_slip"], 
                      vsetup["tire_rear"]["lateral_asymptote_value"], 
                      vsetup["tire_rear"]["lateral_extreme_slip"], 
                      vsetup["tire_rear"]["lateral_extreme_value"], 
                      vsetup["tire_rear"]["longitudal_asymptote_slip"], 
                      vsetup["tire_rear"]["longitudal_asymptote_value"], 
                      vsetup["tire_rear"]["longitudal_extreme_slip"], 
                      vsetup["tire_rear"]["longitudal_extreme_value"],)
    v.setTireGrip ("Rear", vsetup["tire_rear"]["lateral_stiffness_factor"], vsetup["tire_rear"]["longitudal_stiffness_factor"])
    #set wheel
    v.setWheelBehavior ("Front", vsetup["wheel_front"]["mass"], vsetup["wheel_front"]["mass_inv"])
    v.setWheelBehavior ("Rear", vsetup["wheel_rear"]["mass"], vsetup["wheel_rear"]["mass_inv"])
    if vsetup["steering"].has_key("ownaxis") : v.setSteer (vsetup["steering"]["increase"], vsetup["steering"]["decrease"], float(vsetup["steering_arm"]["radian"]), vsetup["steering"]["ownaxis"])
    else : v.setSteer (vsetup["steering"]["increase"], vsetup["steering"]["decrease"], float(vsetup["steering_arm"]["radian"]))
    #set engine
    v.setEngine (vsetup["engine"]["power"],vsetup["engine"]["increase"], vsetup["engine"]["decrease"])
    v.addEnginePowerStep (float(vsetup["engine"]["power_step0"]))
    v.addEnginePowerStep (float(vsetup["engine"]["power_step1"]))
    v.addEnginePowerStep (float(vsetup["engine"]["power_step2"]))
    v.addEnginePowerStep (float(vsetup["engine"]["power_step3"]))
    v.addEnginePowerStep (float(vsetup["engine"]["power_step4"]))
    #set brake
    v.setBrake("Front", float(vsetup["brake_front"]["force"]))
    v.setBrake("Rear", float(vsetup["brake_rear"]["force"]))
    #set drive shaft
    v.setDriveShaftMotion ("Front", "FrontDriveShaft", vsetup["driveshaft_front"]["mesh"], eval("["+vsetup["driveshaft_front"]["pivot"]+"]"), "Connect")
    v.setDriveShaftMotion ("Rear", "RearDriveShaft", vsetup["driveshaft_rear"]["mesh"], eval("["+vsetup["driveshaft_rear"]["pivot"]+"]"), "Connect");
    v.setChassisBolt ("FrontDriveShaft", "",eval("["+vsetup["chassi"]["bracket_driveshaft_f"]+"]"))
    v.setChassisBolt ("RearDriveShaft", "", eval("["+vsetup["chassi"]["bracket_driveshaft_r"]+"]"))
    v.setAxisBolt ("Front", "FrontDriveShaft", "",eval("["+vsetup["axle_front"]["bracket_driveshaft"]+"]"))
    v.setAxisBolt ("Rear", "RearDriveShaft", "",  eval("["+vsetup["axle_rear"]["bracket_driveshaft"]+"]"))
    #set suspension
    v.setSuspensionBehavior ("FrontLeft",vsetup["suspension_front"]["spring"])
    v.setSuspensionBehavior ("FrontRight",vsetup["suspension_front"]["spring"])
    v.setSuspensionBehavior ("RearLeft",vsetup["suspension_rear"]["spring"])
    v.setSuspensionBehavior ("RearRight",vsetup["suspension_rear"]["spring"])
    if vsetup["suspension_front"]["mesh"] <> "" : v.setSuspensionMotion ("FrontLeft", "FrontSuspensionBoltLeft",  vsetup["suspension_front"]["mesh"], eval("["+vsetup["suspension_front"]["pivot"]+"]"), "Connect")
    if vsetup["suspension_front"]["mesh"] <> "" : v.setSuspensionMotion ("FrontRight", "FrontSuspensionBoltRight",vsetup["suspension_front"]["mesh"], eval("["+vsetup["suspension_front"]["pivot"]+"]"), "Connect")
    if vsetup["suspension_rear"]["mesh"] <> "" : v.setSuspensionMotion ("RearRight", "RearSuspensionBoltRight", vsetup["suspension_rear"]["mesh"], eval("["+vsetup["suspension_rear"]["pivot"]+"]"), "Connect")
    if vsetup["suspension_rear"]["mesh"] <> "" : v.setSuspensionMotion ("RearLeft", "RearSuspensionBoltLeft",   vsetup["suspension_rear"]["mesh"], eval("["+vsetup["suspension_rear"]["pivot"]+"]"), "Connect")
    v.setChassisBolt ("FrontSuspensionBoltLeft", vsetup["chassi"]["bracket_suspension_type_f_id"]["mesh_l"], eval("["+vsetup["chassi"]["bracket_suspension_fl"]+"]"))
    v.setChassisBolt ("FrontSuspensionBoltRight",vsetup["chassi"]["bracket_suspension_type_f_id"]["mesh_r"], eval("["+vsetup["chassi"]["bracket_suspension_fr"]+"]"))
    v.setChassisBolt ("RearSuspensionBoltLeft",  vsetup["chassi"]["bracket_suspension_type_r_id"]["mesh_l"], eval("["+vsetup["chassi"]["bracket_suspension_rl"]+"]"))
    v.setChassisBolt ("RearSuspensionBoltRight", vsetup["chassi"]["bracket_suspension_type_r_id"]["mesh_r"], eval("["+vsetup["chassi"]["bracket_suspension_rr"]+"]"))
    v.setAxisBolt ("FrontLeft", "FrontSuspensionBoltLeft", vsetup["axle_front"]["bracket_suspension_type_id"]["mesh_l"], eval("["+vsetup["axle_front"]["bracket_suspension_l"]+"]"))
    v.setAxisBolt ("FrontRight","FrontSuspensionBoltRight",vsetup["axle_front"]["bracket_suspension_type_id"]["mesh_r"], eval("["+vsetup["axle_front"]["bracket_suspension_r"]+"]"))
    v.setAxisBolt ("RearLeft",  "RearSuspensionBoltLeft",  vsetup["axle_rear"]["bracket_suspension_type_id"]["mesh_l"], eval("["+vsetup["axle_rear"]["bracket_suspension_l"]+"]"))
    v.setAxisBolt ("RearRight", "RearSuspensionBoltRight", vsetup["axle_rear"]["bracket_suspension_type_id"]["mesh_r"], eval("["+vsetup["axle_rear"]["bracket_suspension_r"]+"]"))
    #set shockabsorber
    v.setShockAbsorberBehavior ("FrontLeft", float(vsetup["shockabsorber_front"]["damper"]), float(vsetup["shockabsorber_front"]["target"]) )
    v.setShockAbsorberBehavior ("FrontRight", float(vsetup["shockabsorber_front"]["damper"]), float(vsetup["shockabsorber_front"]["target"]) )
    v.setShockAbsorberBehavior ("RearLeft", float(vsetup["shockabsorber_rear"]["damper"]), float(vsetup["shockabsorber_rear"]["target"]) )
    v.setShockAbsorberBehavior ("RearRight", float(vsetup["shockabsorber_rear"]["damper"]), float(vsetup["shockabsorber_rear"]["target"]) )
    if vsetup["shockabsorber_front"]["mesh"] <> "" : v.setShockAbsorberMotion ("FrontLeft", "FrontShockAbsorberBoltLeft",  vsetup["shockabsorber_front"]["mesh"], eval("["+vsetup["shockabsorber_front"]["pivot"]+"]"), "Connect")
    if vsetup["shockabsorber_front"]["mesh"] <> "" : v.setShockAbsorberMotion ("FrontRight", "FrontShockAbsorberBoltRight",vsetup["shockabsorber_front"]["mesh"], eval("["+vsetup["shockabsorber_front"]["pivot"]+"]"), "Connect")
    if vsetup["shockabsorber_rear"]["mesh"] <> "" : v.setShockAbsorberMotion ("RearRight", "RearShockAbsorberBoltRight", vsetup["shockabsorber_rear"]["mesh"], eval("["+vsetup["shockabsorber_rear"]["pivot"]+"]"), "Connect")
    if vsetup["shockabsorber_rear"]["mesh"] <> "" : v.setShockAbsorberMotion ("RearLeft", "RearShockAbsorberBoltLeft",   vsetup["shockabsorber_rear"]["mesh"], eval("["+vsetup["shockabsorber_rear"]["pivot"]+"]"), "Connect")
    v.setChassisBolt ("FrontShockAbsorberBoltLeft", vsetup["chassi"]["bracket_shockabsorber_type_f_id"]["mesh_l"], eval("["+vsetup["chassi"]["bracket_shockabsorber_fl"]+"]"))
    v.setChassisBolt ("FrontShockAbsorberBoltRight",vsetup["chassi"]["bracket_shockabsorber_type_f_id"]["mesh_r"], eval("["+vsetup["chassi"]["bracket_shockabsorber_fr"]+"]"))
    v.setChassisBolt ("RearShockAbsorberBoltLeft",  vsetup["chassi"]["bracket_shockabsorber_type_r_id"]["mesh_l"], eval("["+vsetup["chassi"]["bracket_shockabsorber_rl"]+"]"))
    v.setChassisBolt ("RearShockAbsorberBoltRight", vsetup["chassi"]["bracket_shockabsorber_type_r_id"]["mesh_r"], eval("["+vsetup["chassi"]["bracket_shockabsorber_rr"]+"]"))
    v.setAxisBolt ("FrontLeft", "FrontShockAbsorberBoltLeft", vsetup["axle_front"]["bracket_shockabsorber_type_id"]["mesh_l"], eval("["+vsetup["axle_front"]["bracket_shockabsorber_l"]+"]"))
    v.setAxisBolt ("FrontRight","FrontShockAbsorberBoltRight",vsetup["axle_front"]["bracket_shockabsorber_type_id"]["mesh_r"], eval("["+vsetup["axle_front"]["bracket_shockabsorber_r"]+"]"))
    v.setAxisBolt ("RearLeft",  "RearShockAbsorberBoltLeft",  vsetup["axle_rear"]["bracket_shockabsorber_type_id"]["mesh_l"], eval("["+vsetup["axle_rear"]["bracket_shockabsorber_l"]+"]"))
    v.setAxisBolt ("RearRight", "RearShockAbsorberBoltRight", vsetup["axle_rear"]["bracket_shockabsorber_type_id"]["mesh_r"], eval("["+vsetup["axle_rear"]["bracket_shockabsorber_r"]+"]"))
    #set controlarm
    v.setControlArmBehavior ("FrontLeft", float(vsetup["controlarm_front"]["travel"]))
    v.setControlArmBehavior ("FrontRight", float(vsetup["controlarm_front"]["travel"]))
    v.setControlArmBehavior ("RearLeft", float(vsetup["controlarm_rear"]["travel"]))
    v.setControlArmBehavior ("RearRight", float(vsetup["controlarm_rear"]["travel"]))
    if vsetup["controlarm_front"]["mesh"] <> "" : v.setControlArmMotion ("FrontLeft", "FrontControlArmBoltLeft",  vsetup["controlarm_front"]["mesh"],eval("["+vsetup["controlarm_front"]["pivot"]+"]"), "Connect")
    if vsetup["controlarm_front"]["mesh"] <> "" : v.setControlArmMotion ("FrontRight","FrontControlArmBoltRight", vsetup["controlarm_front"]["mesh"],eval("["+vsetup["controlarm_front"]["pivot"]+"]"), "Connect")
    if vsetup["controlarm_rear"]["mesh"] <> "" : v.setControlArmMotion ("RearLeft",   "RearControlArmBoltLeft",   vsetup["controlarm_rear"]["mesh"], eval("["+vsetup["controlarm_rear"]["pivot"]+"]"), "Connect")
    if vsetup["controlarm_rear"]["mesh"] <> "" : v.setControlArmMotion ("RearRight",  "RearControlArmBoltRight",  vsetup["controlarm_rear"]["mesh"], eval("["+vsetup["controlarm_rear"]["pivot"]+"]"), "Connect")
    v.setChassisBolt ("FrontControlArmBoltLeft",  vsetup["chassi"]["bracket_controlarm_type_f_id"]["mesh_l"], eval("["+vsetup["chassi"]["bracket_controlarm_fl"]+"]"))
    v.setChassisBolt ("FrontControlArmBoltRight", vsetup["chassi"]["bracket_controlarm_type_f_id"]["mesh_r"], eval("["+vsetup["chassi"]["bracket_controlarm_fr"]+"]"))
    v.setChassisBolt ("RearControlArmBoltLeft",   vsetup["chassi"]["bracket_controlarm_type_r_id"]["mesh_l"], eval("["+vsetup["chassi"]["bracket_controlarm_rl"]+"]"))
    v.setChassisBolt ("RearControlArmBoltRight",  vsetup["chassi"]["bracket_controlarm_type_r_id"]["mesh_r"], eval("["+vsetup["chassi"]["bracket_controlarm_rr"]+"]"))
    v.setAxisBolt ("FrontLeft", "FrontControlArmBoltLeft", vsetup["axle_front"]["bracket_controlarm_type_id"]["mesh_l"], eval("["+vsetup["axle_front"]["bracket_controlarm_l"]+"]"))
    v.setAxisBolt ("FrontRight","FrontControlArmBoltRight",vsetup["axle_front"]["bracket_controlarm_type_id"]["mesh_r"], eval("["+vsetup["axle_front"]["bracket_controlarm_r"]+"]"))
    v.setAxisBolt ("RearLeft",  "RearControlArmBoltLeft",  vsetup["axle_rear"]["bracket_controlarm_type_id"]["mesh_l"],  eval("["+vsetup["axle_rear"]["bracket_controlarm_l"]+"]"))
    v.setAxisBolt ("RearRight", "RearControlArmBoltRight", vsetup["axle_rear"]["bracket_controlarm_type_id"]["mesh_r"],  eval("["+vsetup["axle_rear"]["bracket_controlarm_r"]+"]"))
    return v

def getPartTypes ( type=None ) : 
    """
    Returns all the part types from server if type is not None. Else we return only that defined part type. 
    Where type is one of these: "engine", "tire", "chassis", "brake", "suspension", "steer", "axis".
    Raises VehicleManagerError if type is not supported.
    Returns in the format if None {<type>:{<name>:{<info>},<name>:{<info>}} } else {<name>:{<info>},<name>:{<info>}}
    """
    global __vehiclePartType
    __rpc = xmlrpclib.ServerProxy( settings.SERV_STATE_MACHINE, transport=settings.proxy  )
    __vehiclePartType = __rpc.getParts( SystemHandler.getGameClass().getPlayer().id )
    if type == None : return __vehiclePartType
    else : 
        if __vehiclePartType.has_key( type ) : return __vehiclePartType[type]
        else : raise VehicleManagerError, "Could not return types because defined type '"+str( type )+"' is not supported. Supported types are "+str( __vehiclePartType.keys() )

def getVehiclePartTypes () : 
    """
    Returns an list of the different parts names used on the vehicle .
    """
    return __vehiclePart

def getVehicleNames (forceUpdate=False):
    """
    Returns an list of vehicles available to the player 
    """
    global __vehicleDic, __rpc
    playerName = SystemHandler.getGameClass().getPlayer().id
    if __vehicleDic == None or forceUpdate : __vehicleDic = __rpc.getVehicle( playerName )
    if __vehicleDic == None : raise VehicleManagerError, "Got error from server. Probably because user '"+playerName+"' does not exist"
    return __vehicleDic.keys()

def __applyDustAndTracks (vehicleInstance) :
    """ 
    Will use the dust and track definition and apply it to the vehicle. So that when driving we emitt dust and particles.
    Note: Must be called before makeDynamic()
    """
    vehicleInstance.setWheelTracks ("WheelDigger01Tracks")
    vehicleInstance.setWheelParticles ("wheelemission/darkbrown/mud_rotatingx01", 0)
    vehicleInstance.setWheelParticles ("wheelemission/lightbrown/dust_light01", 1)
    vehicleInstance.setWheelParticles ("wheelemission/darkbrown/mud_rotatingx02", 2)
    vehicleInstance.setWheelParticles ("wheelemission/darkbrown/mud_rotatingx03", 3)
    vehicleInstance.setWheelParticles ("", 0)
    vehicleInstance.setWheelParticles ("", 1)
    vehicleInstance.setWheelParticles ("", 2)
    vehicleInstance.setWheelParticles ("", 3)
    vehicleInstance.setWheelParticles ("wheelemission/darkbrown/mud_rotatingx01", 0)
    vehicleInstance.setWheelParticles ("wheelemission/lightbrown/dust_light01", 1)
    vehicleInstance.setWheelParticles ("wheelemission/darkbrown/mud_rotatingx02", 2)
    vehicleInstance.setWheelParticles ("wheelemission/darkbrown/mud_rotatingx03", 3)
    
def isVehicleLoaded (vehicleName):
    """Checks if an vehicle (either physical or static) is loaded into the system"""
    return PhysicsManager.isCarPresent(vehicleName)

def getDynamicVehicle ( playerName, vehicleName, instanceName, useEffects=False ):
    """
    Returns a dynamic vehicle beloning to user having alias 'playerName' and vehicle id is 'vehicleName'. 
    By a dynamic vehicle we mean a vehicle that is interactive and drivable. This is the drivable car.
    Raises VehicleManagerError if playerName does not exist or the player does not have vehicle 'vehicleName'
    """
    if playerName == None : raise VehicleManagerError, "Player name was None"
    if vehicleName == None : raise VehicleManagerError, "Vehicle name was None"
    if instanceName == None : raise VehicleManagerError, "Instance name was None"
    v = __loadVehicle ( playerName, vehicleName, instanceName )
    if useEffects : __applyDustAndTracks(v)
    v.makeDynamic()
    return v

def getStaticVehicle ( playerName, vehicleName, instanceName ):
    """
    Returns a static vehicle beloning to user having alias 'playerName' and vehicle id is 'vehicleName'. 
    By static we mean a vehicle that does not interact with the enviroment (no physical representation) 
    only using graphical entities to represent its parts.
    The main idea is to use it in garage or for eye candy.
    Raises VehicleManagerError if playerName does not exist or the player does not have vehicle 'vehicleName'
    """
    if playerName == None : raise VehicleManagerError, "Player name was None"
    if vehicleName == None : raise VehicleManagerError, "Vehicle name was None"    
    v = __loadVehicle ( playerName, vehicleName, instanceName )
    v.makeStatic()
    return v

def getVehicleProperties ( playerName, vehicleName ) :
    """ 
    Returns dictionary of vehicle data according to the world server. This is used to create
    different players vehicles.
    The playerName is an player in our game world vehicleName is the name of our vehicle we want information on
    Returns None if vehicleName does not exist or is not owned by player 'playerName'.
    """
    if playerName == None : raise VehicleManagerError, "Player name was None"
    if vehicleName == None : raise VehicleManagerError, "Vehicle name was None"    
    global __vehicleDic
    #if not DriverManager.isValidDriver( playerName ) : raise VehicleManagerError, "Driver does not exist '"+playerName+"'"    
    if __vehicleDic == None : __vehicleDic = __loadVehicleFromServer( playerName )
    if not __vehicleDic.has_key( vehicleName ) :  raise VehicleManagerError, "Could not load the vehicle because '" + vehicleName + "' does not exist for player '"+playerName+"'"
    if __vehicleDic.has_key( vehicleName ) : 
        return __vehicleDic[vehicleName]
    raise VehicleManagerError, "The user '"+playerName+"' or vehicle  '"+vehicleName+"' was not valid"

def setVehicleProperty ( vehicleName, type, partName ) :
    """
    Sets the property of type to part on vehicle vehicleName. Where part and type
    is a name identifiers known by the server.
    It will then retreave the entire definition of the vehicle and replace the currently 
    loaded one.
    The vehicleName must be the name of an "owned" vehicle by the user.
    Supported types are 'axis', 'suspension', 'chassi', 'engine', 'tire', 'brake', 'steering'. 
    Supported names are dependent on defined names owned by user.
    Raises VehicleManagerError if vehicleName, type or part is not recoginzed or not owned by the user.
    """
    if not __vehicleDic.has_key( vehicleName ) :  raise VehicleManagerError, "Could not set the vehicle part because '" + vehicleName + "' does not exist for player"
    playerName = SystemHandler.getGameClass().getPlayer().id
    res = __rpc.setBlueprintPart( playerName, vehicleName, type, partName )
    if res == False :  raise VehicleManagerError, "Got error from server. Probably because part '"+partName+"' or '"+type+"' is invalid"    
    if res == None :  raise VehicleManagerError, "Got error from server. Probably because part '"+partName+"' or '"+type+"' is invalid"    
    __vehicleDic[vehicleName] = res[vehicleName]
    if __vehicleDic == None : raise VehicleManagerError, "Got error from server. Probably because user '"+playerName+"' does not exist"    

#############################################################################
#
#     Testing vehicle manager
#
#############################################################################    
import unittest 
class TestVehicleManager ( unittest.TestCase ) :

    def setTestVehicleData ( self ) : 
        global __vehicleDic
        __vehicleDic["Eagle unittest"] = {}
        __vehicleDic["Eagle unittest"]["chassis"] = {}
        __vehicleDic["Eagle unittest"]["tire"] = {}
        __vehicleDic["Eagle unittest"]["chassis"]["mesh"] = "EagleChassis.mesh"
        __vehicleDic["Eagle unittest"]["chassis"]["mass"] = 1000
        __vehicleDic["Eagle unittest"]["chassis"]["front_z"] = 1
        __vehicleDic["Eagle unittest"]["chassis"]["rear_z"] = -1
        __vehicleDic["Eagle unittest"]["chassis"]["mass"] = 1000
        __vehicleDic["Eagle unittest"]["tire rear"]["mesh_left"] = "WheelDigger01_Left.mesh"
        __vehicleDic["Eagle unittest"]["tire rear"]["mesh_right"] = "WheelDigger01_Right.mesh"
        __vehicleDic["Eagle unittest"]["tire front"]["mesh_left"] = "WheelDigger01_Left.mesh"
        __vehicleDic["Eagle unittest"]["tire front"]["mesh_right"] = "WheelDigger01_Right.mesh"

    def testLoadPlayerParts ( self ):
        """VehicleManager and retraving all player parts information from server"""
        parts = getPartTypes()
        self.assertTrue( len(parts.keys()) > 0 )
        self.assertTrue( parts["chassi"].has_key("Eagle") )
        self.assertRaises( VehicleManagerError, getPartTypes, "NoSuchType" )
    
    def testLoadVehicles ( self ):
        """VehicleManager and retraving all vehicles information from server"""
        #self.assertRaises(VehicleManagerError, getVehicleNames, "nosuchplayer")
        a = getVehicleNames()
        self.assertEqual( a, ["Eagle unittest"] )

    def testLoadStaticVehicle ( self ):
        """VehicleManager and retraving static vehicles from server, then destroy them"""
        carnames = getVehicleNames()
        self.assertRaises( VehicleManagerError, getStaticVehicle, "unittest", "nosuchvehicle" )
        self.assertRaises( VehicleManagerError, getStaticVehicle, "nosuchuser", carnames[0] )
        #now try to assembel all of unitttest's cars
        loaded = "( "
        for vehicle in carnames : 
            v = getStaticVehicle ( "unittest", vehicle )
            loaded = loaded + vehicle + " "
            v.destroy()
        #print loaded + ")"

    def testLoadDynamicVehicle ( self ):
        """VehicleManager and retraving dynamic vehicles from server, then destroy them"""
        carnames = getVehicleNames()
        self.assertRaises( VehicleManagerError, getDynamicVehicle, "unittest", "nosuchvehicle" )
        self.assertRaises( VehicleManagerError, getDynamicVehicle, "nosuchuser", carnames[0] )
        #now try to assembel all of unitttest's cars
        loaded = "("
        for vehicle in carnames : 
            v = getDynamicVehicle ( "unittest", vehicle )
            loaded = loaded + vehicle + " "
            v.destroy()
        #print loaded + ")"
            
#    def testChangeVehicleParts ( self ):
#        """VehicleManager and changing vehicles parts"""
#        carnames = getVehicleNames()
#        self.assertRaises ( VehicleManagerError, setVehicleProperty, "NoSuchVehicle", "chassi", "Eagle" )
#        self.assertRaises ( VehicleManagerError, setVehicleProperty, carnames[0], "NoSuchPart", "Eagle" )
#        self.assertRaises ( VehicleManagerError, setVehicleProperty, carnames[0], "chassi", "NoSuchChassis" )
#        self.assertRaises ( VehicleManagerError, getVehicleProperties, "nosuchuser", carnames[0] )
#        self.assertRaises ( VehicleManagerError, getVehicleProperties, "unittest", "NoSuchVehicle" )
#        setVehicleProperty ( carnames[0], "chassi", "Eagle" )
#        res = getVehicleProperties ( "unittest", carnames[0] )
#        self.assertEqual ( res["chassi"]["name"], "Eagle" )
#        setVehicleProperty ( carnames[0], "chassi", "Musso" )
#        res = getVehicleProperties ( "unittest", carnames[0] )
#        self.assertEqual ( res["chassi"]["name"], "Musso" )
#        setVehicleProperty ( carnames[0], "chassi", "Eagle" )
#        #check now changing all the part types
#        types = getPartTypes()
#        for type in types :
#            parts = getPartTypes( type )
#            if type.startswith( "suspension" ) or type.startswith( "axle" )  or type.startswith( "tire" ) or type.startswith( "wheel" ) or type.startswith( "shockabsorber" ) or type.startswith( "driveshaft" ) or type.startswith( "brake" ) : type = type + " front"
#            if len( parts ) > 0 and type <> "bracket" or type <> "steering" or type <> "driveshaft": 
#                #print "Changing for " + type + " -> " + str(parts.keys())
#                if len(parts.keys()) > 0 :
#                    setVehicleProperty ( "Eagle unittest", type, parts.keys()[0] )
#                    self.assertRaises ( VehicleManagerError, setVehicleProperty, carnames[0], type, "NoSuchPart" )
