"""
Camera driver
"""

__author__ = "Kjartan A Jonsson"
__version__ = "$Revision: 1.2 $"
__copyright__ = "Copyright (c) 2007 aGameCompany"
__license__ = "aGameCompany"

import GUI
import SceneManager
import pyrobot.robot.device

class DeviceError (Exception) : pass

class Device :
    """
    This is a driver interfaces that handles lower level management of camera device.
    """
    machine_group = "interface"
    device_type = "camera"
    x = 0
    y = 0
    z = 0
    entity = None #the entity we use to do intersection testing
    __driver = None
    __sensored_cd = {} #is the sensors index from 0 to n
    __index = {} #is an index looking up sensorname and looking up the index
    __ignore = []
    __gui_instance = None #holds the original gui instance
    __view = 1 #views are 0=driver, follower=1, fly=2
    __filter = 0
    gui = None
    
    def __init__(self, entity, machinename) : 
        print "Camera driver initializing" 
        self.entity = entity #body.getEntity()
        self.__gui_instance = DeviceWindow(self)
        self.gui = None
        self.target_name = entity.id #self.__getTargetName(machinename)

    def __getTargetName (self, machinename) :
        """
        Not used as is. Was used to find entity when using car. Then we wanted to use the car chassis centity.
        We always use camera entity now.
        """
        import PhysicsManager
        import MachineManager
        try :
            v = PhysicsManager.getCar( machinename )
            device_target_entity = SceneManager.Entity(v.getPartName("chassi")[0])
            return device_target_entity.id
        except Exception :
            if self.entity == None : return "" #we failed returning a valid NULL value understandably by the follow-camera 
            else :
                return self.entity.id  

    def setView (self, view) : 
        """Sets camera view, 1 or 0"""
        #print "camera.setView " + str(view)
        if self.gui == None : return None
        #print "camera.setView OK ", self.target_name, self.entity.id
        if view < 0 or view > 4 : raise DeviceError, "View index was not correct. Got: " + str(view)
        self.__view = view
        if self.__view == 0 : 
            SceneManager.getCamera().setFollowProperties (0.8, 0.8, 0, 0, 0)
            SceneManager.getCamera().attach (self.target_name) # (self.entity.id)
        elif self.__view == 1 :
            SceneManager.getCamera().setFollowProperties (0.01, 0.01, 9, 15, 50)
            SceneManager.getCamera().followEntity (self.target_name)
        elif self.__view == 4 :
            SceneManager.getCamera().setFollowProperties (0.01, 0.01, 29, 55, 70)
            SceneManager.getCamera().followEntity (self.target_name)            
        elif self.__view == 2 : 
            SceneManager.getCamera().setFlyFreeControl()            
        elif self.__view == 3 : 
	        SceneManager.getCamera().lookAtEntity(self.entity.id)
        #if self.gui <> None : 
        #    GUI.systemGUI.call ("camera.swf", "setCameraViewUpdate", self.getGUI().reverse_dic_lookup( self.getGUI().cam_view, self.__view))

    def setFilter (self, filter) : 
        """Sets camera filter 0, 1, 2, 3"""
        if self.gui == None : return None
        if filter < 0 or filter > 5 : raise DeviceError, "View index was not legal. Got: " + str(filter)
        self.__filter = filter
        if self.__filter == 0 : SceneManager.setCompostionEffect (None)
        elif self.__filter == 1 : 
            SceneManager.setCompostionEffect (None)
            SceneManager.setCompostionEffect ("Camera Filter Soft")
        elif self.__filter == 2 : 
            SceneManager.setCompostionEffect (None)
            SceneManager.setCompostionEffect ("Camera Filter Edges")
        elif self.__filter == 3 : 
            SceneManager.setCompostionEffect (None)
            SceneManager.setCompostionEffect ("Camera Filter Sharpen")
        elif self.__filter == 4 : 
            SceneManager.setCompostionEffect (None)
            SceneManager.setCompostionEffect ("Camera Filter B&W")
        elif self.__filter == 5 : 
            SceneManager.setCompostionEffect (None)
            SceneManager.setCompostionEffect ("Camera Filter Invert")    
        #if self.gui <> None : 
        #    GUI.systemGUI.call ("camera.swf", "setCameraFilterUpdate", self.getGUI().reverse_dic_lookup( self.getGUI().cam_filter, self.__filter))

    def getView (self) :  return self.__view

    def getFilter (self) : return self.__filter

    def setVisible (self, b) : 
        """This is called when we are un-selecting and selecting the machine"""
        if b : 
            self.gui = self.__gui_instance
            self.setView(self.__view)
            self.setFilter(self.__filter)
        else :
            self.gui = None
    
    def getGUI (self) : return self.__gui_instance
    
from window import Window #get base menu class
import math
class DeviceWindow (Window) :
    """Handles camera menu"""
    lastupdate = 0
    __device = None
    cam_view = {"Attach":0,"Follow":1,"Follow Far":4, "Fly":2}
    cam_filter = {"Clear Filter":0, "Soften image":1, "Show edges":2, "Sharpen image":3, "Black and white":4, "Invert colors":5}
    
    def __init__ (self, device):
    	import const
        self.log_attempt ("register Camera interface")
        try : 
            GUI.systemGUI.loadLayout ("camera.swf", 256, 256)
            GUI.systemGUI.setPosition("camera.swf", 0, const.WINDOW_POSTION_DELTA_HEIGHT*1)
        except Exception, e: self.log_failure (False, str(e))
        self.__device = device
        GUI.systemGUI.call ("camera.swf", "cmbClearLists", "")
        GUI.systemGUI.addEventSubscriber ("camera.swf", "setCameraFilter", "receaveMenuEvent", "filter")
        GUI.systemGUI.addEventSubscriber ("camera.swf", "setCameraView", "receaveMenuEvent", "view")
        names = self.cam_view.keys()
        for name in names : GUI.systemGUI.call ("camera.swf", "addCameraView", name)
        names = self.cam_filter.keys()
        for name in names : GUI.systemGUI.call ("camera.swf", "addCameraFilter", name)
        self.log_success ()
        
    def clear (self):
        GUI.systemGUI.call ("camera.swf", "cmbClearLists", "")
        GUI.systemGUI.delLayout ("camera.swf")
        SceneManager.getCamera().lookAtEntity("")
    
    def reverse_dic_lookup(self, d, v):
        for k in d:
            if d[k] == v:
                return k
        raise ValueError    
    
    def callback (self, source, event, message):
        if source == "camera.swf" : 
            arg = eval(message)
            if event == "setCameraView" : self.__device.setView(self.cam_view[arg["arguments"][0]])
            if event == "setCameraFilter" : self.__device.setFilter(self.cam_filter[arg["arguments"][0]])

import unittest
class TestDevice(unittest.TestCase):
    
    def setUp(self):        
        camdev = SceneManager.addEntity ("camdevicetest", "cube.1m.mesh")
        self.device = Device (camdev, "mymachienname")

    def tearDown(self):
        self.device.setVisible(False)
        #self.device.getGUI().clear()
        SceneManager.delEntity ("camdevicetest")

    def testGUIActivate(self):
        """Camera device and test GUI active and un-active"""
        self.device.setVisible(True)
        self.device.setVisible(False)

    def testGUICameraDeviceCallbackView(self):
        """Camera device and test device call-back view"""
        self.device.setVisible(False)
        self.device.getGUI().callback("camera.swf", "setCameraView", "{'arguments':['"+"Attach"+"']}")
        self.assertEqual (1, self.device.getView())
        self.device.getGUI().callback("camera.swf", "setCameraView", "{'arguments':['"+"Follow"+"']}")
        self.assertEqual (1, self.device.getView())        
        self.device.getGUI().callback("camera.swf", "setCameraView", "{'arguments':['"+"Fly"+"']}")
        self.assertEqual (1, self.device.getView())
        self.device.setVisible(True)
        self.device.getGUI().callback("camera.swf", "setCameraView", "{'arguments':['"+"Attach"+"']}")
        self.assertEqual (0, self.device.getView())
        self.device.getGUI().callback("camera.swf", "setCameraView", "{'arguments':['"+"Follow"+"']}")
        self.assertEqual (1, self.device.getView())        
        self.device.getGUI().callback("camera.swf", "setCameraView", "{'arguments':['"+"Fly"+"']}")
        self.assertEqual (2, self.device.getView())
        self.assertRaises(DeviceError, self.device.setView, "{'arguments':['"+"NoSuchView"+"']}")
        self.assertEqual (2, self.device.getView())                
        self.device.setVisible(False)
        self.device.getGUI().callback("camera.swf", "setCameraView", "{'arguments':['"+"Attach"+"']}")
        self.assertEqual (2, self.device.getView())
        self.device.getGUI().callback("camera.swf", "setCameraView", "{'arguments':['"+"Follow"+"']}")
        self.assertEqual (2, self.device.getView())        
        self.device.getGUI().callback("camera.swf", "setCameraView", "{'arguments':['"+"Fly"+"']}")
        self.assertEqual (2, self.device.getView())

    def testCameraDeviceView(self):
        """IR device and test device set view"""
        self.assertEqual (0, self.device.getView())
        self.device.setView (0)
        self.assertEqual (0, self.device.getView())
        self.device.setView (1)
        self.assertEqual (0, self.device.getView())        
        self.device.setView (2)
        self.assertEqual (0, self.device.getView())                         
        self.device.setVisible(True)
        self.assertEqual (0, self.device.getView())
        self.device.setView (0)
        self.assertEqual (0, self.device.getView())
        self.device.setView (2)
        self.assertEqual (2, self.device.getView())                         
        self.device.setView (1)
        self.assertEqual (1, self.device.getView())
        self.assertRaises(DeviceError, self.device.setView, -1)
        self.assertRaises(DeviceError, self.device.setView, 3)        
        self.assertEqual (1, self.device.getView())                
        self.device.setVisible(False)
        self.assertEqual (1, self.device.getView())
        self.device.setView (0)
        self.assertEqual (1, self.device.getView())
        self.device.setView (1)
        self.assertEqual (1, self.device.getView())     
        self.device.setView (2)
        self.assertEqual (1, self.device.getView())
        self.device.setView (0)                 

    def testCameraDeviceView(self):
        """IR device and test device set view"""
        self.device.setView (1)
        self.device.setFilter (1)
        self.device.setVisible(True)
        self.device.setView (2)
        self.device.setFilter (2)

    def testCameraDeviceFilter(self):
        """IR device and test device set filter"""
        self.assertEqual (0, self.device.getFilter())
        self.device.setFilter (0)
        self.assertEqual (0, self.device.getFilter())
        self.device.setFilter (1)
        self.assertEqual (0, self.device.getFilter())        
        self.device.setFilter (2)
        self.assertEqual (0, self.device.getFilter())                         
        self.device.setVisible(True)
        self.assertEqual (0, self.device.getFilter())
        self.device.setFilter (0)
        self.assertEqual (0, self.device.getFilter())
        self.device.setFilter (2)
        self.assertEqual (2, self.device.getFilter())                         
        self.device.setFilter (1)
        self.assertEqual (1, self.device.getFilter())
        self.assertRaises(DeviceError, self.device.setFilter, -1)
        self.assertEqual (1, self.device.getFilter())                
        self.device.setVisible(False)
        self.assertEqual (1, self.device.getFilter())
        self.device.setFilter (0)
        self.assertEqual (1, self.device.getFilter())
        self.device.setFilter (1)
        self.assertEqual (1, self.device.getFilter())     
        self.device.setFilter (2)
        self.assertEqual (1, self.device.getFilter())
        self.device.setFilter (0)                 

    def testGUICameraDeviceCallbackFilter(self):
        """Camera device and test device call-back filter"""
        self.device.getGUI().callback("camera.swf", "setCameraFilter", "{'arguments':['"+"Clear Filter"+"']}")
        self.assertEqual (0, self.device.getFilter())
        self.device.getGUI().callback("camera.swf", "setCameraFilter", "{'arguments':['"+"Soften image"+"']}")
        self.assertEqual (0, self.device.getFilter())        
        self.device.getGUI().callback("camera.swf", "setCameraFilter", "{'arguments':['"+"Show edges"+"']}")
        self.assertEqual (0, self.device.getFilter())
        self.device.setVisible(True)
        self.device.getGUI().callback("camera.swf", "setCameraFilter", "{'arguments':['"+"Clear Filter"+"']}")
        self.assertEqual (0, self.device.getFilter())
        self.device.getGUI().callback("camera.swf", "setCameraFilter", "{'arguments':['"+"Soften image"+"']}")
        self.assertEqual (1, self.device.getFilter())        
        self.device.getGUI().callback("camera.swf", "setCameraFilter", "{'arguments':['"+"Show edges"+"']}")
        self.assertEqual (2, self.device.getFilter())
        self.assertRaises(DeviceError, self.device.setFilter, "{'arguments':['"+"NoSuchFilter"+"']}")
        self.assertEqual (2, self.device.getFilter())                
        self.device.setVisible(False)
        self.device.getGUI().callback("camera.swf", "setCameraFilter", "{'arguments':['"+"Clear Filter"+"']}")
        self.assertEqual (2, self.device.getFilter())
        self.device.getGUI().callback("camera.swf", "setCameraFilter", "{'arguments':['"+"Soften image"+"']}")
        self.assertEqual (2, self.device.getFilter())        
        self.device.getGUI().callback("camera.swf", "setCameraFilter", "{'arguments':['"+"Show edges"+"']}")
        self.assertEqual (2, self.device.getFilter())