"""
GUI information flow works as follows: 
driver always output to gui instance. If the gui instance is None then we do not output to that instance.
We restore the instance through setVisible(True/False) from __gui_instance 
"""
__author__ = "Kjartan A Jonsson"
__version__ = "$Revision: 1.11 $"
__copyright__ = "Copyright (c) 2007 aGameCompany"
__license__ = "aGameCompany"

import MachineManager 

class ControlException : pass

import sys
import const
import settings
sys.path.append (settings.PATH_USER_DIRECTORY) #add the path to user scripts

import control_pyro
from window import Window #get base menu class
import GUI
import sys
import string
from traceback import print_exc, format_exception
import traceback
import pyrobot.gui

class PyroCommandGUI (pyrobot.gui.gui) :
    def __init__ (self, name = 'abstract gui', options = {}, engine = 0):
        pyrobot.gui.gui.__init__ (self, name, options=options, engine=engine)
        
    def help (self) :
        helpall =           "==================================================================*\n"

	helpall = helpall + "run => will execute and run the robot brain \n"
	helpall = helpall + "runtillquit => see run \n"
	helpall = helpall + "step => will to a robot step \n"
	helpall = helpall + "info => will return robot information \n"
	helpall = helpall + "! => will display the command history and the index of the command that you can execute through command '!'. See '! <index start> - <index end>' and '! <index>' for more detail. \n"
	helpall = helpall + "!! => will execute last command \n"
	helpall = helpall + "! <start>-<end> => will execute command in history from index start to stop \n"
	helpall = helpall + "! <index> => will execute command at index in history. See ! for more information. \n"
	helpall = helpall + "$ <expression> => will forward command to operative system console \n"
	helpall = helpall + "about => will display about text \n"
	helpall = helpall + "reload => will reset engine \n"
	helpall = helpall + "reset => see reload \n"
	helpall = helpall + "restart => see reload \n"
	helpall = helpall + "load brain <name> => will load the specified brain. Omit file extension. \n"
	helpall = helpall + "stop => will stop the robot execution \n"
	helpall = helpall + "quit => see stop \n"
	helpall = helpall + "exit => see stop \n"
	helpall = helpall + "bye => see stop \n"
	helpall = helpall + "edit => opens the currently executed brain in notepad editor. The program will freaze until the editor has been closed. \n"
	helpall = helpall + "watch <expression> => will add the expression to the watch expression list. And execute and output the result of the expression each time a command is executed by the expression editor  \n"
	helpall = helpall + "unwatch <expression> => will remove the expression from the watched expression list \n"
	helpall = helpall + "browse <expression> => currently not implemented \n"
	helpall = helpall + "<python expression> => will execute any python expression displaying a trace back error description on failure. \n"
        helpall = helpall + "==================================================================="
        return helpall

    def usage (self) :
        usageall =            "===================================================================\n"
        usageall = usageall + "Expression box, bellow, allow a user to dynamically evaluate \n"
        usageall = usageall + "the robot current status. \n"
        usageall = usageall + "Access all the variables a robot and devices is working with. \n"
        usageall = usageall + "Combine them using regular Python expressions. \n"
        usageall = usageall + "Type help for more information.\n"
        usageall = usageall + "==================================================================="
        return usageall        

    def about (self) :
        from pyrobot.system.version import version as version
        res = ""    
        res = res +  "===================================================================\n"
        res = res +  "Machine Race, (c) 2007, Kjartan Akil Jonsson \n"
        res = res +  "http://machinerace.com \n"
        res = res +  "Version 0.5.0 \n"
        res = res +  "\n"
        res = res +  "Pyrobot, Python Robotics, (c) 2005, D.S. Blank \n"
        res = res +  "http://PyroRobotics.org \n"
        res = res +  "Version " + version() + "\n"
        res = res +  "==================================================================="
        return res

    def objectBrowser(self, var) :
        return "objectBrowser not implemented, got " + str(var)


class ControlAIPyro (Window) :
    module = None
    brainname = None
    machine = None
    window = None
    engine = None
    __control_machine = None
    pyrogui = None
    
    def __init__ (self) :
        pass

    def log (self, msg):
        print msg
        if self.__control_machine == self.machine :
            msg = const.toHTML(msg)
            lines = msg.split("\n")
            lines.reverse() #since we append the string to the gui log
            for msgline in lines:
                msgline = msgline.replace('\t', "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;")
                msgline = msgline.replace(' ', "&nbsp;&nbsp;")
                GUI.systemGUI.call ("aipyro.swf", "addLogMessage", msgline)
    
    def getGUI(self) : return self.pyrogui
    
    def __load (self, name):
        self.time_since_last_watch = 0
        self.brainname = name
        self.log("Reading pyro module '"+self.brainname+"' ... " + str(dir(self.module)))
        try :
            self.module = __import__(name)#, globals(),  locals(), [], -1)
        except ImportError, err:
            self.log("Couldn't load %s: %s" % (name, err))
        
    def __listf(self, data):
        buffer = ""
        for line in data:
            buffer = buffer + line + "\n"
        return buffer

    def __recompile(self, modulename): # taken from http://www.elifulkerson.com/projects/python-dynamically-reload-module.php
        self.log("Compiling module " + modulename) #first, see if the module can be imported at all...
        try:
            tmp = __import__(modulename)
        except:
            return self.log("Couldn't import module " + modulename)
        pycfile = tmp.__file__ #Use the imported module to determine its actual path
        modulepath = string.replace(pycfile, ".pyc", ".py")
        try: #Try to open the specified module as a file
            code=open(modulepath, 'rU').read()
        except:
            return self.log("Error opening file: " + modulepath + ".  Does it exist?")
        #see if the file we opened can compile.  If not, return the error that it gives.
        #if compile() fails, the module will not be replaced.
        try:
            compile(code, modulename, "exec")
            self.log("Compiled file successfully ...")
        except:
           return self.log("Error in compilation: " + str(sys.exc_info()[0]) +"\r\n" + __listf(format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback)))
        else: #Ok, it compiled.  But will it execute without error? 
            self.log("Compiling module " + modulename + " success")
            try:
                execfile(modulepath)
                self.log("Executed successfully ...")
            except:
                return self.log("Error in execution: " + str(sys.exc_info()[0]) +"\r\n" + __listf(format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback)))
            else:
                #at this point, the code both compiled and ran without error.  Load it up
                #replacing the original code.
                reload(sys.modules[modulename])
                self.log("Reloaded successfully ...")
        return self.log("Module successfully recompiled")

    def __change(self, file):
        if file == "" : 
            self.log("Could not load brain. Got empty string")
            return False 
        self.giveControl()
        self.log("Changing script to '" + file + "' ...")
#        try: #Try to open the specified module as a file
            #code=open(file, 'rU').read()
        try : self.module = __import__(file)
        except ImportError, err: 
            self.log("Could not load %s: %s. Aborting load and restoring to previous brain." % (file, err))
            file = self.brainname #restore to previous brain
        self.brainname = file
 #       except:
 #           self.log("Error opening file: " + file + ".  Does it exist? Aborting load and restoring to previous brain.")        
        self.takeControl(self.machine)
        return True
        
    def setup (self, data) :
        """
        Remember the script name
        """
        self.engine = None
        self.__load(data)
        try :
            GUI.systemGUI.loadLayout ("aipyro.swf", 512, 512)
            GUI.systemGUI.setPosition("aipyro.swf", 300, 100)
            GUI.systemGUI.addEventSubscriber ("aipyro.swf", "loadScript", "receaveMenuEvent", "change script")
            GUI.systemGUI.addEventSubscriber ("aipyro.swf", "reloadScript", "receaveMenuEvent", "reloading script")
            GUI.systemGUI.addEventSubscriber ("aipyro.swf", "executeScript", "receaveMenuEvent", "execute string")
            self.__load_scripts()
        except Exception, e : pass

    def clear (self) :
        """Will unload the GUI for the controller"""
        GUI.systemGUI.delLayout ("aipyro.swf")

    def takeControl (self, machine) :
        """
        Takes the devices and stores it in a robot
        """
        try :
            if self.engine == None : self.engine = control_pyro.AIPyroEngine(machine.id, machine) #HERE IS STARTUP MACHINE, MOVE TO CALLBACK AND ADD TO GUI START/SHUTOWN
            self.__brain = self.module.INIT(self.engine)
            self.machine = machine
            self.engine.robot.brain = self.__brain
            self.__brain.activated()
            self.log("AI '"+self.brainname+"' took control of machine '" + str(self.machine.name) + "'")# + str(self.machine.getDevices().keys()))
            self.pyrogui = PyroCommandGUI(name=self.machine.name + " gui", options={}, engine=self.engine)
            self.log(self.module.__doc__)
            self.log("*** "+self.brainname+" document string ***")
            self.engine.setBrain (self.__brain)
            self.engine.setRobot(self.engine.robot)
            self.__updateWatchedVariables()
            self.__updateScriptSelection()
            self.engine.brainfile = settings.PATH_USER_DIRECTORY + self.brainname + ".py"
        except : 
            self.log (str(traceback.format_exc()))
        
    def giveControl (self):
        self.__brain.deactivated()
        del self.__brain
        self.__brain = None
        self.log("AI '"+self.brainname+"' gave control of machine '" + str(self.machine.name) + "'")# + str(self.machine.getDevices().keys()))

    def __updateScriptSelection (self) :
        if self.__control_machine == self.machine :
            GUI.systemGUI.call ("aipyro.swf", "setScript", self.brainname)

    def __updateWatchedVariables (self) :
        import time
        if time.clock()-self.time_since_last_watch > 1 :
            self.time_since_last_watch = time.clock()
            if self.__control_machine == self.machine :
                  
                try :
                    GUI.systemGUI.call ("aipyro.swf", "setWatchValues", self.engine.robot.getView())
                except : pass           
                
    def setTime (self, time) : 
        self.time = time
        self.engine.robot.brain.timestamp = time
        self.engine.robot.timestamp = time
        self.engine.timestamp = time
    
    def callback  (self, event, act) :
        """
        Handles the "virtual thread" callback. Since we do not allow threads in python we use this callback 
        to make sure that the brain's tick.
        """
        if self.__brain == None : return None
        if event == const.CONTROL_CALLBACK_TIMER : 
            if self.__brain.isAlive() : self.__brain.run()
            self.__updateWatchedVariables ()
        else : 
            if event == const.CONTROL_CALLBACK_KEYBOARD :
                if self.__control_machine == self.machine :
                    self.__brain.callback (const.CONTROL_CALLBACK_KEYBOARD, act["down"], act["key"])

    def receaveEvent (self, src, evt, msg) :
        if self.__control_machine == self.machine : #make sure we apply GUI events on current machine
            arg = eval(msg)
            if evt == "reloadScript" :
                self.__load_scripts()
                self.__recompile(self.brainname)
                self.__change(self.brainname)
            elif evt == "executeScript" :
                try : 
                    command = arg["arguments"][0]
                    if command.startswith("load brain") : #special case these command since we control them best here
                        command = command.replace("load brain", "").strip()
                        if self.__change(command) == None : self.brainname = command
                    elif command == "reload" : 
                        self.__recompile(self.brainname)
                        self.__change(self.brainname)
                    else : self.log (self.pyrogui.processCommand(command))
                except : 
                    self.log (str(traceback.format_exc()))
            elif evt == "loadScript" :
                if len(arg["arguments"]) == 1 :
                    if arg["arguments"][0] <> "" :
                        self.__change (arg["arguments"][0])

    def setMachineControl (self, machine_name) :
        self.__control_machine = None
        machines = MachineManager.getMachines()
        if machines.has_key (machine_name) : 
            self.__control_machine = machines[machine_name]
        
    def __use_script_NOTUSED (self, control):
        import settings
        import xmlrpclib
        profile = SystemHandler.getGameClass().player_profile
        __rpc_machine = xmlrpclib.ServerProxy(settings.SERV_STATE_MACHINE, transport=settings.proxy )
        self.log("myprofile " + str(profile))
        self.log("mycontroll " + str(control))
        __rpc_machine.setController(profile["account_key"], 'robbie', 2, self.__brain)
    
    def __load_scripts (self) :
        import dircache
        userdir = settings.PATH_USER_DIRECTORY #'../../../scripts/user/'
        loaded_aiscripts = {}
        self.log("Opening user directory '"  + str(userdir) + "'")
        aiscripts=dircache.listdir(userdir)
        GUI.systemGUI.call ("aipyro.swf", "clearScriptList", None)
        for aiscript in aiscripts : 
            if aiscript.endswith(".py") :
                filename = aiscript.replace(".py", "")
                loaded_aiscripts[filename] = None 
                GUI.systemGUI.call ("aipyro.swf", "addScriptItem", filename)
        self.log("Loaded user scripts "  + str(loaded_aiscripts.keys()))

import __vehiclemanager as vehiclemanager
import MachineManager
import ir
import rover
import SceneManager
import unittest
class TestPyroGUI(unittest.TestCase):
    
    control = None
    
    def setUp(self):
        import xmlrpclib
        import settings
        __rpc_state = xmlrpclib.ServerProxy( settings.SERV_STATE_ENTITY )
        profile = __rpc_state.login( "unittest", "unittest" )
        
        machine = MachineManager.Machine()
        machine.name = "MyMachine"
        machine.id = "MyMachine_id"
        MachineManager.add (machine.id, machine, machine.name)
        machine.addDevice ("IR Test Device", ir.Device(SceneManager.addEntity("IR Test Entity", "cube.1m.mesh")))
        self.roverdev = vehiclemanager.getDynamicVehicle (profile["account_key"], "4x2 Terrain Rover x1.2", "robbie")
        machine.addDevice ("Rover Test Device", rover.Device(self.roverdev))
        self.control = ControlAIPyro()
        self.control.setup("UserBrain")
        machine.setControl (self.control)
        self.control.setMachineControl(machine.id)

    def tearDown(self):
        self.roverdev.destroy()
        #self.control.clear()
        
    def testExecuteCommand(self):
        """Control and test executing cmd commands through pyrobot"""
        self.control.receaveEvent (None, "executeScript", "{'arguments':['run']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['runtillquit']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['step']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['info']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['help']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['usage']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['update']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['info']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['!']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['$']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['!!']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['about']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['manual']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['reload']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['load robot']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['load brain']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['load brain NoSuchBrain']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['load brain UserBrain']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['load simulator']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['stop']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['quit']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['exit']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['bye']}")
        #self.control.receaveEvent (None, "executeScript", "{'arguments':['edit']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['a = robot.x']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['watch a']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['watch robot.x']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['unwatch robot.x']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['unwatch a']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['view robot']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['browse robot']}")
        self.control.receaveEvent (None, "executeScript", "{'arguments':['nosuchcommand']}")

