"""
Install pyro : inspired by http://pyrorobotics.org/?page=Pyro3Installation 
"""
__author__ = "Kjartan A Jonsson"
__version__ = "$Revision: 1.7 $"
__copyright__ = "Copyright (c) 2007 aGameCompany"
__license__ = "aGameCompany"

import GUI
import sys
import os
import time
sys.path.append ("..\\..\\..\\scripts\\MachineRace\\modules\\user\\") #add the path to game script
os.environ["PYROBOT"] = "C:\\Development\\Game Development\\Game\\Common\\python\\Lib\pyrobot\\"
import const
import math
import random
import SystemHandler
__state = {"brains":[]}
from pyrobot.robot import Robot
from pyrobot.engine import Engine
from pyrobot.brain import Brain

#####################################################
#    Drivers
#####################################################
from pyrobot.robot.device import *
import pyrobot.gui.console as console
import math

class AIPyroSenseDriver(Device):
    def __init__(self, dev, type = "ir"):
        Device.__init__(self, type)
        self.dev = dev
        self.devData['units']    = "SCALED" # current report units, METERS, 
        self.devData["radius"] = 55 / 1000.0 # meters
        # natural units:
        self.devData["rawunits"] = "METERS"
        self.devData['maxvalueraw'] = 6.0 # in rawunits

        self.devData["count"] = 3

        self.subDataFunc['x'] = self.hitX
        self.subDataFunc['y'] = self.hitY
        self.subDataFunc['z'] = self.hitZ
        self.subDataFunc['value'] = self.getRange
        
        console.log(console.INFO,'AIPyro sense drivers loaded')
        self.startDevice()  

    def hitX(self, pos):
        # convert to x,y relative to robot
        dist = self.getVal(pos)
        angle = (-self.th(pos)  - 90.0) / 180.0 * math.pi
        return dist * math.cos(angle)
        
    def hitY(self, pos):
        # convert to x,y relative to robot
        dist = self.getVal(pos)
        angle = (-self.th(pos) - 90.0) / 180.0 * math.pi
        return dist * math.sin(angle)

    def hitZ(self, pos): return 0.0

    def getRange(self, dev):
        return 0

#####################################################
#    MRO Robot
#####################################################
import ir
class AIPyroRobotException (Exception) : pass
class AIIPyroRobot(Robot): 
    """
    Main defninition
    """
    name = "AIPyroRobot"
    id = "AIPyroRobot"
    machineDrivers = None #all our device drivers as defined in the modules
    def __init__(self, name, machine): 
        Robot.__init__(self) # robot constructor
        if name <> None : self.id = name
        self.__translate = 0
        self.__rotate = 0
        self.devData = {}
        self.stall = 0
        self.x = 0
        self.y = 0
        self.z = 0
        self.th = 0
        self.thr = 0
        self.radius = machine.getSize()
        self.timestamp = time.time()
        #load machine defined devices
        machine_devices = machine.getDevices()
        self.machineDrivers = {}
        for d in machine_devices : 
            if machine_devices[d].device_type == "camera" : pass #the camera does not do anything for the AI scripts so do not register it
            elif machine_devices[d].device_type <> "robot" :
                self.machineDrivers[machine_devices[d].device_type] = machine_devices[d].Driver
                self.builtinDevices.append(machine_devices[d].device_type)
                dev = self.startDevice(machine_devices[d].device_type)
                machine_devices[d].setDriver( dev ) #save it back to our device so we can control it
            else :
                self.devData.update(machine_devices[d].getInterface())
        #load the 'robot' / vehicle stuff
        self.range = self.ir[0]

    def localize(self, x = 0.0, y = 0.0, th = 0.0) : 
        print "AIPyroRobot '" + str(self.id) + "' localize .... ", str([x,y,th])
        
    def move(self, translate, rotate) : 
        """
        Motion control abstractions, abstracts movements in terms of turning and forward/backward changes.
        A programmer only specifies values in a range -1.0..1.0
        """
        if not self.devData.has_key("move") : raise AIPyroRobotException, "AIPyroRobot '" + str(self.id) + "' has not a required 'move' interface."
        self.devData["move"](translate, rotate) #forward the call to the machine that this ai pyro robot controls
        self.__translate = translate
        self.__rotate = rotate
        
    def motors(self, leftpower, rightpower) : 
        """
        Motion control abstractions, movements in terms of apply power to the left and right sides.
        A programmer only specifies values in a range -1.0..1.0
        """
        if not self.devData.has_key("motor") : raise AIPyroRobotException, "AIPyroRobot '" + str(self.id) + "' has not a required 'motor' interface."
        self.devData["motor"](leftpower, rightpower) #forward the call to the machine that this ai pyro robot controls
        
    def translate(self, value) : 
        self.__translate = value
        self.move(self.__translate, self.__rotate)

    def rotate(self, value) : 
        self.__rotate = value
        self.move(self.__translate, self.__rotate)
    
    def enableMotors(self):
        print "AIPyroRobot '" + str(self.id) + "' enable motors  .... "
        pass

    def disableMotors(self):
        print "AIPyroRobot '" + str(self.id) + "' disable motors  .... "
        pass
        
    def update(self) : 
        """
        This is called during update. Here we get data on current status.
        """
        #print "AIPyroRobot '" + str(self.id) + "' updating .... "
        if not self.devData.has_key("getPosition") : raise AIPyroRobotException, "AIPyroRobot '" + str(self.id) + "' has not a required 'getPosition' interface."
        pos = self.devData["getPosition"]() #forward the call to the machine that this ai pyro robot controls
        self.x = pos[0] #global pos
        self.y = pos[2] #global pos
        self.z = pos[1] #height
        #TODO
        self.th = float(self.devData["th"]()) #theta
        self.thr = math.radians(self.th) # theta, in radians
        if not self.devData.has_key("isStall") : raise AIPyroRobotException, "AIPyroRobot '" + str(self.id) + "' has not a required 'isStall' interface."
        self.stall = self.devData["isStall"]() #True if robot is stuck or if the robot is bumping into something
    
    def startDeviceBuiltin(self, item) :
        if not item in self.machineDrivers.keys() : raise AttributeError, "AIPyroRobot '" + str(self.id) + "' does not support device '%s'" % item
        return {item: self.machineDrivers[item](self)}

    def setup(self, **kwargs):
        """
        Is called from __init__ so users don't have to call parent
        constructor and all the gory details.
        """
        print "AIPyroRobot '" + str(self.id) + "' setup .... "        

    def cleandata(self, s):
        #return const.toHTML(str(s))
        return str(s).replace("<","").replace(">","").replace("'","").replace("[","").replace("]","").replace(",","")

    def getView(self, thing = None, toplevel = "robot", indent = 0):
        """A printable representation of the robot's attribute tree. """
        import pyrobot.robot
        if thing == None: thing = self
        dictable = 0
        html = ""
        splitter=":"
        indenter = " "
        self.stdprint = False
        try:
            thing.__dict__
            dictable = 1
        except: pass
        if dictable:
            #print "%s%s:" % (" " * indent, toplevel)
            html = html + ""+toplevel+"="+splitter
            dictkeys = thing.__dict__.keys()
            dictkeys.sort()
            for item in dictkeys:
                if item[0] == indenter :
                    pass # skip it; private
                elif type(thing.__dict__[item]) in [types.FunctionType, types.LambdaType, types.MethodType]:
                    pass # skip it; function
                else:
                    if item in self.devices:
                        count = 0
                        for i in thing.__dict__[item]:
                            html = html + self._displayDevice(i, indent + 3, count)
                            count += 1
                    elif type(thing.__dict__[item]) == type({}): # dict
                        if self.stdprint : print "%s%-15s = {%s}" % (" " * (indent + 3), "." + item, pyrobot.robot.commas(thing.__dict__[item].keys()))
                        html = html + ""+ indenter * (indent + 3)+str(item)+"="+self.cleandata(pyrobot.robot.commas(thing.__dict__[item].keys()))+splitter
                    elif type(thing.__dict__[item]) == type(''): # string
                        if self.stdprint : print "%s%-15s = '%s'" % (" " * (indent + 3), "." + item, thing.__dict__[item])
                        html = html + ""+ indenter * (indent + 3)+str(item)+"="+self.cleandata(thing.__dict__[item])+splitter
                    else:
                        if self.stdprint : print "%s%-15s = %s" % (" " * (indent + 3), "." + item, thing.__dict__[item])
                        html = html + ""+ indenter * (indent + 3)+str(item)+"="+self.cleandata(thing.__dict__[item])+splitter
        else:
            if type(thing) == type(''):
                if self.stdprint : print "%s%-15s = '%s'" % (" " * indent, "." + toplevel, thing)
                html = html + ""+ indenter * (indent + 3)+str(toplevel)+"="+self.cleandata(thing)+splitter
            else:
                if self.stdprint : print "%s%-15s = %s" % (" " * indent, "." + toplevel, thing)
                html = html + ""+ indenter * (indent + 3)+str(toplevel)+"="+self.cleandata(thing)+splitter
        html = html.replace("Ok", "")
        return html
        
    def _displayDevice(self, device, indent = 0, count = 0):
        """Used in print device."""
        if self.stdprint : print "%s[%d]" % (device.type, count)
        toplevel = "%s[%d]" % (device.type, count)
        html = self.getView(device, toplevel, indent)
        return html
        
class AIPyroEngine (Engine) : 
    brainfile = ''
    def __init__(self, name, machine) :
        Engine.__init__(self)
        self.robot = AIIPyroRobot(name, machine)
        self.brainfile = ''
        self.worldfile = "None"

    def reset(self):
        """Shuts down the brain, and reloads it."""
        print "AIPyroEngine.reset - method not implemented"

    def resetFirstAttempts(self):
        """Test method."""
        print "AIPyroEngine.resetFirstAttempts - method not implemented"

    def osRunPython(self, command): 
        print "AIPyroEngine - method not implemented"
  
    def loadSimulator(self, file, worldfile):
        """Finds and loads the simulator."""
        print "AIPyroEngine.loadSimulator - method not implemented"
      
    def loadRobot(self,file):
        """Finds and loads the robot."""
        print "AIPyroEngine.loadRobot - method not implemented"

    def loadBrain(self,file):
        """Finds and loads the brain file."""
        print "AIPyroEngine.loadBrain - method not implemented"
  