from string import atof, atoi
import itertools
import random
import pickle
import GUI
import pyrobot.robot.device

square_width = 20

class Device :
    """
    """
    __driver = None
    machine_group = "map"
    device_type = "gps"    
    gui = None
    body = None
    __robot = None
    def __init__(self, body):
        self.body = body
        self.entity = self.body
        
    def setDriver (self, d) : 
        """
        Called by the Driver when it was created
        """
        self.__driver = d
        self.__driver.setEntity (self.entity)        

    def getDriver (self) : return self.__driver  
        
    def setVisible (self, b) : 
        """This is called when we are un-selecting and selecting the machine"""
        self.__driver.setVisible(b)
        self.gui = self.__driver.gui

    def getGUI (self) : return self.__driver.getGUI()
                	
    class Driver(pyrobot.robot.device.Device): 
        __visual_box_grid = None
        gui = None
        __gui_instance = None #holds the original gui instance
        complete = 0
                
        def __init__(self, dev, type = "gps") :
            pyrobot.robot.device.Device.__init__(self, type)
            self._dev = dev            
            self.__visual_data_grid = None
            self.__show_visual_cells = False
            self.inaccessible = [(13, 2), (10, 4), (8, 6), (9, 5), (9, 4), (8, 5), (8, 4)];
        
            self.path    = []
            self.visited = []
            self.pits    = []
            self.goal    = []
            
            self.goal_id = -1
            self.pit_ids = []
        
            # how many states ?
            self.num_squares_x = 15
            self.num_squares_y = 15
            self.num_squares = max(self.num_squares_x, self.num_squares_y)
            self.square_width = square_width
            self.entity = None
            self.physical_location = None
    
            # various object members
            self.done    = 0
            self.quit    = 0
            self.complete = 0
    
            # initialize the world
            self.initWorld()
            self.resetStates()
            
            # used by simulator
            self.properties = ["location", "obstacles", "goal", "home", \
                               "final", "visited", "complete", \
                               "pits", "path"]

            self.movements = ["up", "right", "down", "left"]
            self.ports = [60000]
            self.__gui_instance = DeviceWindow(self.num_squares)

        def setInaccessiblePosition (self, x, y, b=True):
            """
            Will figure out what cell this is based on world coordinates
            """
            x = int((x-self.physical_center[0])/self.square_width)
            y = int((y-self.physical_center[2])/self.square_width)
            self.setInaccessibleCell (x,y,b)

        def setInaccessibleCell (self, x, y, b=True):
            """
            Will figure out what cell this is based on world coordinates
            """
            grid_pos = x,y
            if (b) : #then add
                if not grid_pos in self.inaccessible :
                    self.inaccessible.append(grid_pos)
                    print "Adding " + str(grid_pos) + " to inaccessible"
            else :
                if grid_pos in self.inaccessible :
                    self.inaccessible.remove(grid_pos)
                else : print "Could not set " + str(grid_pos) + " to inaccessible because it does not exist"
            print "The inaccessible list is now " + str(self.inaccessible)
            self.__updateVisualCells()

        def setEntity (self, entity) :
            self.entity = entity
            self.physical_location = [-465,0,-1080] #self.entity.getPosition(True)
            self.physical_center = self.physical_location
    
        def resetStates(self):
            # various states
            self.home = (int(self.num_squares_x/2), int(self.num_squares_y/2))
            self.location = (int(self.num_squares_x/2), int(self.num_squares_y/2))
           
            num_pits = random.randrange(5) + 1
    
            del self.pits[0:]
    
            for i in range(num_pits):
                self.pits.append((random.randrange(self.num_squares_x), random.randrange(self.num_squares_y)))
                
            #self.goal = []
            if self.goal == [] :
                while self.goal == [] or self.goal in self.inaccessible or self.goal in self.pits:
                    self.goal = (random.randrange(self.num_squares_x), random.randrange(self.num_squares_y))
            print "goal state is at " + str(self.goal)
            self.final_states = [self.goal] + self.pits
    
        def initWorld(self):
            self.completed = 0
            self.location = (int(self.num_squares_x/2), int(self.num_squares_y/2)) #we will always start from the middle
            del self.path[0:]
            del self.visited[0:]
            self.__resetVisualMap()
            
        # checks the current motion, if it is valid the location will be changed, otherwise no change
        def moveInDirection(self, loc, dir):
            x = 0
            if (loc[0] + dir[0], loc[1] + dir[1]) in self.inaccessible:
                x = loc[0]
                y = loc[1]
            else:
                if loc[0] + dir[0] >= 0 and loc[0] + dir[0] < self.num_squares_x:
                    x = loc[0] + dir[0]
                else:
                    x = loc[0]
                if loc[1] + dir[1] >= 0 and loc[1] + dir[1] < self.num_squares_y:
                    y = loc[1] + dir[1]
                else:
                    y = loc[1]
            self.location = (x, y)
    
        # check if we are at the goal state
        def checkMovement(self):        
            xloc = self.location[0]
            yloc = self.location[1]
            if ((xloc, yloc) in self.final_states):
                return "Success!"
            return "ok0"
    
        # removes all elements in path after el
        def removeAfter(self, el, path):
            if len(path) <= 1:
                return path
            path.reverse()
            path = list(itertools.dropwhile(lambda n: not el == n, path))[1:]
            path.reverse()
            return path
    
        # returns a list of all elements in l1 that are not in l2
        def getDifferences(self, l1, l2):
            dl = []
            for e in l1:
                if not(e in l2):
                    dl.append(e)
            return dl
    
        # adds a new location to the current path, and removes new loops in the process
        def addToPath(self, el):
            path = self.path
            # removes loops -- they're not significant
            if len(path) == 0:
                pass
            elif path[len(path)-1] == el:
                return
                self.visited.append(el)
            if el in path:
                path = self.removeAfter(el, path)
            path.append(el)
            if el in self.final_states:
                self.complete = 1
            else:
                self.complete = 0
            self.path = path
    
        def ask (self, what):  return self.process(what)
        def tell (self, what):  return self.process(what)
        
        def showVisualCells (self, b, robot=None) :
            self.__robot = robot 
            self.__show_visual_cells = b
            self.__updateVisualCells()
    
        def showVisualMap (self, b) :
            self.__show_visual_map = b

        def __updateVisualMap (self):
            if self.gui <> None : 
                self.gui.setGridPath (self.path)
                self.updatePosition()

        def updatePosition (self):
            if self.gui <> None :
                if self.entity <> None :
                    self.physical_location = self.entity.getPosition(True)
                    grid_pos = [int((self.physical_location[0]-self.physical_center[0])/self.square_width),int((self.physical_location[2]-self.physical_center[2])/self.square_width)]
                    grid_pos[0] = self.location[0] + grid_pos[0]
                    grid_pos[1] = self.location[1] + grid_pos[1]
                    self.gui.setMachinePosition ([(grid_pos[0],grid_pos[1])])            

        def __resetVisualMap (self):
            import SystemHandler
            import const
            if self.gui <> None :
                self.gui.resetGrid(self.num_squares)
                self.gui.setGridPits (self.pits)
                self.gui.setGridInaccessible (self.inaccessible)                                
                self.gui.setGridGoal (self.goal)
            #NOTE THIS SHOULD NOT BE HERE, BUT IS BECAUSE THE VEHICLE MUST BE AT POSOTION
            #if self.entity <> None : 
            #    self.physical_location = self.entity.getPosition(True)
            #    self.physical_center = self.physical_location
        
        def __updateVisualCells (self):
            import SceneManager
            if self.gui <> None : 
                pass
            if not self.__show_visual_cells : 
                if self.__visual_data_grid == None : return None
                for i in self.__visual_data_grid :
                    e = self.__visual_data_grid[i]
                    e.setVisible(False)
            else :
                if self.__visual_data_grid == None :
                    self.__visual_data_grid = {}
                    for i in range(0, 15) :
                        for j in range(0, 15) :
                            e = SceneManager.addEntity("visual_data_grid_"+str(i)+str(j)+str(random.random()), "cube.1m.mesh")
                            if self.__robot <> None : self.__robot.ir[0].getDevice().addIgnore (e.id)
                            pose = {}
                            pose["position"] = [(i*self.square_width)+(self.physical_center[0]), 0, (j*self.square_width)+(self.physical_center[2])]
                            pose["scale"] = [self.square_width, 320, self.square_width]
                            e.setPosition(pose["position"][0], 0, pose["position"][2])
                            e.setScale(pose["scale"][0], pose["scale"][1], pose["scale"][2])
                            self.__visual_data_grid[(i,j)] = e
                for i in self.__visual_data_grid :
                    e = self.__visual_data_grid[i]
                    e.setVisible(False)
                    e.setMaterial("TransparentCell/General")
                    if i in self.inaccessible : 
                        e.setMaterial("TransparentCell/Inaccessible")
                        e.setVisible(True)
                    if i in self.pits : 
                        e.setMaterial("TransparentCell/Pits")
                        e.setVisible(True)
                    if i in [self.goal] : 
                        e.setMaterial("TransparentCell/Goal")
                        e.setVisible(True)
                    if i in self.visited : 
                        e.setMaterial("TransparentCell/General")
                        e.setVisible(True)
                    if i in self.path : 
                        e.setMaterial("TransparentCell/Path")
                        e.setVisible(True)
       
        
        # process incoming requests
        def process(self, request):
            if request != 'location' and 'location' in request:
                request = 'location'
            elif request != 'moves' and 'moves' in request:
                request = 'moves'
    
            retval = "error"
    
            if request.count('connectionNum'):
                connectionNum, port = request.split(":")
                retval = self.ports.index(int(port))
    
            elif request[0:2] == "c_" :
                position= (atoi(request[2:4]), atoi(request[4:6]))
                color = request[7:]
                retval = color
    
            elif request == 'location':
                self.addToPath(self.location)
                retval = self.location[0], self.location[1]
            elif request == 'complete':
                retval = self.complete
            elif request == 'path':
                retval = self.path
            elif request == 'final':
                retval = self.final_states
            elif request == 'obstacles':
                retval = self.inaccessible
            elif request == 'visited':
                retval = self.visited
            elif request == 'goal':
                retval = (self.goal)
            elif request == 'pits':
                retval = (self.pits)
            elif request == 'home':
                retval = (self.home)
            elif request == 'reset':
                print "RESET!!"
                self.initWorld()
                self.resetStates()
                retval = "reset complete"
            elif request == 'end' or request == 'exit':
                retval = "exiting"
                self.done = 1
            elif request == 'quit':
                retval = "quitting"
                self.done = 1
                self.quit = 1
            elif request == 'properties':
                retval = self.properties
            elif request == 'movements':
                retval = self.movements
            elif request == 'up':
                self.moveInDirection(self.location, (0, -1))
                self.addToPath(self.location)
                retval = self.checkMovement()
                self.__updateVisualMap()
                self.__updateVisualCells()
            elif request == 'right':
                self.moveInDirection(self.location, (1, 0))
                self.addToPath(self.location)
                retval = self.checkMovement()
                self.__updateVisualMap()                
                self.__updateVisualCells()
            elif request == 'left':
                self.moveInDirection(self.location, (-1, 0))
                self.addToPath(self.location)
                retval = self.checkMovement()
                self.__updateVisualMap()
                self.__updateVisualCells()
            elif request == 'down':
                self.moveInDirection(self.location, (0, 1))
                self.addToPath(self.location)
                retval = self.checkMovement()
                self.__updateVisualMap()
                self.__updateVisualCells()
            elif request == 'supportedFeatures':
                retval = []
            elif request == 'builtinDevices':
                retval = []
            elif request == 'start':
                self.complete = 0
                self.initWorld()
                retval = self.location
            else:   # unknown command; returns "error"
                pass
            pickle.dumps(retval)
            return retval
    
        def destroy(self):
            self.done = 1 # stop processing requests, if handing
            self.quit = 1 # stop accept/bind toplevel
            
        def setVisible (self, b) : 
            """This is called when we are un-selecting and selecting the machine"""
            if b : 
                self.gui = self.__gui_instance
                self.__resetVisualMap()
            else :
                self.gui = None
    
        def getGUI (self) : return self.__gui_instance

        def world2Local(self, world_grid) :
            """Takes the local grid and extracts information for local grid"""
            self.inaccessible = []
            self.pits = []
            for w in world_grid.keys() :
                if world_grid[w]["value"] == "inaccessible" : self.inaccessible.append(eval(w))
                if world_grid[w]["value"] == "pits" : self.pits.append(eval(w))
                if world_grid[w]["value"] == "home" : self.home = [eval(w)]
                if world_grid[w]["value"] == "goal" : self.goal = [eval(w)]

        def getGridIndex (self, x, y):
            return (x/self.square_width, y/self.square_width)

from window import Window #get base menu class
import math
class DeviceWindow (Window) :
    """Handles camera menu"""
    lastupdate = 0
    path_tuple = None

    def __init__ (self, cell_count):
        pass
        
    def resetGrid(self, cell_count):
        pass

    def clear (self):
        pass 
    
    def __getFlashGridString (self, tuple_val):
        tuple_val = str(tuple_val)
        tuple_val = tuple_val.replace(",","#")
        if tuple_val.startswith("set") :
            tuple_val = tuple_val[5:len(tuple_val)-2] #here we get set tuple 'set([(1,2), (22,3)])' index 5 to remove 'set([' and index -2 to remove '])'
        else : 
            tuple_val = tuple_val[1:len(tuple_val)-1] #here we get set tuple '[(1,2), (22,3)]' index 1 to remove '[' and index -1 to remove ']'
        return tuple_val        
    
    def setGridPath (self, tuple_val):
        """Takes a tuple and forwards to GUI"""
        pass

    def setGridGoal (self, tuple_val):
        """Takes a tuple and forwards to GUI"""
        pass

    def setGridPits (self, tuple_val):
        """Takes a tuple and forwards to GUI"""
        pass

    def setGridInaccessible (self, tuple_val):
        """Takes a tuple and forwards to GUI"""
        pass

    def setMachinePosition (self, tuple_val):
        """Takes a tuple and forwards to GUI"""
        pass

    def callback (self, source, event, message):
        pass    
    
import SceneManager
import unittest
class TestDevice(unittest.TestCase):

    def setUp(self):
        gpsdev = SceneManager.addEntity ("gpsdevicetest", "cube.1m.mesh")
        self.device = Device (gpsdev)
        self.device.setDriver(Device.Driver(None))

    def testGetGridPosition(self):
        self.assertEqual (self.device.getDriver().getGridIndex(0,0), (0, 0))
        self.assertEqual (self.device.getDriver().getGridIndex(self.device.getDriver().square_width, -self.device.getDriver().square_width), (1, -1))
        self.assertEqual (self.device.getDriver().getGridIndex(-2*self.device.getDriver().square_width, 20*self.device.getDriver().square_width), (-2, 20))

    def testDoWorldGrid2LocalGrid(self):
        world = {}
        for x in range (100,  200) :
        	for y in range (100,  200) :
        		tple = str(x)+", "+str(y)
        		world["("+tple+")"] = {"y":y+10, "x":x+10, "value":None}
        world["(180, 180)"]["value"] = "inaccessible"
        world["(181, 180)"]["value"] = "inaccessible"
        world["(182, 180)"]["value"] = "inaccessible"
        world["(183, 180)"]["value"] = "inaccessible"
        world["(184, 181)"]["value"] = "pits"
        world["(185, 181)"]["value"] = "pits"
        world["(186, 181)"]["value"] = "pits"
        world["(187, 181)"]["value"] = "home"
        world["(188, 181)"]["value"] = "goal"
        #now I want the grid surrounding 180 m, 180 m with a 15 (the center) boxed diameter
        local = self.device.getDriver().world2Local (world)
        self.assertEqual (self.device.getDriver().process("obstacles"), [(181, 180), (182, 180), (180, 180), (183, 180)])
        self.assertEqual (self.device.getDriver().process("pits"), [(186, 181), (185, 181), (184, 181)])
        self.assertEqual (self.device.getDriver().process("home"), [(187, 181)])
        self.assertEqual (self.device.getDriver().process("goal"), [(188, 181)])

    def tearDown(self):
        self.device.setVisible(False)
        #self.device.getGUI().clear()
        SceneManager.delEntity ("gpsdevicetest")

    def testGUIActivate(self):
        """GPS device and test GUI active and unactive"""
        pass
        self.device.setVisible(True)
        self.device.setVisible(False)

    def testGUIUpdateGUIPath(self):
        """GPS device and test GUI path"""
        self.device.getGUI().setGridPath([(1, 1), (1, 1)])
        self.device.getGUI().setGridPath([(1, 1), (1, 2)])
        self.device.getGUI().setGridPath([(1, 1), (2, 1)])        
        self.device.setVisible(True)
        self.device.getGUI().resetGrid(16)
        self.device.getGUI().setGridPath([(1, 1), (1, 1)])
        self.device.getGUI().setGridPath([(1, 1), (1, 2)])
        self.device.getGUI().setGridPath([(1, 1), (2, 1)])
        self.device.setVisible(False)
        self.device.getGUI().setGridPath([(1, 1), (1, 1)])
        self.device.getGUI().setGridPath([(1, 1), (1, 2)])
        self.device.getGUI().setGridPath([(1, 1), (2, 1)])  
        
    def testGUIUpdateGUIGrid(self):
        """GPS device and test GUI path"""
        self.device.getGUI().setGridPath([(1, 1), (1, 1)])
        self.device.getGUI().setGridGoal([(12, 12)])
        self.device.getGUI().setGridPits([(2, 2), (2, 3), (2, 4), (3, 3)])
        self.device.getGUI().setGridInaccessible([(12, 2), (12, 3), (12, 4), (13, 3)])
        self.device.getGUI().setGridPath([(12, 12), (2, 2)])
        self.device.setVisible(True)
        self.device.getGUI().setGridPath([(1, 1), (2, 1)])
        self.device.getGUI().setGridGoal([(12, 12)])
        self.device.getGUI().setGridPits([(2, 2), (2, 3), (2, 4), (3, 3)])
        self.device.getGUI().setGridInaccessible([(12, 2), (12, 3), (12, 4), (13, 3)])        
        self.device.getGUI().resetGrid(16)
        self.device.getGUI().setGridPath([(1, 1), (2, 1)])
        self.device.getGUI().setGridGoal([(12, 12)])
        self.device.getGUI().setGridPits([(2, 2), (2, 3), (2, 4), (3, 3)])
        self.device.getGUI().setGridInaccessible([(12, 2), (12, 3), (12, 4), (13, 3)])
        self.device.getGUI().setGridPath([(12, 12), (2, 2)])        
        self.device.setVisible(False)
        self.device.getGUI().setGridPath([(1, 1), (2, 1)])          
        self.device.getGUI().setGridGoal([(12, 12)])
        self.device.getGUI().setGridPits([(2, 2), (2, 3), (2, 4), (3, 3)])
        self.device.getGUI().setGridInaccessible([(12, 2), (12, 3), (12, 4), (13, 3)])
        self.device.getGUI().setGridPath([(12, 12), (2, 2)])
        #self.device.getGUI().resetGrid(16)
        