#####################################################
#    Drivers
#####################################################
import os
os.environ["PYROBOT"] = "C:\\Development\\Game Development\\Game\\Common\\python\\Lib\pyrobot\\"

import GUI
import SceneManager
import pyrobot.robot.device
import pyrobot.gui.console as console
import math
from pyrobot.geometry import PIOVER180

class Device :
    """
    This is a driver interfaces that handles lower level management of the infrared device.
    """
    machine_group = "sensor"
    device_type = "ir"
    x = 0
    y = 0
    z = 0
    entity = None #the entity we use to do intersection testing
    __driver = None
    __sensored_cd = {} #is the sensors index from 0 to n
    __index = {} #is an index looking up sensorname and looking up the index
    __ignore = []
    __gui_instance = None #holds the original gui instance
    gui = None
    
    def __init__(self, entity) : 
        print "IR driver initalizing"
        self.__ignore = []
        self.entity = entity
        self.gui = None
        #build the sensor index
        sensors = self.entity.getChildren()
        for i in range(0, len(sensors)) : 
            self.__index[sensors[i]] = i
            self.__sensored_cd[i] = []

    def setVisible (self, b) : 
        """This is called when we are un-selecting and selecting the machine"""
        if self.__driver <> None : 
            self.__driver.setVisible (b)
            self.gui = self.__driver.gui

    def setDriver (self, d) : 
        """
        Called by the Driver when it was created
        """
        self.__driver = d
        self.__driver.setDevice(self)
    
    def addIgnore (self, name) : self.__ignore.append(name)
    
    def getGUI (self) : return self.__driver.getGUI()
    
    def _addIntersection (self, sensorname, dic) :
        """
        Internal do not call: used by MachineRaceOnline intersection call-back notefeing an intersection
        """
        dic = eval(dic)
        intersection = dic.values()[0]
        objectname = dic.keys()[0]
        if objectname in self.__ignore : return None
        #get the actual object
        try :
            o = SceneManager.getEntity (objectname)
            index = self.__index[sensorname] #gets the index of the sensor in our device sensor list
            if intersection : 
                if not o in self.__sensored_cd[index] :
                    self.__sensored_cd[index].append(o)
            else : 
                self.__sensored_cd[index].remove(o)
            self.__driver._deviceStateUpdate(self.__sensored_cd, self.entity)
        except SceneManager.SceneError : return #if it was not created by scene manager we can ignore it

    class Driver(pyrobot.robot.device.Device):
        __sensored_cd = None #the entity list created by the device
        instance = None
        gui = None
        __device = None
        
        def __init__(self, dev, type = "ir"):
            pyrobot.robot.device.Device.__init__(self, type)
            self._dev = dev
            self.units    = "M" # current report in meters
            self.radius = dev.radius # meters
            # natural units:
            self.rawunits = "RAW"   #TODO
            self.maxvalueraw = 1.0 #TODO 
            self.count = 8
            self.arc = 0 #TODO
            self.groups = {'all': range(8),
                           'front' : (2, 3), 
                           'front-left' : (0, 1), 
                           'front-right' : (4, 5),
                           'front-all' : (1, 2, 3, 4),
                           'left' : (0, ), 
                           'right' : (5, ), 
                           'left-front' : (1, ), 
                           'right-front' : (4, ), 
                           'left-back' : (7, ), 
                           'right-back' : (6, ), 
                           'back-left' : (7, ), 
                           'back-right' : (6, ), 
                           'back-all' : (6, 7), 
                           'back' : (6, 7)} 

            console.log(console.INFO,'ir sense driver loaded')
            self.startDevice()  
            console.log(console.INFO,'ir sense driver started')
            self.__gui_instance = DeviceWindow()

        def setDevice (self, d) : self.__device = d
        
        def getDevice (self) : return self.__device

        def setVisible (self, b) :
            if b : 
                self.gui = self.__gui_instance
                for i in range(0, self.count) : 
                    self.gui.setSensorReading(i, 1.0)
            else :
                self.gui = None

        def getGUI (self) : return self.__gui_instance

        def _deviceStateUpdate(self, tracking, instance) :
            """
            Notifies on what new objects we should be tracking
            """
            self.__sensored_cd = tracking
            self.instance = instance
            
        def _getVal (self, pos): 
            """
            TODO:
            return the value of our sensor at pos 
            """
            mindist = self.maxvalueraw #assume that no object is near
            if self.__sensored_cd == None : return mindist #no update has been made
            sensors = self.instance.getChildren()
            instance = SceneManager.getEntity(sensors[pos]) #get the sensor entity to do the checking with
            #now we just return the minimum distance
            for e in self.__sensored_cd[pos] :
                p = e.getPosition()
                m = instance.getDistanceTo(e.id)
                if m['distance'] <> None : 
                    dist = m['distance']
                    mindist = min(dist, mindist)
            if self.gui <> None : self.gui.setSensorReading(pos, mindist)
            return mindist

        def _ox(self, pos):
            # in mm
            if pos == 0:
                retval = 20.0
            elif pos == 1:
                retval = 40.0
            elif pos == 2:
                retval = 60.0
            elif pos == 3:
                retval = 60.0 
            elif pos == 4:
                retval = 40.0
            elif pos == 5:
                retval = 20.0
            elif pos == 6:
                retval = -60.0
            elif pos == 7:
                retval = -60.0
            return retval
        
        def _oy(self, pos):
            # in mm
            if pos == 0:
                retval = 60.0
            elif pos == 1:
                retval = 40.0
            elif pos == 2:
                retval = 20.0
            elif pos == 3:
                retval = -20.0 
            elif pos == 4:
                retval = -40.0
            elif pos == 5:
                retval = -60.0
            elif pos == 6:
                retval = -20.0
            elif pos == 7:
                retval = 20.0
            return retval
        
        def _thr(self, pos):
            return self._th(pos) * PIOVER180
        
        def _th(self, pos):
            if pos == 0:
                return 90.0
            elif pos == 1:
                return 45.0
            elif pos == 2:
                return 0.0
            elif pos == 3:
                return 0.0 
            elif pos == 4:
                return -45.0
            elif pos == 5:
                return -90.0
            elif pos == 6:
                return 180.0
            elif pos == 7:
                return 180.0

        def getSensorValue(self, pos):
            """
            Should be overloaded by device implementations to return a
            SensorValue object.
            Send sensor device, dist, pos, geometry (ox, oy, oz, thr, arc).
            """
            value = pyrobot.robot.device.SensorValue(self, self._getVal(pos), pos,
                               (self._ox(pos) / 10.0, # meters
                                self._oy(pos) / 10.0, # meters
                                20.0 / 10.0, # meters
                                self._thr(pos),
                                self.arc),)
            return value

from window import Window #get base menu class
import math
class DeviceWindow (Window) :
    """Handles IR menu"""
    lastupdate = 0
    lastvalues = {}
    def __init__ (self):
    	import const
        self.log_attempt ("register IR interface")
        try : 
            GUI.systemGUI.loadLayout ("ir.swf", 256, 256)
            GUI.systemGUI.setPosition ("ir.swf", 0, const.WINDOW_POSTION_DELTA_HEIGHT*3)
        except Exception, e: self.log_failure (False, str(e))
        self.log_success ()
    
    def setSensorReading (self, nr, value) : 
        GUI.systemGUI.call ("ir.swf", "setSensor"+str(nr), value)
        self.lastvalues[nr] = value #save for debugin purposes
    
    def clear (self):
        GUI.systemGUI.delLayout ("ir.swf")
    
    def callback (self, source, event, message):
        pass    
        
        
import unittest
class TestDevice(unittest.TestCase):
    
    def setUp(self):
        irdev = SceneManager.addEntity ("myirdevicetest", "cube.1m.mesh")
        for i in range(0,8) :
    	    sensor = SceneManager.addEntity("_ir_"+str(i), "cube.1m.mesh")
    	    irdev.attach(sensor)
        self.device = Device (irdev)

    def tearDown(self):
        for i in range(0,8) :
    	    sensor = SceneManager.delEntity("_ir_"+str(i))
        SceneManager.delEntity ("myirdevicetest")
        #self.device.getGUI().clear()

    def testGUIActivate(self):
        """IR device and test GUI active and unactive"""
        self.device.setVisible(True)
        self.device.setVisible(False)

    def testGUITest(self):
        """IR device and test GUI set values"""
        pass
        
