"""
Handles menu including console
"""
__author__ = "Kjartan A Jonsson"
__version__ = "$Revision: 1.20 $"
__copyright__ = "Copyright (c) 2007 aGameCompany"
__license__ = "aGameCompany"

import GUI
import xmlrpclib
import settings

from window import Window

import const
import MachineManager
import SystemHandler

import rover
import ir

__view__ = {"menu":None} #the menu view
__modl__ = {} #dictionary of modules

def log (message, a, b, c) : print "REDIRECT THIS MESSAGE " + message

def callback (source, event, message):
    """Will pass call-back to next menu"""
    if source == None and message == "authentication done" :
            __view__["menu"][__view__["current"]].clear()
            __view__["current"] = "machines" #set the new menu interface to be machines interfaces (was login)
            __load_device_view ()
            return None
    if __view__["menu"] <> None :
        current = __view__["current"]
        if __view__["menu"].has_key (current) :
            __view__["menu"][current].callback(source, event, message)
        else :
            log ("Menu having name " + current + " not loaded", LOG_ERROR)

def keyboard_callback (bKeyDown, strKey) :
    if bKeyDown :
        if strKey == "ESCAPE" : 
            __view__["view"].destroyModules()
        callback (const.MENU_CALLBACK_KEYBOARD, bKeyDown, strKey)

class MenuLogin (Window) :
    __module = None
    def __init__ (self):
        import authenticate
        self.__menu = authenticate.open()
        
    def clear (self):
        self.__menu.clear()

    def callback (self, source, event, message): 
        return self.__menu.callback (source, event, message)


class MenuMachines (Window) :
    """This menu is the base menu that shows all the machines that the user can switch between"""
    __module = None
    __machine_active = None
    
    def __init__ (self):
        SystemHandler.keyEnabled("F1", True)
        SystemHandler.keyEnabled("F2", True)
        SystemHandler.keyEnabled("F3", True)
        SystemHandler.keyEnabled("F4", True)
        SystemHandler.keyEnabled("F5", True)
        SystemHandler.keyEnabled("F6", True)
        SystemHandler.keyEnabled("F7", True)
        SystemHandler.keyEnabled("F8", True)
        SystemHandler.keyEnabled("F9", True)                
        GUI.systemGUI.loadLayout ("_machines.swf", 256, 256)
        GUI.systemGUI.setPosition ("_machines.swf", 256, 0)
        pass #KAJ NOTE: GUI.systemGUI.loadLayout ("_news.swf", 512, 512)
        SystemHandler.timerCallBack(1000, const.MENU_CALLBACK_TIMER, "menu.callback PENDING MACHINES")
        SystemHandler.timerCallBack(500, const.MENU_CALLBACK_TIMER, "menu.callback GET NEWS")
        GUI.systemGUI.addEventSubscriber ("_machines.swf", "selectMachine", "receaveMenuEvent", "None")
        self.__machine_active = None
        
    def clear (self):
        pass

    def setActiveMachine (self, name):
        machines = MachineManager.getMachines()
        selected_machine = None
        for machine_name in machines :
            devices = machines[machine_name].getDevices()
            for device_name in devices :
                devices[device_name].setVisible(False)
        devices = machines[name].getDevices()
        for device_name in devices : devices[device_name].setVisible(True)
        for machine_name in machines : machines[machine_name].control.setMachineControl(name)
        self.__machine_active = machines[name]

    def callback (self, source, event, message): 
        if source == const.MENU_CALLBACK_KEYBOARD :
            if message[0] == 'F' and len(message) == 2: 
                index = int(message[1])-1
                machines = MachineManager.getMachines().values()
                GUI.systemGUI.call ("_machines.swf", "setMachine", machines[index].name) #notifies the menu name so that we can make it current
                self.setActiveMachine (machines[index].id)
        elif source == "_machines.swf" and event == "selectMachine" :
            message = eval(message)
            machine = message['arguments'][0]
            machines = MachineManager.getMachines()
            for machine_name in machines : #get the actual id from nice machine name
                if machines[machine_name].name == machine : self.setActiveMachine (machines[machine_name].id)
        elif source == None and event == const.MENU_CALLBACK_TIMER and message == "menu.callback GET NEWS":
            pass #KAJ NOTE: GUI.systemGUI.call ("_news.swf", "getUserNews", "http://"+settings.SERV_STATE+"/menu-news/"+settings.profile["contractor_key"]+"/")
        elif source == None and event == const.MENU_CALLBACK_TIMER and message == "menu.callback PENDING MACHINES":
            machines = MachineManager.getMachines()
            if len (machines) == 0 : 
                SystemHandler.timerCallBack(1000, const.MENU_CALLBACK_TIMER, "menu.callback PENDING MACHINES")
            else :
                for m in machines : 
                    GUI.systemGUI.call ("_machines.swf", "addMachine", machines[m].name) #use nice machine name
                return None
        else : #if we did not catch menu events or timer events it gets passed to the device windows
            if self.__machine_active <> None : 
                devices = self.__machine_active.getDevices() 
                for device_name in devices :
                    if devices[device_name].gui <> None :
                        devices[device_name].gui.callback (source, event, message)

def create () :
    __view__["menu"] = {}
    __view__["menu"]["login"] = MenuLogin()
    __view__["current"] = "login"

def destroy () :
    writelog ("Layout unloaded ... OK")

class MachineDeviceView (Window) :
    _modules_ = {}

    def __init__ (self, device_list) : 
        self.module_names = device_list

    def setupModules (self) : 
        """
        First thing called after registration.
        1. get user module list
        2. initiate module list (errors output to cmd)
            a. will compile the modules (use default if none defined) - ok
            b. load gui - ok
            c. register to this class - ok
            d. do minor tests (module specific) - ok
            e. notify that it is ready - ok
            f. initiate, will set to ready state - ok
        3. notify done
        """
        pass

    def callback (self, source, event, message): 
        print "Doing the callback for MACHINEDEVICE VIEW"
        
    def destroyModules (self) : 
        """
        First thing called when game system is destroyed
        1. all user module list
        2. shutdown module list (errors output to cmd)
            a. do shutdown sequence
            b. un register this class
            c. un load gui            
            d. notify that it is done
        3. notify done        
        N. will do __main__.py stuff (this will be buried down to C++ level in the future)        
        """
        pass

def __load_device_view ():
    """
    Will load all device windows for an view
    """
    view = MachineDeviceView(["rover", "ir", "gps", "radio"])
    view.setupModules()
    __view__["view"] = view
    __view__["menu"]["machines"] = MenuMachines()