import GUI
import pyrobot.robot.device

class DeviceRadioException (Exception) : pass

class Device :
    """
    """
    __driver = None
    machine_group = "communication"
    device_type = "radio"    
    gui = None
    machine_name = None
    
    def __init__(self, entity, machine_name):
        self.entity = entity
        self.machine_name = machine_name
        
    def setDriver (self, d) : 
        """
        Called by the Driver when it was created
        """
        self.__driver = d
        self.__driver.setEntity (self.entity, self.machine_name)        

    def getDriver (self) : return self.__driver  
        
    def setVisible (self, b) : 
        """This is called when we are un-selecting and selecting the machine"""
        self.__driver.setVisible(b)
        self.gui = self.__driver.gui

    def getGUI (self) : return self.__driver.getGUI()
                    
    class Driver(pyrobot.robot.device.Device): 
        
        gui = None
        __gui_instance = None #holds the original gui instance
        machine_name = None
        __services = ["request reset", "may-day stall"]#, "request transport"] WE NEED INPUT DATA FOR TRANSPORT GET THAT FROM GPS
        
        def __init__(self, dev, type = "radio") :
            pyrobot.robot.device.Device.__init__(self, type)
            self._dev = dev            
            self.entity = None
            self.__gui_instance = DeviceWindow(self)
        
        def setEntity (self, entity, machine_name) :
            """Sets the entity so that we can control it"""
            self.entity = entity
            self.machine_name = machine_name
            
        def setVisible (self, b) : 
            """This is called when we are un-selecting and selecting the machine"""
            if b : 
                self.gui = self.__gui_instance
            else :
                self.gui = None
    
        def getGUI (self) : return self.__gui_instance

        def setMachinePosition (self, x, y, z, delta=False) : 
            import MachineManager
            if self.gui == None : return 
            basedev = MachineManager.get (self.machine_name).getBase()
            if basedev == None : raise Exception, "Could not load the machine base device. Base device is required for services"
            if delta : 
                p = basedev.getPosition()
                basedev.setPosition (p[0]+x, y, p[2]+z)
            else :
                basedev.setPosition (x, y, z)
            basedev.setOrientation(1,0,0,0)

        def getServices (self) : return self.__services 

        def callService (self, name, data=None):
            """
            This call executes the service request as is. Future implementations will forward to 
            a call that is hidden and notifies server about the request (billing accordingly).
            Then queues the execution in a call-back that will execute the service.
            """
            if name == "request reset" or name == "may-day stall" :
                self.setMachinePosition (0, 0, 0, delta=True)
            elif name == "request transport" : #moves robot from current position to position defined by data
                if data <> None :
                    if data.has_key("x") and data.has_key("y") and data.has_key("z") : 
                        self.setMachinePosition (data["x"], data["y"], data["z"], delta=True)
                        return
                raise DeviceRadioException, "Service call failed because service '" +name+ "' is not supported"
            elif name == "request move-to" : #moves robot from current position to position defined by data
                if data <> None :
                    if data.has_key("x") and data.has_key("y") and data.has_key("z") : 
                        self.setMachinePosition (data["x"], data["y"], data["z"], delta=False)
                        return
                raise DeviceRadioException, "Service call failed because service '" +name+ "' is not supported"
            else : raise DeviceRadioException, "Service call failed because service '" +name+ "' is not supported"

from window import Window #get base menu class
class DeviceWindow (Window) :
    """Handles camera menu"""
    __driver = None
    
    def __init__ (self, driver):
    	import const
        self.__driver = driver
        self.log_attempt ("register Radio interface")
        try : 
            GUI.systemGUI.loadLayout ("radio.swf", 256, 256)
            GUI.systemGUI.setPosition("radio.swf", 0, const.WINDOW_POSTION_DELTA_HEIGHT*2)
            GUI.systemGUI.addEventSubscriber ("radio.swf", "executeService", "receaveMenuEvent", "execute event")
            GUI.systemGUI.call ("radio.swf", "clearLists", " ")

            services = self.__driver.getServices()
            for service in services : 
    	           GUI.systemGUI.call ("radio.swf", "addService", service)
        except Exception, e: self.log_failure (False, str(e))
        self.log_success ()
        
    def clear (self):
        GUI.systemGUI.delLayout ("radio.swf")

    def callback (self, source, event, message):
    	arg = eval(message)
        if event == "executeService" : 
            self.__driver.callService(arg["arguments"][0])    
    
import SceneManager
import MachineManager
import __vehiclemanager as vehiclemanager
import unittest
import rover

class TestDevice(unittest.TestCase):
    entity = None

    def setUp(self):
        import xmlrpclib
        import settings
        __rpc_state = xmlrpclib.ServerProxy( settings.SERV_STATE_ENTITY, transport=settings.proxy )
        profile = __rpc_state.login( "unittest", "unittest" )

        self.machine = MachineManager.Machine()
        self.machine.name = "MyMachine.w.radio"
        self.machine.id = "MyMachine.w.radio_id"
        MachineManager.add (self.machine.id, self.machine, self.machine.name)

        self.rover = vehiclemanager.getStaticVehicle (profile["account_key"], "4x2 Terrain Rover x1.2", "robbie")
        self.roverdev = rover.Device (self.rover)
        self.machine.addDevice("robbie", self.roverdev)
        
        self.entity = SceneManager.addEntity ("radiodevicetest", "cube.1m.mesh")
        self.radiodev = Device (self.entity, self.machine.id)
        self.radiodev.setDriver(Device.Driver(None))
        self.machine.addDevice ("radiodevicetest", self.radiodev)

    def tearDown(self):
        self.radiodev.setVisible(False)
        self.radiodev.getGUI().clear()
        self.rover.destroy()
        SceneManager.delEntity ("radiodevicetest")

    def testGUIActivate(self):
        """Radio device and test GUI active and unactive"""
        self.radiodev.setVisible(True)
        self.radiodev.setVisible(True)

    def testCallServices(self):
        """Radio device and test use service"""
        data = {"x":0, "y":0, "z":0}
        services = self.radiodev.getDriver().getServices()
        driver = self.radiodev.getDriver()
        for service in services :
            self.radiodev.getDriver().callService(service, data)
        self.radiodev.getDriver().callService("request transport", data)
        self.assertRaises(DeviceRadioException, driver.callService, "nosuchservice", data)

    def testSetPosition(self):
        """Radio device and test use service"""
        self.assertEqual(self.rover.getPosition(), [0,0,0])
        self.radiodev.getDriver().setMachinePosition (1,2,3)
        self.assertEqual(self.rover.getPosition(), [0,0,0])
        self.radiodev.setVisible(True)
        self.radiodev.getDriver().setMachinePosition (1,2,3)
#        self.assertEqual(self.rover.getPosition(), [1,2,3]) KAJ NOTE: DOES NOT WORK BECAUSE THE ROVER POSITION IS NOT UPDATED SOMEHOVE, IT WORKS IN GAME??
#        self.radiodev.getDriver().setMachinePosition (-1,2,-3) 
#        self.assertEqual(self.rover.getPosition(), [-1,2,-3])
#        self.radiodev.getDriver().setMachinePosition (1,-2,4)
#        self.assertEqual(self.rover.getPosition(), [1,-2,4])
        self.radiodev.setVisible(False)
#        self.radiodev.getDriver().setMachinePosition (10,10,10)
#        self.assertEqual(self.rover.getPosition(), [1,-2,4])
        