"""
Rover driver
"""
###################################################
#GUI information flow works as follows: 
#driver always output to gui instance. If the gui instance is None then we do not output to that instance.
#We restore the instance through setVisible(True/False) from __gui_instance
###################################################

__author__ = "Kjartan A Jonsson"
__version__ = "$Revision: 1.2 $"
__copyright__ = "Copyright (c) 2007 aGameCompany"
__license__ = "aGameCompany"

import PhysicsManager
import SystemHandler

class Device :
	"""
	This is a driver interfaces that handles lower level management of rover vehicle.
	"""
	entity = None #this is set by __core.__build_device_vehicle
	__th = None
	gui = None
	device_type = "robot"
	machine_group = "vehicle"
	x = 0
	y = 0
	z = 0
	instance = None #the instance we control / "drive"
	__input_state = {"Name":None, "Gas":str( False ).lower(), "Brake":str( False ).lower(), "Left":str( False ).lower(), "Right":str( False ).lower()}
	__driver = None
	__gui_instance = None
			
	def __init__(self, instance) : 
		print "rover driver starting"
		self.instance = instance
		self.__input_state["Name"] = self.instance.id
		self.__driver = self.Driver(instance)
		self.__gui_instance = DeviceWindow()
		
	def setVisible (self, b) : 
		if b : self.gui = self.__gui_instance
		else : self.gui = None

	def setPosition (self, x, y, z) :
		self.__driver.setPosition(x, y, z)
		if self.gui <> None : self.gui.setPosition(x, y, z)
		
	def getPosition (self) :
		p = self.__driver.getPosition()
		if self.gui <> None : self.gui.setPosition(p[0], p[1], p[2])
		return p
	
	def setOrientation (self, w, x, y, z): 
		self.instance.setOrientation(w, x, y, z)
		r = self.__driver.getOrientation()
		if self.gui <> None : self.gui.setRotation(r[0],r[1],r[2])
		
	def getOrientation(self):
		r = self.__driver.getOrientation()
		if self.gui <> None : self.gui.setRotation(r[0],r[1],r[2]) 
		return r
	def getRotation(self):
		r = self.__driver.getRotation() 
		if self.gui <> None : self.gui.setRotation(r[0],r[1],r[2])
		return r
	def getTh(self):
		th = self.__driver.getTh()
		if self.gui <> None : self.gui.setTheta (th)
		return th
	def move(self, translation, rotation) : 
		if self.gui <> None : self.gui.setMove (translation, rotation)
		self.__driver.move(translation, rotation)
	def motor(self, left, right) :
		if self.gui <> None : self.gui.setMotor (left, right)
		self.__driver.motor(left, right)
		
	def isStall (self) : return self.__driver.isStall()
		
	def getDriver(self) : return self.__driver
	def getGUI (self) : return self.__gui_instance		
	def getInterface (self):
		"""
		Returns device interface
		"""
		return {"move": self.move, "motors": self.motor, "x": self.x, "y": self.y, "z": self.z, "getPosition":self.getPosition, "getRotation":self.getRotation, "th":self.getTh, "isStall":self.isStall }

	class Driver : 
		"""
		The robot driver is a special case so we do not need to override the call pyrobot.robot.device.Device
		"""
		__last_position = [0,0,0]
		__last_rotation = [0,0,0,0]
		__translation = [1,1,1]
		__orientation = [1,1,1,1]
		__stall_possible = 0
		__is_stall = False
		
		def __init__ (self, instance) :
			self.instance = instance
			self.__th = None
			
		def isStall (self) : return self.__is_stall
		
		def doStall (self, p=None, o=None):
			"""
			Calculates if we are stalled
			"""
			if p == None : p = self.instance.getPosition()
			if o == None : o = self.instance.getOrientation()
			for i in range(0, 3) : self.__translation[i] = p[i] - self.__last_position[i]
			for i in range(0, 4) : self.__orientation[i] = o[i] - self.__last_rotation[i]
			self.__last_position = p
			self.__last_rotation = o

			sum = 0
			for i in range(0, 3) : sum = sum + self.__translation[i]
			for i in range(0, 4) : sum = sum + self.__orientation[i]
			if sum < 0.2 : self.__stall_possible = self.__stall_possible + 1
			else : self.__stall_possible = 0
			
			if self.__stall_possible > 100 : self.__is_stall = True
			else : self.__is_stall = False 
			
		def move (self, translation, rotation) :
			"""
			Moves a vehicle using translation and rotation where
			- translation is along the vehicles z axis having value (-1 ... 1)
			- rotation is in the y axis having negative to positive 1 and -1 value, max rotation is vehicle dependent. Max rotation is 1 (Max value == 1)
			"""
			brake=0.0
			if translation == 0 : brake = 1.0
			self.__input_state = {"Relative":str(True).lower(), "Name":self.instance.id, "Gas":translation, "Brake":brake, "Rotate":rotation}
			PhysicsManager.setCar( self.instance.id, self.__input_state )
			self.doStall()
	
		def motor (self, right, left): 
			print "rover driver motor", str(right), str(left)
			self.doStall()
		
		def setPosition (self, x, y, z): self.instance.setPosition (x, y, z)
		def getPosition (self): return self.instance.getPosition() 
		def setOrientation (self, w, x, y, z): self.instance.setOrientation(w, x, y, z)
		def getOrientation (self): return self.instance.getOrientation()
		def getRotation (self): return self.instance.getRotation()
		def getTh (self): 
			self.__th = float(self.getRotation()[1])
			return self.__th	
import GUI		

from window import Window #get base menu class
class DeviceWindow (Window) :
	def __init__ (self):
		pass
	def setRotation (self, x,y,z):
		pass
	def setPosition (self, x,y,z):
		pass
	def setTheta (self, th):
		pass
	def setMove (self, a, b):
		pass
	def setMotor (self, a,b):
		pass
	def clear (self):
		pass
	def callback (self, source, event, message):
		pass	
		
import unittest
import __vehiclemanager as vehiclemanager
class TestDevice(unittest.TestCase):
    
    def setUp(self):
    	import xmlrpclib
    	import settings
    	__rpc_state = xmlrpclib.ServerProxy( settings.SERV_STATE_ENTITY, transport=settings.proxy )
    	profile = __rpc_state.login( "unittest", "unittest" )
        self.rovdev = vehiclemanager.getDynamicVehicle (profile["account_key"], "4x2 Terrain Rover x1.2", "robberto") #SceneManager.addEntity ("myroverdevicetest", "cube.1m.mesh")
        self.device = Device (self.rovdev)

    def tearDown(self):
        self.rovdev.destroy()
        #self.device.getGUI().clear()

    def testStall (self) :
    	"""Rover and test is stall""" 
    	for i in range(0, 100) :
    		self.device.getDriver().doStall ([i, 0, 0])
    		self.assertEqual (False, self.device.getDriver().isStall())
    	for i in range(0, 100) : #because after 10 times of null we know that it is stalled
    		self.device.getDriver().doStall ([0, 0, 0], [i, 0, 0, 0])
    		self.assertEqual (False, self.device.getDriver().isStall())
    	for i in range(0, 100) : #because after 10 times of null we know that it is stalled
    		self.device.getDriver().doStall ([0, 0, 0], [0, 0, 0, 0]) #prepare to get stalled
    		self.assertEqual (False, self.device.getDriver().isStall())    		
    	for i in range(0, 100) : #here the we should be stalled
    		self.device.getDriver().doStall ([0, 0, 0], [0, 0, 0, 0])
    		self.assertEqual (True, self.device.getDriver().isStall())
    		
    def testGUIActivate(self):
        """Rover device and test GUI active and unactive"""
        self.device.setVisible(True)
        self.device.setVisible(False)

    def testGUIValues(self):
        """Rover device and test GUI active and unactive"""
        self.device.setVisible(True)
        self.device.getPosition()
        self.device.getRotation()
        self.device.getOrientation()
        self.device.getTh()
        self.device.setVisible(False)
        self.device.getPosition()
        self.device.getRotation()
        self.device.getOrientation()
        self.device.getTh()        
        