"""
	Author: Kjartan Akil Jonsson, aGame Company
	Brain script learning to reach a goal on top of a mountain.
	This script uses RLtoolkit and is based on code originally by Rich Sutton
    
"""

# This was taken from example program in RLtoolkit named mountainDemoG 
# (mountainAgent and mountaintEnv) for reinforcement learning with linear function
# approximation. The learning algorithm used is sarsa(lambda) with traces.
# Written by Rich Sutton 12/17/00
# Converted to Python in April, 2004
# Modified to use the RL interface in April 2004
# Modified for Machine Race May 2009

# For this example we need some system tools
# Import scene and system tools as well as Pyrobot brain

import SceneManager 
import SystemHandler
from pyrobot.brain import Brain

total_episode_reward = 0    #collect rewards for statistics
prefered_th = 90            #this is the prefered vehicle rotation against the mountain
from RLtoolkit.RLinterface import RLinterface
from RLtoolkit.Tiles.tiles import *
from RLtoolkit.traces import *
from RLtoolkit.utilities import *

#################################################################################################
# Here begins the code that is taken from RLToolkit\examples\mountainEnv.py
# Changes to the original code is noted using KAJ NOTE
#################################################################################################
# Mountain car environment
#  functions getStats -> steps, episodes, episodesteps
#        and getState -> position, velocity, action       return current globals
# RL interface function is mountainEnv(a=None)
#   use   rli = RLinterface(mountainAgent, mountainEnv)
# initialize with setupEnvsetupEnv()

from math import *
from RLtoolkit.utilities import *

minPosition = 120 			# Minimum car position, KAJ NOTE: was -1.2 denoting max position in the curve
maxPosition = 320 			# Maximum car position (past goal), KAJ NOTE: was 0.6 denoting bottom of curve
maxVelocity = 10.07			# Maximum velocity of car, KAJ NOTE: was 0.07
goalPosition = 0.5			# Goal position - how to tell we are done, KAJ NOTE: not used
velocity = 0.0
position = 0.0
action = 1
steps = episodesteps = episodes = 0
randomStarts = False

def mountainEnv(a=None):
    if a == None:               # start of new episode
        return s0()             # return initial sensation
    else:                   
        s, r = do(a)            # do action
        return s, r             # return new sensation and state
	    
def setupEnv():
    global steps, episodesteps, episodes, velocity, position, action
    velocity = 0.0
    position = 0.0
    action = 1
    steps = episodesteps = episodes = 0

def s0():
    "Returns initial state of mountain car for a new episode"
    global steps, position, velocity, episodes, episodesteps, action, data
    data = [[0 for i in range(resolution)] for j in range(resolution)]    #KAJ NOTE: added to populate data
    #lasts, lastr, action = None, 0, 1
    action = 1
    if steps != 0:
        episodes += 1
    episodesteps = 0
    position, velocity = mCarInit()
    #print "Start of episode, sensation is", position, velocity
    return position, velocity
    
time_since_last_flush = 0    
def do(a):
    "Does the specified action and returns a new state and reward"
    global action, total_episode_reward, time_since_last_flush
    action = a
    p, v = mCarStep(a)			# do the action
    global position, velocity, steps, episodesteps, episodes
    position = p
    velocity = v
    steps += 1
    episodesteps += 1
    r = -1				# get reward (always -1 in MC)
    if mCarGoal(p):			# if we've reached the goal
        sp = 'terminal'		        # set the new state to indicate that
        print str(carbrain.robot.timestamp).replace(".0","") + ", " + str(total_episode_reward)
        total_episode_reward = 0 #KAJ NOTE: added to reset the reward
    else:
        sp = (p, v)			# otherwise new state is position and velocity
    #print "New state is", sp, "reward is", r
    total_episode_reward = total_episode_reward + r
    return sp, r
	
def mCarHeight(position):           #KAJ NOTE: Not used by MC implementation
    return sin(3 * position)
	
def mCarSlope(position):            #KAJ NOTE: Not used by MC implementation
    return cos(3 * position)
    
def mCarStep(a):
    "Applies the action and calculates the new position and velocity"
    global velocity, position
    oldp, oldv = position, velocity
    velocity = carbrain.getVelocity()   #KAJ NOTE: minmax(velocity + (.001 * factor)  + (-.0025 * cos(3 * position)), maxVelocity)
    position = carbrain.getPos()        #KAJ NOTE: original implementation the position was calculated using position += velocity
    thrust = 0                          #KAJ NOTE: thrust is added because it is used by our vehicle
    if a == 0 :                         #KAJ NOTE: this checks if the vehicle is "stuck". If not then we rotate it
        if math.fabs(velocity) < 0.01 : 
            carbrain.goto("rotate")    
            
    if a==0:				        # action is backward thrust
        factor = -1
        thrust = -1
    elif a == 1:				# action is coast
        factor = 0
    else:					# action is forward thrust
        factor = 1
        thrust = 1
    carbrain.robot.move(thrust, 0)
    return position, velocity
	
def mCarGoal (p):
    "Is the car past the goal?"
    return carbrain.isPassedGoal()      #KAJ NOTE: was 'return p >= goalPosition'
	
def mCarInit ():
    "Set car to its initial position for an episode"
    global randomStarts
    if randomStarts:
        position = randomInInterval(minPosition, goalPosition)
        velocity = randomInInterval(- maxVelocity, maxVelocity)
    else:
        carbrain.initPos()              #KAJ NOTE: was 'position = -0.5'
        position = carbrain.getPos()
        velocity = 0.0
    #print "Initial state is", position, velocity
    return position, velocity

def curStats():
    global steps, episodes, episodesteps
    return steps, episodes, episodesteps

def curState():
    global position, velocity, action
    return position, velocity, action	
        
#################################################################################################
# Here begins some tool functions used to output state value data taken from 
#################################################################################################


def calcD(Fs):
    "calculate distances for 3d window"
    global theta, m
    d = -10000.0
    for a in range(m):
        s = 0.0
    for v in Fs[a]:
        s += theta[v]
    if s > d:
        d = s
    return d    

def dataFlush():
    global minPosition, maxPosition, maxVelocity, m, theta, numTilings, resolution
    global posWidth, velWidth, data
    _minPosition = float(minPosition)
    _maxPosition = float(maxPosition)
    res = resolution
    Fs = [[0 for item1 in range(numTilings)] for item2 in range(m)]
    for i in range(res):
        pos = _minPosition + ((_maxPosition - _minPosition) * (float(i) / (res-1)))
        for j in range(res):
            vel = (2.0 * float(maxVelocity) * (float(j) / (res-1))) - float(maxVelocity)
            loadF(Fs, pos, vel)
            if mCarGoal(pos):
                d = 0.0
            else:
                d = calcD(Fs)
            data[i][j] = d
    print str(data)

#################################################################################################
# Here begins the code that is taken from RLToolkit\examples\mountainAgent.py
# Changes to the original code is noted using KAJ NOTE
#################################################################################################

# Mountain car agent -- uses Sarsa learning with replace traces
# (Re)Initialize with mountainEnv.setupAgent()
# Functions for setting agent parameters: setAlpha, setEpsilon
# RL interface function is mountainAgent(s, r=None)
#    e.g. rli = RLinterface(mountainAgent, mountainEnv)


numTilings = 10				# the number of tilings
posWidth = (maxPosition-minPosition)/2 #1.7 / 8			# tile width for position
velWidth = 8 #0.14 / 8			# tile width for velocity
n = 8192				# number of parameters in theta (mem size)
m = 3					# number of actions (back, coast, forward)
epsilon = 0.01				# probability of random action
alpha = 0.5				# step size parameter
lambd = 0.9				# trace-decay parameter
gamma = 1.0				# discount rate
resolution = 30

lasts = None
lasta = 1
lastr = 0
traceH = TraceHolder(n, 0.01, 1000)
F = [[0 for item1 in range(numTilings)] for item2 in range(m)]	# vector of eligibility traces
theta = [0. for item in range(n)]
qValues = [0. for item in range(m)]		    		# array of action values
cTable = CollisionTable(4096, 'safe')  #was 2048

# Mountain car agent -- uses Sarsa learning with replace traces
# (Re)Initialize with mountainEnv.setupAgent()
# Functions for setting agent parameters: setAlpha, setEpsilon
# RL interface function is mountainAgent(s, r=None)
#    e.g. rli = RLinterface(mountainAgent, mountainEnv)

def mountainAgent(s, r=None):
    """Main agent function to send to RLinterface"""
    global lasts, lasta, lastq, traceH, qValues
    if r == None:                       # initialize traces at start of episode
        traceH.decayTraces(0)
        lasts, lastr, lasta = None, 0, 1 
    if s != 'terminal':
        a = choose(s)
        if r !=None:		        # not initial state
            learnSarsa(lasts, lasta, r, s, a)
        lasts, lasta = s, a
        lastq = qValues[a]
        return a
    else:
            return None

def setupAgent():
    "(Re)Initialize agent"
    global traceH, F, qValues, theta, lasts, lasta, lastr, cTable
    lasts, lasta, lastr = None, 1, 0
    traceH = TraceHolder(n, 0.01, 1000)
    F = [[0 for item1 in range(numTilings)] for item2 in range(m)]
    qValues = [0.0 for item in range(m)]
    theta = [0.0 for item in range(n)]
    cTable = CollisionTable(4096)
	
def choose (s):
    "Chooses next action"
    global epsilon, m, qValues
    pos, vel = s					# break state up into state vars
    for a in range(m):
        loadFeatures(F, pos, vel, a)			# compute feature sets for new state with a
        qValues[a] = computeQ(a)			# compute action values
    chooseA = egreedy(epsilon, m, qValues)
    # print "Agent chose action", chooseA, "qValues", qValues
    return chooseA

def learnSarsa(lastS, lastA, r, s, a):
    "Learns using sarsa"
    global lastq, alpha, gamma, lambd, numTilings, traceH, F, m, theta, qValues
    #print "learnSarsa", lastS, lastA, r, s, a
    if lastS != None:
        delta = r - lastq
        delta += gamma * qValues[a]
        amt = delta * (alpha / numTilings)
        for i in traceH.getTraceIndices():
            theta[i] += amt * traceH.getTrace(i)
        traceH.decayTraces(gamma * lambd)
        traceH.replaceTraces(F[a])
        #traceH.accumulateTraces(F[a])
        #alist = range(m)
        #alist.remove(a)
        #traceH.replaceTracesZero(F[a], [F[act] for act in alist])
    lastq = qValues[a]
	
def setAlpha(new):
    "reset alpha - works from other modules"
    global alpha
    alpha = new

def setEpsilon(new):
    "reset epsilon - works from other modules"
    global epsilon
    epsilon = new

def computeQ (a):
    "compute value of action for current F and theta"
    q = 0
    for i in F[a]:
        q += theta[i]
    return q
	
def loadFeatures (F, pos, vel, a):
    "Compute feature sets for action at current state"
    global cTable, posWidth, velWidth
    statevars = [pos / posWidth, vel / velWidth]
    loadtiles(F[a], 0, numTilings, cTable, statevars, [a])

def loadF (F, pos, vel):
    "Compute feature sets for each action at current state"
    global m, cTable, posWidth, velWidth
    statevars = [pos / posWidth, vel / velWidth]
    for a in range(m):
        loadtiles(F[a], 0, numTilings, cTable, statevars, [a])
    
def updateTheta (amt):
    for i in traceH.getTraceIndices():
            theta[i] += amt * traceH.getTrace(i)	

#################################################################################################
# Here begins our integration of the example code
#################################################################################################



from pyrobot.brain import Brain
import random
import time
import datetime
import math
from pyrobot.brain.behaviors import State, FSMBrain

rotdir = -1

class rotate (State):
    def onActivate(self):
        self.state = "rotation"
        
    def step(self):
        global rotdir
        th = prefered_th #math.degrees(math.atan2(self.src[0]-self.trg[0],self.src[1]-self.trg[1]))
        delta_th = th-self.robot.th
        rotdir = -1
        if delta_th > 0 : rotdir = 1
        
        if delta_th < -180 : 
            delta_th = 360 + delta_th
        elif delta_th > 180 : 
            delta_th = delta_th - 360          
        change_th = math.fabs(delta_th)
        velocity = carbrain.getVelocity()
        if velocity < 0 : return None #dont do any rotation while backing up
        if velocity == 0.0 : 
            velocity = 0.000001
            if self.robot.stall : 
                carbrain.initPos()
        if change_th < 10 : 
            self.state = "drive"
            self.goto('learn')
        elif change_th > 45 and math.fabs(velocity) < 1 :
            self.state = "rotation"
            self.robot.move (0, delta_th)
        else :
            self.state = "rotation"
            if math.fabs(velocity) < 1 : rot = rotdir
            else : rot = (delta_th/360.0)
            self.robot.move (1, rot)
			
class learn(State):

    rli = None
    def getRotation(self) :
        a = self.robot.devData["getRotation"]()[0]
        return a
        
    def onActivate(self): # method called when activated or gotoed
        if self.rli == None : 
			setupEnv ()
			self.rli = RLinterface(mountainAgent, mountainEnv)
			self.initPos()
			self.last_time = time.time()
			self.robot.move(0,0)
			self.goto('rotate')
			self.posx = SceneManager.getEntity ("target").getPosition()[0]

    def initPos (self) : 
        self.setPosition (287.43, 0, 209)
        self.robot.move (0,0)
        self.robot.stall = False
        
    def getVelocity (self) :
        b = [self.robot.x, self.robot.y]
        a = self.last_pos
        time_now = self.robot.timestamp
        timedelta = time_now - self.last_time
        distance = math.sqrt (abs(math.pow(a[0]-b[0],2) + math.pow(a[1]-b[1],2)))
        self.last_pos = b
        if distance == 0.0 : return 0.0
        if timedelta == 0.0 : return 0.0
        velocity  = float(distance) / float(timedelta)
        self.last_time = time_now
        return velocity
        
    def getPos (self) : return self.robot.z
    
    def setPosition (self, x,y,z) :
        dic = {"x":x,"y":y,"z":z}
        self.robot.radio[0].callService("request move-to", dic)
        self.last_time = self.robot.timestamp
        self.last_pos = [self.robot.x, self.robot.y]
        
    def step(self) : 
        global rotdir
        self.rli.step ()
        if (abs(prefered_th-self.robot.th)) > 45 : self.goto ("rotate")
    
    def isPassedGoal (self) : return self.robot.x > self.posx
    
def INIT(engine):
    global carbrain
    fsm = FSMBrain( engine=engine )
    carbrain = learn(1)
    # add behaviors, lowest priorities first:
    fsm.add( rotate (0) )
    fsm.add( carbrain )
    return fsm

