"""
Learning on-the-fly to avoid obstacles using neural networks. 
This brain is more or less taken as is from the collection of examples provided by the Pyrobot community. 
It teaches a neural network avoidance by moving the machine in an environment having obstacles and forwarding 
the sensor readings and appropriate behavior into the neural network.
"""
#http://emergent.brynmawr.edu/emergent/Robot_20Learning_20using_20Neural_20Networks

from pyrobot.brain import Brain
from pyrobot.brain.conx import *
from time import *

class NNBrain(Brain):
    def setup(self):
        self.net = Network()
        self.net.addLayers(3,2,2)
        self.net.setEpsilon(0.25)
        self.net.setMomentum(.1)
        self.maxvalue = self.robot.range.getMaxvalue()
        self.counter = 0
        self.doneLearning = 0

    # Scale the range readings to be between 0 and 1.  Also make close 
    # obstacles register high readings, and distance obstacles low readings. 
    def scale(self, val):
        return (1 - (val / self.maxvalue))
    
    # The robot will get translate and rotate values in the range [-0.5,0.5],  
    # but the neural network will generate outputs in the range [0,1].  
    def toNetworkUnits(self, val):
        return (val + 0.5)
    
    def toRobotUnits(self, val):
        return (val - 0.5)
    
    def determineTargets(self, left, front, right):
        if front < 1.0:
            print "front"
            return([0.0, 0.5])
        elif left < 1.0:
            print "left"
            return([0.2, -0.5]) #compared to original we increase 0.0 to 0.2
        elif right < 1.0:
            print "right"
            return([0.2, 0.5])  #compared to original we increase 0.0 to 0.2
        else:
            print "clear"
            return([0.3, 0.0])  #compared to original we reduce 0.5 to 0.3

    def step(self):
        if self.doneLearning:
            self.net.setLearning(0)
        else:
            self.net.setLearning(1)
        # Set inputs 
        left  = min([s.distance() for s in self.robot.range["front-left"]])
        front = min([s.distance() for s in self.robot.range["front"]])
        right = min([s.distance() for s in self.robot.range["front-right"]])
        inputs = map(self.scale, [left, front, right])
        trnTarget,rotTarget = self.determineTargets(left, front, right)
        targets = [self.toNetworkUnits(trnTarget), self.toNetworkUnits(rotTarget)]
        # Learn 
        self.net.step(input=inputs,output=targets) # input and output are the names of the layers
        trnActual = self.toRobotUnits(self.net['output'].activation[0])
        rotActual = self.toRobotUnits(self.net['output'].activation[1])
        # Check if the robot is stuck and give it a kick to unjam it 
        if self.robot.stall:
            print "stuck--reversing"
            self.move(-0.5, 0.0)
            #note: altered compared to original, removed the sleep call
        else:
            if self.doneLearning:
                print "move", trnActual, rotActual
                self.move(trnActual*1.0, rotActual*2.0) #note: altered compared to original
            else:
                print "step", self.counter, "target", trnTarget, rotTarget, \
                    "network", trnActual, rotActual
                self.move(trnTarget*0.5, rotTarget*2.0) #note: altered compared to original
            self.counter += 1


def INIT(engine):
    return NNBrain('NNBrain', engine)