"""
Robot goes forward and then slows and backup up when it detects something 
"""
from pyrobot.geometry import *   # import distance function
from pyrobot.brain.behaviors import State, FSMBrain
from random import *
from pyrobot.brain.td import *
import SceneManager
import gps

dic_globl = {}
dic_globl["target"] = [-465,-1080]
dic_globl["grid_center_x"] = -465
dic_globl["grid_center_y"] = -1080 
dic_globl["target_grid"] = None
dic_globl["grid_width"] = gps.square_width
dic_globl["target_marker"] = None
dic_globl["path"] = None

class drivepath (State):

    steps = 0
    path = None
    dir = 1
    
    def onActivate(self): # method called when activated or gotoed
        self.path = dic_globl["path"]
       
    def step(self):
        print "WALKING PATH @%f %s" % (self.steps, str(self.path)) 
        if len (self.path) == 0 : 
            self.goto('pathlearner')
        elif len(self.path) == self.steps and self.dir == 1 : 
            print "EXCEEEDING PATH SIZE, NEED TO GO BACK NOW"
            self.dir = -1
            self.steps = self.steps + self.dir
        else :
            if self.dir == -1 and self.steps == 0: 
                self.path = []
                self.steps = 0
                self.dir = 1
                return None
            trgpnt = self.path[self.steps]
            trgpnt = ((trgpnt[0]*dic_globl["grid_width"])+dic_globl["grid_center_x"],(trgpnt[1]*dic_globl["grid_width"])+dic_globl["grid_center_y"])
            dic_globl["target"] = trgpnt
            self.goto('drivecar')
            self.steps = self.steps + self.dir

class drivecar (State):
    trg = dic_globl["target"] 
    src = [0,0]
    radius = 10
    
    def collision (self):
        front = min([s.distance() for s in self.robot.range["front"]])
        if front < 1.0 : return True
        return False
    
    def inrange (self):
        """
        Checks if src is within 500 radius of trg
        """
        if self.src[0] < (self.trg[0] + self.radius) :
            if self.src[0] > (self.trg[0] - self.radius) :
                if self.src[1] < (self.trg[1] + self.radius) :                     
                    if self.src[1] > (self.trg[1] - self.radius) : 
                        return True
        return False
    
    def onActivate(self):
        self.src = [0,0]
        self.state = "rotation"
        self.trg = dic_globl["target"]
        dic_globl["target_marker"].setPosition(self.trg[0], 50, self.trg[1])
        dic_globl["target_marker"].setScale(1, 500, 1)

    def step(self):
        self.src[0] = self.robot.x
        self.src[1] = self.robot.y
        self.th = self.robot.th
        self.trg = dic_globl["target"]
        th = math.degrees(math.atan2(self.src[0]-self.trg[0],self.src[1]-self.trg[1]))
        th = th + 180 #turn around because the vehicle is oriented down the negative z axis not the positive
        delta_th = th-self.th
        if delta_th < -180 : delta_th = 360 + delta_th
        elif delta_th > 180 : delta_th = delta_th - 360  
        change_th = abs(delta_th)
        print "TURN: th: %f, delta: %f, change: %f" % (th, th-self.th, change_th)
        if self.collision () :
            print "WE DETECTED A COLLISION !!!!" #currently we do not do anything except notify the user 
        elif self.inrange () :
            print "WE FOUND IT!!!!!!!!!!!!!!!!!!!!!!" + str(self.src)
            self.goto('drivepath')
            self.robot.move (0, 0)
        elif change_th < 10 : 
            self.state = "drive"
            self.robot.move (1, 0)
        elif change_th > 45 :
            self.state = "rotation"
            self.robot.move (0, delta_th)
        else :
            self.state = "rotation"
            self.robot.move (1, delta_th)


class pathlearner(State):

   random_percent = 50
   reset = 0
   # reinforcement value for temporal differencing
   REINFORCEMENT_VALUE = .2
   
   def onActivate (self):
       self.gps.showVisualCells(True, self.robot)
   
   # just calls find_successor
   def find_path( self ):
      loc = self.gps.ask('location')
      # make sure we got a tuple
      if type(loc) != type( () ):
         return "";         
      direction = self.find_successor(loc)
      return direction

   # finds the next step, by random, utility-based, or direct moves
   def find_successor( self, loc ):
      valid_state = 0
      moves = self.get_valid_moves( loc )
      if not moves:
         path = self.gps.ask('path')
         cx, cy = loc
         px, py = path[len(path) - 2]
         if px==cx:
            if py < cy:
               return "up"
            else:
               return "down"
         else:
            if px < cx:
               return "left"
            else:
               return "right"
      else:
         move_type = randrange(0, 100, 1)
         if move_type < self.random_percent:  # chance of random movement
            return moves[ randrange(0,len(moves)) ]
         else:
            return self.util_successor( loc, moves )
      
      
   # returns the next location, based on utility values
   def util_successor( self, loc, valid_moves ):
      # best move will be the move with the highest utility value
      best_move = "none"
      zero_count = 0
      max_util = -999
      for m in valid_moves:
         # get the state
         new_state = self.valid_move( m, loc )
         # if we get a valid state, check to see if its the best one yet
         if new_state != 0:
            util = self.get_util( new_state )
            if util == 0.0:
               zero_count += 1
            if util > max_util:
               max_util = util
               best_move = m
      # make sure we aren't all zero
      if max_util == 0 and zero_count == len(valid_moves):
         return valid_moves[ randrange(0,len(valid_moves)) ]                     
      # make the move
      elif best_move != "none":
         return best_move
      else:
         # should NEVER get here, but just in case!
         print "NO GOOD SUCCESSOR FOUND"
         return valid_moves[ randrange(0,len(valid_moves)) ]            

   def get_valid_moves( self, loc ):
      valid = []
      states = ("up", "down", "left", "right") 
      for s in states:
         if self.valid_move( s, loc ):
            valid.append( s )
      return valid

   #checks to make sure that a move is valid
   def valid_move ( self, move, loc ):
      (locX, locY) = loc
      visited = self.gps.ask('visited')
      if move == "up":
         new_state = (locX  , locY - 1)
      elif move == "right":
         new_state = (locX+1, locY)
      elif move == "down":
         new_state  = (locX  , locY + 1)
      elif move == "left":
         new_state = (locX-1, locY)
      # if the new state is onto an obstacle, return 0
      if new_state in self.obstacles:
         return 0
      # if the new state is off the map, return 0
      elif min(new_state) < 0 or max(new_state) > 14:
         return 0
      elif new_state in visited:
         return 0
      else:
         return new_state
      
   # returns the utility value for a given location
   def get_util( self, loc ):
      #utils = self.gps.ask('util')
      (locX,locY) = loc
      if locX < 0 or locX > 14 or locY < 0 or locY > 14:
         return -99
      else:
         return self.td.get_utility(locX, locY)
     
   # gets called before first run
   def setup(self, **args):
      self.gps = self.getDevice("gps")
      # set reinforcement value
      alpha = self.REINFORCEMENT_VALUE
      # get td instance
      self.td = Temporal_Difference( 15, 15, self.gps.ask('goal'), self.gps.ask('pits'), alpha )
      self.obstacles = self.gps.ask("obstacles")
      # initialize the number of iterations to zero
      self.count = 0
      self.gps.showVisualCells(True, self.robot)

   # run once every iteration
   def step(self):
      if self.reset == 1:
         self.gps.tell("reset")
         self.gps.tell("start")
         self.setup()
         self.reset = 0
      # if the goal has not been reached, continue to find a path
      if self.gps.ask('complete') == 0:
         direction = self.find_path()
         self.gps.tell(direction)
         self.robot.move( 0, 0 ) #self.robot.move( direction )
      # if the goal has been reached, display the current path
      else:
         path = self.gps.ask('path')
         dic_globl["path"] = []
         for p in path :
             dic_globl["path"].append(p)
         self.count += 1
         # show the path
         print "PATH: ", path
         # adjust the utility values of the map
         print "Computing TDs: #", self.count, " length: ", len(path)
         self.td.do_td( path )
         # start a new run
         self.gps.tell('start')
         for e in path:
            # sends the robot a formatted string including the x,y coordinates, and a color
            self.gps.tell( self.td.get_utility_color(e[0], e[1]));
         alpha = self.td.get_alpha()
         if( (self.count % 15) == 0 and alpha > 0 ):
             self.td.set_alpha( alpha - 0.01 )
             self.random_percent -= 5
         if self.gps.ask('goal') in dic_globl["path"] :
             #go along the decided path
             self.gps.showVisualCells(True, self.robot)
             self.goto('drivepath')        

def INIT(engine):
    brain = FSMBrain(engine=engine)
    try :
        dic_globl["target_marker"] = SceneManager.addEntity("debug-target", "cube.1m.mesh")
    except Exception : # we could not create it beacuse it probably exists in our backend. Lets just get it
        dic_globl["target_marker"] = SceneManager.getEntity("debug-target") 
    # add a few states:
    brain.add(drivecar(0))
    brain.add(drivepath(0))
    brain.add(pathlearner(1)) # 1 makes it active
    return brain