"""
	Author: Kjartan Akil Jonsson, aGame Company
	Brain script forwarding key actions to rover. 
	This script allows users to take manual control of the vehicle.
"""
from pyrobot.brain import Brain
import SystemHandler

class UserBrain(Brain):
    
    key_map = {"LEFT":"ROTATE_LEFT", "RIGHT":"ROTATE_RIGHT", "UP":"MOVE_FORWARD", "DOWN":"MOVE_BACKWARD"} #default keys
    last = {"MOVE":0, "ROTATE":0}
    gps = None
    
    def __init__(self, engine): 
        Brain.__init__(self, 'UserBrain', engine)
        #register the keyboard keys
        for k in self.key_map.keys() : SystemHandler.keyEnabled(k, True)
        if self.hasA ("gps") : self.gps = self.getDevice("gps")

    def step(self): pass

    def activated (self): 
        """called each time we activate the brain"""
        self.robot.move(0, 0)    

    def deactivated (self): 
        """called each time we deactivate the brain"""
        self.robot.move(0, 0)    
    
    def callback (self, source, event, message):
        self.__setDeviceValue(message, event)
        if self.gps <> None : 
            self.gps.updatePosition()
        
    def __setDeviceValue (self, action, enable) :
        action = self.key_map[action]
        if action == "ROTATE_LEFT" : self.last["ROTATE"] = 1*int(enable)
        elif action == "ROTATE_RIGHT" : self.last["ROTATE"] = -1*int(enable)
        elif action == "MOVE_FORWARD" : self.last["MOVE"] = 1*int(enable)
        elif action == "MOVE_BACKWARD" : self.last["MOVE"] = -1*int(enable)
        self.robot.move(self.last["MOVE"], self.last["ROTATE"])
            
def INIT(engine):
    brain = UserBrain(engine=engine)
    return brain