"""
VehicleManager handles everything related to an vehicle. It creates, destroys (static and dynamic)
instances of the vehicle. It manages changing parts and the clients vehicle parts that he currently
has available to him.
This class does not handle:
- purchasing
- exchangin parts with other players
- sponsoring system
- vehicle parts (resource) information
"""
__author__ = "Kjartan A Jonsson"
__version__ = "$Revision: 1.2 $"
__copyright__ = "Copyright (c) 2007 aGameCompany"
__license__ = "aGameCompany"

import xmlrpclib
import settings

__rpc = xmlrpclib.ServerProxy( settings.SERV_STATE_MACHINE, transport=settings.proxy )

class DeviceManagerError ( Exception ): pass

def __loadIRFromServer( playerName, irname ):
    """
    Will retreave the player 'playerName' IR information. Used to update internal state and data.
    """
    global __rpc, __vehicleDic
    #resnotused = {'Physical Eagle': {'wheel_rear': {'mesh_left': ' ', 'mass_inv': 1.0, 'mass': 0.0, 'description': 'This is a beadlook wheel used for offroad driving', 'mesh_right': ' '}, 'engine': {'description': ' ', 'power': 1.0, 'increase': 0.69999999999999996, 'mesh': ' ', 'decrease': 0.9, 'power_step3': 1.0, 'power_step2': 1.0, 'power_step1': 1.0 'power_step0': 0.0, 'power_step4': 1.0}, 'shockabsorber_front': {'damper': 50.0, 'pivot': '0,0,0.5', 'mesh': 'Suspension-D010.mesh', 'target': 0.75, 'description': ' '}, 'brake_rear': {'mesh': ' ', 'force': 200.0, 'description': ' '}, 'wheel_front': {'mesh_left': ' ', 'mass_inv': 1.0, 'mass': 0.0, 'description': 'This is a beadlook wheel used for offroad driving', 'mesh_right': ' '}, 'suspension_front': {'spring': 750.0, 'mesh': 'S-R-KJ-100.mesh', 'description': 'Simple suspension having stiffness of 750', 'pivot': '0,0,0.5'}, 'axle_rear': {'bracket_suspension_r': '-0.693967,0.117106,-0.0206645', 'description': ' ', 'bracket_controlarm_r': '-0.64502,0.00528836,-0.157996', 'bracket_driveshaft': '-0.328897,0.00639343,-0.157996', 'resource_id': 20, 'bracket_suspension_l': '0.693967,0.117106,-0.0206645', 'mesh': 'Axle-D001-Rear.mesh', 'bracket_suspension_type_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' ', 'resource_id': 9}, 'bracket_controlarm_type_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' ', 'resource_id': 9}, 'pivot': '0,0,0', 'bracket_controlarm_l': '0.64502,0.00528836,-0.157996', 'bracket_shockabsorber_l': '0.485944,0.0409608,0.0579794', 'bracket_shockabsorber_type_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' ', 'resource_id': 9}, 'bracket_shockabsorber_r': '-0.485944,0.0409608,0.0579794'}, 'chassi': {'bracket_suspension_fl': '0.674157, -0.172985, 1.90496', 'bracket_shockabsorber_rl': '0.799057,-0.16716,-1.60724', 'bracket_controlarm_type_f_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' '}, 'bracket_controlarm_fl': '0.669678,-0.0843039,-0.175895', 'axis_position_front': 2.2999999999999998, 'bracket_controlarm_fr': '-0.669678,-0.0843039,-0.175895', 'bracket_driveshaft_type_r_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' '}, 'bracket_shockabsorber_rr': '-0.799057,-0.16716,-1.60724', 'bracket_suspension_fr': '-0.674157, -0.172985, 1.90496', 'bracket_suspension_type_f_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' '}, 'bracket_shockabsorber_fl': '0.674157, -0.172985, 1.90496', 'bracket_driveshaft_type_f_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' '}, 'bracket_suspension_rl': '0.799057,-0.16716,-1.60724', 'bracket_shockabsorber_type_r_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' '}, 'bracket_suspension_rr': '-0.799057,-0.16716,-1.60724', 'sponsor_material': 'EagleSponsorMtr', 'bracket_shockabsorber_fr': '-0.674158,-0.172985,1.90496', 'sponsor_stickers': '<?xml version="1.0" encoding="UTF-8"?>\n\n<Imageset Name="eagle" Imagefile="..\\..\\data\\eagle.jpg" >\n    <Image Name="{*group*:*A*,*id*:*A1*,*rotate*:*Up*}" XPos="686" YPos="562" Width="114" Height="102" />\n    <Image Name="{*group*:*B*,*id*:*B1*,*rotate*:*Up*}" XPos="318" YPos="67" Width="176" Height="134" />\n    <Image Name="{*group*:*B*,*id*:*B2*,*rotate*:*Up*}" XPos="320" YPos="251" Width="169" Height="135" />\n    <Image Name="{*group*:*D*,*id*:*D4*,*rotate*:*Up*}" XPos="717" YPos="224" Width="57" Height="20" />\n    <Image Name="{*group*:*E*,*id*:*E1*,*rotate*:*Up*}" XPos="472" YPos="565" Width="53" Height="53" />\n    <Image Name="{*group*:*E*,*id*:*E2*,*rotate*:*Up*}" XPos="953" YPos="565" Width="51" Height="51" />\n    <Image Name="{*group*:*G*,*id*:*G1*,*rotate*:*Up*}" XPos="926" YPos="337" Width="50" Height="50" />\n    <Image Name="{*group*:*C*,*id*:*C2*,*rotate*:*Down*}" XPos="692" YPos="351" Width="109" Height="52" />\n    <Image Name="{*group*:*D*,*id*:*D1*,*rotate*:*Down*}" XPos="965" YPos="119" Width="51" Height="51" />\n    <Image Name="{*group*:*D*,*id*:*D2*,*rotate*:*Down*}" XPos="613" YPos="118" Width="51" Height="51" />\n    <Image Name="{*group*:*F*,*id*:*F1*,*rotate*:*Down*}" XPos="788" YPos="91" Width="54" Height="22" />\n    <Image Name="{*group*:*D*,*id*:*D3*,*rotate*:*Down*,*inv*:*True*}" XPos="805" YPos="31" Width="115" Height="51" />\n</Imageset>\n', 'sponsor_texture': 'EagleSponsorLayer.tga', 'mesh': 'Chassi-Eagle.mesh', 'bracket_controlarm_type_r_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' '}, 'axis_position_rear': -1.6000000000000001, 'bracket_shockabsorber_type_f_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' '}, 'bracket_suspension_type_r_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' '}, 'bracket_controlarm_rl': '0.67166,-0.896534,-0.725403', 'name': 'Eagle', 'bracket_driveshaft_r': '-0.05189885,-0.841699,-0.819795', 'mass': 1000.0, 'bracket_driveshaft_f': '-0.32117,0.841699,0.620208', 'bracket_controlarm_rr': '-0.67166,-0.896534,-0.725403'}, 'controlarm_rear': {'pivot': '0,0,0.5', 'travel': 0.10000000000000001, 'mesh': 'Controlarm-A001-Front.mesh', 'description': ' '}, 'driveshaft_rear': {'pivot': '0,0,0.5', 'mesh': 'DriveShaft.mesh', 'description': ' '}, 'steering': {'increase': 3.6000000000000001, 'mesh': '  ', 'decrease': 3.6000000000000001, 'description': ' ', 'ownaxis':30}, 'steering_arm': {'mesh': ' ', 'description': 'Simple basic steering arm ', 'radian': 0.59999999999999998}, 'controlarm_front': {'pivot': '0,0,0.5', 'travel': 0.10000000000000001, 'mesh': 'Controlarm-A001-Front.mesh', 'description': ' '}, 'axle_front': {'bracket_suspension_r': '-0.693967,0.117106,-0.0206645', 'description': ' ', 'bracket_controlarm_r': '-0.64502,0.00528836,-0.157996', 'bracket_driveshaft': '-0.328897,0.00639343,-0.157996', 'resource_id': 19, 'bracket_suspension_l': '0.693967,0.117106,-0.0206645', 'mesh': 'Axle-D001-Front.mesh', 'bracket_suspension_type_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' ', 'resource_id': 9}, 'bracket_controlarm_type_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' ', 'resource_id': 9}, 'pivot': '0,0,0', 'bracket_controlarm_l': '0.64502,0.00528836,-0.157996', 'bracket_shockabsorber_l': '0.485944,0.0409608,0.0579794', 'bracket_shockabsorber_type_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' ', 'resource_id': 9}, 'bracket_shockabsorber_r': '-0.485944,0.0409608,0.0579794'}, 'suspension_rear': {'spring': 750.0, 'mesh': 'S-R-KJ-100.mesh', 'description': 'Simple suspension having stiffness of 750', 'pivot': '0,0,0.5'}, 'tire_front': {'longitudal_asymptote_slip': 2.0, 'lateral_asymptote_value': 0.025000000000000001, 'lateral_extreme_value': 0.025000000000000001, 'description': 'Is an Wheel Diggers type of wheel that is used for off road tracking', 'longitudal_extreme_value': 0.025000000000000001, 'mesh_left': 'WheelHookers01_Left.mesh', 'longitudal_stiffness_factor': 100000.0, 'lateral_extreme_slip': 1.0, 'mesh_right': 'WheelHookers01_Right.mesh', 'lateral_asymptote_slip': 2.0, 'longitudal_asymptote_value': 0.025000000000000001, 'lateral_stiffness_factor': 100000.0, 'longitudal_extreme_slip': 1.0}, 'brake_front': {'mesh': ' ', 'force': 44.0, 'description': ' '}, 'shockabsorber_rear': {'damper': 50.0, 'pivot': '0,0,0.5', 'mesh': 'Suspension-D010.mesh', 'target': 0.75, 'description': ' '}, 'driveshaft_front': {'pivot': '0,0,0.5', 'mesh': 'DriveShaft.mesh', 'description': ' '}, 'tire_rear': {'longitudal_asymptote_slip': 2.0, 'lateral_asymptote_value': 0.025000000000000001, 'lateral_extreme_value': 0.025000000000000001, 'description': 'Is an Wheel Diggers type of wheel that is used for off road tracking', 'longitudal_extreme_value': 0.025000000000000001, 'mesh_left': 'WheelHookers01_Left.mesh', 'longitudal_stiffness_factor': 100000.0, 'lateral_extreme_slip': 1.0, 'mesh_right': 'WheelHookers01_Right.mesh', 'lateral_asymptote_slip': 2.0, 'longitudal_asymptote_value': 0.025000000000000001, 'lateral_stiffness_factor': 100000.0, 'longitudal_extreme_slip': 1.0}}} 
    #res = {'Physical Eagle': {'wheel_rear': {'mesh_left': ' ', 'mass_inv': 1.0, 'mass': 0.0, 'description': 'This is a beadlook wheel used for offroad driving', 'mesh_right': ' '}, 'engine': {'description': ' ', 'power': 6.0, 'increase': 0.69999999999999996, 'mesh': ' ', 'decrease': 0.90000000000000002, 'power_step3': 0.1, 'power_step2': 0.1, 'power_step1': 0.1, 'power_step0': 0.0, 'power_step4': 0.1}, 'shockabsorber_front': {'damper': 50.0, 'pivot': '0,0,0.5', 'mesh': 'Suspension-D010.mesh', 'target': 0.75, 'description': ' '}, 'brake_rear': {'mesh': ' ', 'force': 200.0, 'description': ' '}, 'wheel_front': {'mesh_left': ' ', 'mass_inv': 1.0, 'mass': 0.0, 'description': 'This is a beadlook wheel used for offroad driving', 'mesh_right': ' '}, 'suspension_front': {'spring': 750.0, 'mesh': 'S-R-KJ-100.mesh', 'description': 'Simple suspension having stiffness of 750', 'pivot': '0,0,0.5'}, 'axle_rear': {'bracket_suspension_r': '-0.693967,0.117106,-0.0206645', 'description': ' ', 'bracket_controlarm_r': '-0.64502,0.00528836,-0.157996', 'bracket_driveshaft': '-0.328897,0.00639343,-0.157996', 'resource_id': 20, 'bracket_suspension_l': '0.693967,0.117106,-0.0206645', 'mesh': 'Axle-D001-Rear.mesh', 'bracket_suspension_type_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' ', 'resource_id': 9}, 'bracket_controlarm_type_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' ', 'resource_id': 9}, 'pivot': '0,0,0', 'bracket_controlarm_l': '0.64502,0.00528836,-0.157996', 'bracket_shockabsorber_l': '0.485944,0.0409608,0.0579794', 'bracket_shockabsorber_type_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' ', 'resource_id': 9}, 'bracket_shockabsorber_r': '-0.485944,0.0409608,0.0579794'}, 'chassi': {'bracket_suspension_fl': '0.674157, -0.172985, 1.90496', 'bracket_shockabsorber_rl': '0.799057,-0.16716,-1.60724', 'bracket_controlarm_type_f_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' '}, 'bracket_controlarm_fl': '0.669678,-0.0843039,-0.175895', 'axis_position_front': 2.2999999999999998, 'bracket_controlarm_fr': '-0.669678,-0.0843039,-0.175895', 'bracket_driveshaft_type_r_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' '}, 'bracket_shockabsorber_rr': '-0.799057,-0.16716,-1.60724', 'bracket_suspension_fr': '-0.674157, -0.172985, 1.90496', 'bracket_suspension_type_f_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' '}, 'bracket_shockabsorber_fl': '0.674157, -0.172985, 1.90496', 'bracket_driveshaft_type_f_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' '}, 'bracket_suspension_rl': '0.799057,-0.16716,-1.60724', 'bracket_shockabsorber_type_r_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' '}, 'bracket_suspension_rr': '-0.799057,-0.16716,-1.60724', 'sponsor_material': 'EagleSponsorMtr', 'bracket_shockabsorber_fr': '-0.674158,-0.172985,1.90496', 'sponsor_stickers': '<?xml version="1.0" encoding="UTF-8"?>\n\n<Imageset Name="eagle" Imagefile="..\\..\\data\\eagle.jpg" >\n    <Image Name="{*group*:*A*,*id*:*A1*,*rotate*:*Up*}" XPos="686" YPos="562" Width="114" Height="102" />\n    <Image Name="{*group*:*B*,*id*:*B1*,*rotate*:*Up*}" XPos="318" YPos="67" Width="176" Height="134" />\n    <Image Name="{*group*:*B*,*id*:*B2*,*rotate*:*Up*}" XPos="320" YPos="251" Width="169" Height="135" />\n    <Image Name="{*group*:*D*,*id*:*D4*,*rotate*:*Up*}" XPos="717" YPos="224" Width="57" Height="20" />\n    <Image Name="{*group*:*E*,*id*:*E1*,*rotate*:*Up*}" XPos="472" YPos="565" Width="53" Height="53" />\n    <Image Name="{*group*:*E*,*id*:*E2*,*rotate*:*Up*}" XPos="953" YPos="565" Width="51" Height="51" />\n    <Image Name="{*group*:*G*,*id*:*G1*,*rotate*:*Up*}" XPos="926" YPos="337" Width="50" Height="50" />\n    <Image Name="{*group*:*C*,*id*:*C2*,*rotate*:*Down*}" XPos="692" YPos="351" Width="109" Height="52" />\n    <Image Name="{*group*:*D*,*id*:*D1*,*rotate*:*Down*}" XPos="965" YPos="119" Width="51" Height="51" />\n    <Image Name="{*group*:*D*,*id*:*D2*,*rotate*:*Down*}" XPos="613" YPos="118" Width="51" Height="51" />\n    <Image Name="{*group*:*F*,*id*:*F1*,*rotate*:*Down*}" XPos="788" YPos="91" Width="54" Height="22" />\n    <Image Name="{*group*:*D*,*id*:*D3*,*rotate*:*Down*,*inv*:*True*}" XPos="805" YPos="31" Width="115" Height="51" />\n</Imageset>\n', 'sponsor_texture': 'EagleSponsorLayer.tga', 'mesh': 'Chassi-Eagle.mesh', 'bracket_controlarm_type_r_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' '}, 'axis_position_rear': -1.6000000000000001, 'bracket_shockabsorber_type_f_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' '}, 'bracket_suspension_type_r_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' '}, 'bracket_controlarm_rl': '0.67166,-0.896534,-0.725403', 'name': 'Eagle', 'bracket_driveshaft_r': '-0.05189885,-0.841699,-0.819795', 'mass': 1000.0, 'bracket_driveshaft_f': '-0.32117,0.841699,0.620208', 'bracket_controlarm_rr': '-0.67166,-0.896534,-0.725403'}, 'controlarm_rear': {'pivot': '0,0,0.5', 'travel': 0.10000000000000001, 'mesh': 'Controlarm-A001-Front.mesh', 'description': ' '}, 'driveshaft_rear': {'pivot': '0,0,0.5', 'mesh': 'DriveShaft.mesh', 'description': ' '}, 'steering': {'increase': 3.6000000000000001, 'mesh': '  ', 'decrease': 3.6000000000000001, 'description': ' ', 'rotateownaxisonstop':10}, 'steering_arm': {'mesh': ' ', 'description': 'Simple basic steering arm ', 'radian': 0.59999999999999998}, 'controlarm_front': {'pivot': '0,0,0.5', 'travel': 0.10000000000000001, 'mesh': 'Controlarm-A001-Front.mesh', 'description': ' '}, 'axle_front': {'bracket_suspension_r': '-0.693967,0.117106,-0.0206645', 'description': ' ', 'bracket_controlarm_r': '-0.64502,0.00528836,-0.157996', 'bracket_driveshaft': '-0.328897,0.00639343,-0.157996', 'resource_id': 19, 'bracket_suspension_l': '0.693967,0.117106,-0.0206645', 'mesh': 'Axle-D001-Front.mesh', 'bracket_suspension_type_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' ', 'resource_id': 9}, 'bracket_controlarm_type_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' ', 'resource_id': 9}, 'pivot': '0,0,0', 'bracket_controlarm_l': '0.64502,0.00528836,-0.157996', 'bracket_shockabsorber_l': '0.485944,0.0409608,0.0579794', 'bracket_shockabsorber_type_id': {'mesh_r': 'Bolts-CA-eagle.mesh', 'mesh_l': 'Bolts-CA-eagle.mesh', 'description': ' ', 'resource_id': 9}, 'bracket_shockabsorber_r': '-0.485944,0.0409608,0.0579794'}, 'suspension_rear': {'spring': 750.0, 'mesh': 'S-R-KJ-100.mesh', 'description': 'Simple suspension having stiffness of 750', 'pivot': '0,0,0.5'}, 'tire_front': {'longitudal_asymptote_slip': 2.0, 'lateral_asymptote_value': 0.025000000000000001, 'lateral_extreme_value': 0.025000000000000001, 'description': 'Is an Wheel Diggers type of wheel that is used for off road tracking', 'longitudal_extreme_value': 0.025000000000000001, 'mesh_left': 'WheelHookers01_Left.mesh', 'longitudal_stiffness_factor': 100000.0, 'lateral_extreme_slip': 1.0, 'mesh_right': 'WheelHookers01_Right.mesh', 'lateral_asymptote_slip': 2.0, 'longitudal_asymptote_value': 0.025000000000000001, 'lateral_stiffness_factor': 100000.0, 'longitudal_extreme_slip': 1.0}, 'brake_front': {'mesh': ' ', 'force': 20.0, 'description': ' '}, 'shockabsorber_rear': {'damper': 50.0, 'pivot': '0,0,0.5', 'mesh': 'Suspension-D010.mesh', 'target': 0.75, 'description': ' '}, 'driveshaft_front': {'pivot': '0,0,0.5', 'mesh': 'DriveShaft.mesh', 'description': ' '}, 'tire_rear': {'longitudal_asymptote_slip': 2.0, 'lateral_asymptote_value': 0.025000000000000001, 'lateral_extreme_value': 0.025000000000000001, 'description': 'Is an Wheel Diggers type of wheel that is used for off road tracking', 'longitudal_extreme_value': 0.025000000000000001, 'mesh_left': 'WheelHookers01_Left.mesh', 'longitudal_stiffness_factor': 100000.0, 'lateral_extreme_slip': 1.0, 'mesh_right': 'WheelHookers01_Right.mesh', 'lateral_asymptote_slip': 2.0, 'longitudal_asymptote_value': 0.025000000000000001, 'lateral_stiffness_factor': 100000.0, 'longitudal_extreme_slip': 1.0}}}
    res = __rpc.getIR( playerName, irname )
    if res == None : raise DeviceManagerError, "Request was bad, got None from server. Probably because user '" + playerName + "' is not valid"
    return res

def getIR ( playerName, irname ):
    """
    Returns a dynamic vehicle beloning to user having alias 'playerName' and vehicle id is 'vehicleName'. 
    By a dynamic vehicle we mean a vehicle that is interactive and drivable. This is the drivable car.
    Raises VehicleManagerError if playerName does not exist or the player does not have vehicle 'vehicleName'
    """
    if playerName == None : raise DeviceManagerError, "Player name was None"
    if irname == None : raise DeviceManagerError, "IR name was None"
    ir = __loadIRFromServer ( playerName, irname)
    return ir

#############################################################################
#
#     Testing device manager
#
#############################################################################    
import unittest 
class TestDeviceManager ( unittest.TestCase ) :

    def setTestIR ( self ) : 
        pass

    def testIR ( self ):
        """DeviceManager and retraving IR from server"""
        self.assertRaises( DeviceManagerError, getIR, "unittest", "nosuchvehicle" )
        self.assertRaises( DeviceManagerError, getIR, "nosuchuser", "8 IR Sensor" )
        print str(getIR("unittest", "8 IR Sensor" ))
